//////////////////////////////////////////////////
//  Equation d'Euler compressible 2D	C
//////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//	Maxime Isnard & Sophie Hecht & Lilian BESSON
//		 ENS de Cachan & LSV & CMLA - Avril-Mai 2012
//			CMLA
////////////////////////////////////////////////////////////////


	///////////////////////////////
	// # include <bibliotheque.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <stdio.h>		// librairie standard d'input & d'output
#include <stdlib.h>		// librairie standard de C

#include <math.h>		// librairie pour utiliser les fonctions mathematiques.
#include <string.h>		// librairie pour utiliser les fonctions "plus elaborees" pour la gestion de chaines de caracteres
#include <time.h>		// librairie pour utiliser des fonctions de gestions du temps.

#include <assert.h>		
// librairie qui fournit assert(<booleen>) pour faire du debug

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pthread.h>

	///////////////////////////////////
	// #define <macro>

		// fonctions macro utiles
#define min(a,b) ((a)<=(b) ? (a) : (b))

/* Coefficient CFL pour notre pas de temps. TODO : changer un peu pour voir ! */
/* A CHANGER ICI */
#define CFL 0.5
#define PICK_TOL              10.
#define PICK_BUFFER_SIZE      256
unsigned int      PickBuffer[PICK_BUFFER_SIZE]; /* picking buffer */

//#define GET_DEBUG__VOIR_VTK ( 0 )
	/////////////////////////////////////////////////
	// 		Calcul matriciel
	/////////////////////////////////////////////////

// Pour calculer l'indice d'une matrice.
//  Pour les conventions d'indicages, voir la doc.
int I2D(int i, int j, int taille){
	return (i*taille + j);
}

//////////////////////
// Produit Matriciel
//
// direction est une matrice *carree* de taille taille * taille
// A une matrice carree de taille taille * taille
void produitmatriciel(double* A, double* B, int taille, double* direction){

	int kloc, iloc, jloc;

	for(iloc = 0; iloc < taille; iloc++){

		for(jloc = 0; jloc < taille; jloc++){

			direction[I2D(iloc ,jloc ,taille)] = 0.0;

			for(kloc = 0; kloc < taille; kloc++){

				direction[I2D(iloc ,jloc ,taille)] += (A[I2D(iloc ,kloc ,taille)])*(B[I2D(kloc ,jloc ,taille)]);

			}
		}
	}
}


////////////////////
// Somme Matricielle
//
//
// direction est une matrice *carree* de taille taille * taille
// A une matrice carree de taille taille * taille
void sommematriciel(double* A, double* B, int taille, double* direction){

	int iloc, jloc;

	for(iloc = 0; iloc < taille; iloc++){

		for(jloc = 0; jloc < taille; jloc++){

			direction[I2D(iloc ,jloc ,taille)] = (A[I2D(iloc ,jloc ,taille)])*(B[I2D(iloc ,jloc ,taille)]);

		}
	}
}

/////////////////////////////
// Produit Matrice * Vecteur
//
//
// direction est un vecteur de taille taille
// A une matrice *carree* de taille taille * taille
void produitvecteur(double* A, double* X, int taille, double* direction){

	int iloc, jloc;

	for(iloc = 0; iloc < taille; iloc++){

		direction[iloc] = 0.0;

		for(jloc = 0; jloc < taille; jloc++){

			direction[iloc] += (A[I2D(iloc ,jloc ,taille)])*(X[jloc]);

		}
	}
}


/////////////////////////////
// Test I = I_4x4
//
void test_identite_4_4(int debugint, double* matriceI, int it, int i, int j){
	int iloc, jloc;
	int result = 0;

	for(iloc = 0; iloc < 4; iloc++){

		for(jloc = 0; jloc < 4; jloc++){

			result += ((iloc == jloc) == (matriceI[I2D(iloc ,jloc ,4)]));
		if(debugint > 3)	printf("    R*L [%d,%d] = %f\n", iloc, jloc, (matriceI[I2D(iloc ,jloc ,4)]));

		}
		if(debugint > 3)	printf("\n");
	}
	if (result != 16){
		if(debugint > 3){
			fprintf(stderr, "Le calcul de R * L ne donne pas I_4x4 : i=%d\tj=%d et\tit=%d\n", i, j, it);
			fflush(stderr);
		}
	}
	else {if(debugint > 3) printf("R * L donne I_4x4 : i=%d\tj=%d et\tit=%d\n", i, j, it);}
}


	/////////////////////////////////////////////////
	// 	FIN	Calcul matriciel
	/////////////////////////////////////////////////

// Petite fonction pour mettre un point d'arrêt dans le programme
void debug(int debugint) {
		printf("Point de controle... ? (-> Enter) ");
		if(debugint > 3)	getchar();
		printf(" :) !\n");
}
// Petite fonction pour mettre un point d'arrêt dans le programme
double debugm(int debugint, const char* message) {
		if(debugint > 3){
			printf("**************************************************************\n");
			printf("%s ? (-> Enter) ", message);
			getchar();
		}
/*		printf(" :) !\n");*/
}


	/////////////////////////////////////////
	//	Prototypes de fonctions

	// Pour calculer le maximum d'un vecteur de flottans doubles precisions.
double maxu(double* u,int N);

	// deux formules differentes pour calculer le pas de temps
double pas_de_temps(double* u,double* v,double* uu, double* vv,int N, int M, double hx,double hy);
double pas_de_temps2(double* u, double* c, double* v, int N, int M, double hx, double hy, int it);
//double pas_de_temps2(double* u, double* c, double* v, int N, int M, double hx, double hy);


	// Fonction signe mathematique (usuelle)
double sgn(double x);


////////////////////////////////////////
// Fonction norme
double norme(double x, double y)
{
	return sqrt(x*x + y*y);
}


////////////////////////////////////////
// Fonction carré
double carre(double x)
{
	return (x*x);
}


////////////////////////////////////////
// Fonction norme au carré
double norme_carre(double x, double y)
{
	return (x*x + y*y);
}

	// Macro pour ecrire nos donnees dans un fichier .vtk
void WriteVtk(char *nom, double *f, int N, float h, char* nom_variable);
void WriteVtk2D(char *nom, double *f, int N, int M, float hx, float hy, char* nom_variable, int debugint);



////////////////////////////////////////////////////////////////
//	Fonctions auxiliaires CPU


////////////////////////////////////////
// Pour calculer le maximum du vecteur u
double maxu2D(double* u,int N, int M)
{
	double m = u[0];
	int i=0;
	int j=0;
	for (i=0; i<M; i++){
		for (j=0; j<N; j++){
			if (u[i*N+j]> m)
			{
				m = u[i*N+j];
			}
		}
	}
	return m;
}



////////////////////////////////////////
// Fonction signe
double sgn(double x)
{
	if (x<0)
	{
		return -1.;
	}
    
	else if (x>0)
	{
		return 1.;
	}
	else
	{
		return 0.0;
	}
}

///////////////////////////////////////
// Actualise le pas de temps.
// Selon la formule dt = CFL * hx * hy * MAX ( Max ( |u| + v), Max( |uu| + vv))
//	N est la taille totale de u et v,
//	M est la taille totale de uu et vv,
double pas_de_temps(double* u, double* v, double* uu, double* vv, int N, int M, double hx, double hy){
	// 0.9 * h / max(abs(ubar)+cbar);
	double maxcalc, pasdetemps;
	int indice;
	
	maxcalc = 0.0;
	pasdetemps = 0.0001;
	
	for (indice=0; indice<N; indice++) {
		if (maxcalc < (fabs(u[indice]) + v[indice] ) ) {maxcalc = (fabs(u[indice]) + v[indice] );}
	}
	for (indice=0; indice<M; indice++) {
		if (maxcalc < (fabs(uu[indice]) + vv[indice] ) ) {maxcalc = (fabs(uu[indice]) + vv[indice] );}
	}
	if (maxcalc != 0.0){pasdetemps = (CFL * hx*hy) / maxcalc;}	// on se protège.
	// if (maxcalc != 0.0){pasdetemps = 0.0001;}
	return (pasdetemps);
}


///////////////////////////////////////
// Actualise le pas de temps.
// Selon la formule dt = CFL * min(hx , hy) * MAX ( |w| + c)
//	w = (u, v)
//	N * M est la taille de u,c,v,
double pas_de_temps2(double* u, double* c, double* v, int N, int M, double hx, double hy, int it){
	// 0.9 * h / max(abs(ubar)+cbar);
	double maxcalc, pasdetemps;
	int indice;
	
	maxcalc = 0.0;
	pasdetemps = 0.00001;
	
	for (indice=0; indice<N*M; indice++) {
		if (maxcalc < ((norme(u[indice], v[indice])) + c[indice])) {
			maxcalc = ((norme(u[indice], v[indice])) + c[indice]);
		}
	}
	if (maxcalc != 0.0){pasdetemps = (CFL * (min(hx,hy))) / maxcalc;}	// on se protège.
	else{
        /*	if ((maxcalc == 0.0)||(maxcalc == nan)||(maxcalc == -nan)){*/
		fprintf(stderr, "Pas de temps foireux ! it=%d\n", it);
		fflush(stderr);
		exit(2);
	}
	//if (maxcalc != 0.0){pasdetemps = 0.01*(CFL * (min(hx,hy))) / maxcalc;}	// on se protège.
	// if (maxcalc != 0.0){pasdetemps = 0.0001;}
	return (pasdetemps);
}


///////////////////////////////////////
// Fonction openGL

typedef double R; 
typedef int  bool; 
R pi=M_PI; 
R max(R x,R y){ return x<y ? x: y;}
//R min(R x,R y){ return x<y ? y: x;}

 

/////////////////////////////////////////
//
//	Fonction main_calcul
//
/////////////////////////////////////////

/// Quelques variable global

int N=100;
int M=100;
int fin=0;  
int choix=2;

double * U=0;
double	*rho=0;
double	*p=0;



void * main_calcul (void * data)
{
    
	
	printf("\nInitialisation\n\tSimulation numerique des equations d'Euler compressible, en 2D, en C ...\n");
    
    /*debug();*/
	// variables sur le CPU
	///////////////////////
	printf(" Declarations variables en cours ... \n");
    
    /************/
    // entiers, flottants scalaires :
    // a changer quand on veut
    //int	N;	// nombre de points de la discretisation en espace
    //int	M;
    double hx;		// pas de temps (unique car nlsd = N)
    double hy;
    
    
    
    /************/
    // vecteurs :
    double	*x;	// points centraux de nos cellules
    double	*y;	// points centraux de nos cellules
    //double	*U;	// vecteurs des variables conservatives
    double	*Ubarx;	// valeurs aux interfaces de U
    double	*Ubary;
    /*
     TODO : Notations : pour un vecteur U, Ubar represente les valeurs moyennes aux interfaces (de dimension : dim(Ubar) = dim(U) + 1)
     */
    double	*Phix;	// flux numeriques
    double	*Phiy;	// flux numeriques
    double	*F;	// termes F(U) de l'EDP
    double	*Fc;	// flux centres
    double	*G;
    double	*Gc;
    
    double	*deltaF;// differences des flux
    double	*deltaG;// differences des flux
    //double	*rho;	// masse volumique du fluide
    double	*u;	// vitesse SELON X
    double	*v;	// SELON Y
    //double	*p;	// pression P
    double	*c;	// vitesse du son c
    double	*Sx;	// vecteur intermediaire des valeurs propres de A (cf equation)
    double	*Sy;	// vecteur intermediaire des valeurs propres de A (cf equation)
    double	*Rx;	// matrice des vecteurs propres de A
    double	*Ry;	// matrice des vecteurs propres de A
    double	*Lx;	// L^t = R^{-1} matrice des vecteurs propres de A^t
    double	*Ly;	// L^t = R^{-1} matrice des vecteurs propres de A^t
    
    
    // variables de boucles et pour le temps
    int i;	// variable de boucle
    int j;	// souvent, k bouge de 0 a 3, j de 0 a N
    int k;
    double	t	= 0.0;	// T temps maximum & t temps courant
    int	it	= 0;	//nb d'iterations
    int	borne;	//nb max d'iterations
    int	debug_valeur_negative;	// un compteur si p ou c deviens negatif. 
    double	T	= 1000;	/* TODO : a modifier si besoin*/
    
    /************/
    // suite des vecteurs :
    double	*ubarx;	// valeurs aux interfaces
    double	*ubary;	// valeurs aux interfaces
    double	*vbarx;
    double	*vbary;
    double	*pbarx;
    double	*pbary;
    double	*cbarx;
    double	*cbary;
    double	*Hbarx;
    double	*Hbary;
    double	*pdt;	// vecteur intermediaire pour le calcul du pas de temps
    double	*SSx;	// matrice diagonale des vecteurs propres
    double	*SSy;	// matrice diagonale des vecteurs propres
    double	*resu;	// matrices temporaires pour les produits matriciels
    double	*IR;
    double	*SG;
    double	*PROD;
    double	*df2;	// vecteur intermediaire
    
    /************/
    // Chaines de caracteres
    char*	nom_graphique;	// titre du fichier dans lequel on imprime les resultats
    double	*testid;
    
	//Initialisation des variables qui seront fixes :
	/////////////////////////////////////////////////
    // FIXME : Selon les plateformes, Sophie ou Maxime compilent avec un IDE, donc sans donner d'arguments
    //		et Lilian compile dans un terminal, et peut donner des arguments
	//N	= GET_N();
	//M	= GET_M();
    //printf("n= %d, M= %d \n",N,M);
	/*
     //N 	= atoi (argv[1]);	//nb de points du decoupage du segment [0,1] horizontal : argument de notre executable
     N 	= 5;	//nb de points du decoupage du segment [0,1] horizontal : argument de notre executable
     //M 	= atoi (argv[2]);	//nb de points du decoupage du segment [0,1] vertical : argument de notre executable
     M	= 5;	//nb de points du decoupage du segment [0,1] vertical : argument de notre executable
     */
    
	hx= 1. / N;			//pas de la discretisation
	hy= 1. / M;
    
	//borne 	= atoi(argv[3]);			//nombre maximum d'iteration : argument de notre executable
	borne 	= 1000;			//nombre maximum d'iteration : argument de notre executable
	//borne = GET_Borne();
	
	/*********************************/
    /*
     debug_valeur_negative = 0;
     void check_negatif(double a, const char* nom_variable, int i, int j){ 
     debug_valeur_negative += (a < 0); 
     if(a < 0) {
     fprintf(stderr, "\n/!\\ Une valeur a ete trouvee negative pour la variable %s, valant %f, en i=%d et j=%d. C'est l'erreur numero = %d.", nom_variable, a, debug_valeur_negative, i, j);
     fflush(stderr);
     if (GET_DEBUG__VOIR_VTK) {debug(GET_DEBUG__VOIR_VTK);}
     }
     }
     */
	/*********************************/
    
	//Messages d'erreurs en cas de conditions non respectees sur les arguments de l'executable.
	if (borne <= 0 )
    {
		fprintf (stderr, "ERROR - ./euler_2D_C : La borne doit etre un entier positif.\n"); fflush (stderr); exit (4);
    }
	if (N <= 0 )
    {
		fprintf (stderr, "ERROR - ./euler_2D_C : Le nombre de points <N> selon x doit etre un entier > 0.\n"); fflush (stderr); exit (5);
    }
	if (M <= 0 )
    {
		fprintf (stderr, "ERROR - ./euler_2D_C : Le nombre de points <M> selon y doit etre un entier > 0.\n"); fflush (stderr); exit (6);
    }
    
	//Ecrire les parametres de la simulation :
	printf ("\nNombre de points de la simulation selon x : N = %d\n", N);
	printf ("\t\tPas de la simulation hx selon x = %f\n", hx);
	printf ("Nombre de points de la simulation selon y : M = %d\n", M);
	printf ("\t\tPas de la simulation hy selon y = %f\n", hy);
    
	printf ("Nombre d'etapes maximum de la simulation = %d\n", borne);
	//printf ("Orientation des conditions initiales de pression (P) et de masse (rho) : %s.\n\n", ((GET_CONDITION_INITIALES == 0)? ("y") : ((GET_CONDITION_INITIALES == 1)? ("x") : ("circ"))));
	
    printf("**************************************************************\n");
	printf(" Declarations variables C finie !\n");
    
    // allocations de memoires CPU
    ///////////////////////////////
	printf(" Allocation memoire C en cours ... \n");
    
    /********************************************
     Il y a plusieurs sortes de tableaux :
     taille N : valeurs moyennes sur les cellules (u, v, p, c, rho; et x)
     taille N+1 : valeurs aux interfaces (ubar, vbar, pbar, cbar, etc)
     taille 4*N : quatres informations en chaque cellule (U, F)
     taille 4*(N+1) : quatres informations en chaque interface (Ubar, Fc, deltaF, Phi etc)
     taille 4*4 : matrice 4x4 temporaire pour le calcul des flux (methode VFFC)
     taille 4*4*(N+1) : matrices 4x4 en chaque interfaces pour le calcul des flux (L et R et SS)
     */
    x	= (double*) malloc(	N	  * sizeof(double));
    y	= (double*) malloc(	M	  * sizeof(double));
    //U	= (double*) malloc(	4*N*M	  * sizeof(double));
    Ubarx	= (double*) malloc(	4*(N+1)*M	  * sizeof(double));
    Ubary	= (double*) malloc(	4*(N)*(M+1)	  * sizeof(double));
    Phix	= (double*) malloc(	4*(N+1)*M	  * sizeof(double));
    Phiy	= (double*) malloc(	4*(N)*(M+1)	  * sizeof(double));
    F	= (double*) malloc(	4*N*M	  * sizeof(double));
    Fc	= (double*) malloc(	4*(N+1)*M	  * sizeof(double));
    deltaF	= (double*) malloc(	4*(N+1)*M	  * sizeof(double));
    G	= (double*) malloc(	4*N*M	  * sizeof(double));
    Gc	= (double*) malloc(	4*N*(M+1)	  * sizeof(double));
    deltaG	= (double*) malloc(	4*N*(M+1)	  * sizeof(double));
    //rho	= (double*) malloc(	N*M	  * sizeof(double));
    u	= (double*) malloc(	N*M		  * sizeof(double));
    v	= (double*) malloc(	N*M		  * sizeof(double));
    //p	= (double*) malloc(	N*M		  * sizeof(double));
    c	= (double*) malloc(	N*M		  * sizeof(double));
    Sx	= (double*) malloc(	4*(N+1)*M	  * sizeof(double));
    Sy	= (double*) malloc(	4*N*(M+1)	  * sizeof(double));
    Rx	= (double*) malloc(	4*4*(N+1)*M * sizeof(double));
    Ry	= (double*) malloc(	4*4*N*(M+1) * sizeof(double));
    ubarx	= (double*) malloc(	1*(N+1)*M	  * sizeof(double));
    ubary	= (double*) malloc(	1*N*(M+1)	  * sizeof(double));
    vbarx	= (double*) malloc(	1*(N+1)*M	  * sizeof(double));
    vbary	= (double*) malloc(	1*N*(M+1)	  * sizeof(double));
    pbarx	= (double*) malloc(	1*(N+1)*M	  * sizeof(double));
    pbary	= (double*) malloc(	1*N*(M+1)	  * sizeof(double));
    cbarx	= (double*) malloc(	1*(N+1)*M	  * sizeof(double));
    cbary	= (double*) malloc(	1*N*(M+1)	  * sizeof(double));
    Hbarx	= (double*) malloc(	1*(N+1)*M	  * sizeof(double));
    Hbary	= (double*) malloc(	1*N*(M+1)	  * sizeof(double));
    pdt	= (double*) malloc(	1*(N+1)	  * sizeof(double));
    SSx	= (double*) malloc(	4*4*(N+1)*M	  * sizeof(double));
    SSy	= (double*) malloc(	4*4*N*(M+1)	  * sizeof(double));
    resu	= (double*) malloc(	4*4	  * sizeof(double));
    IR	= (double*) malloc(	4*4	  * sizeof(double));
    SG	= (double*) malloc(	4*4	  * sizeof(double));
    PROD	= (double*) malloc(	4	  * sizeof(double));
    df2	= (double*) malloc(	4	  * sizeof(double));
    Lx	= (double*) malloc(	4*4*(N+1)*M	* sizeof(double));
    Ly	= (double*) malloc(	4*4*N*(M+1)	* sizeof(double));
    testid	= (double*) malloc(	16	  * sizeof(double)); // TODO : deprecated
    
    // Nom du fichier texte dans lequel on ecrit les donnees numeriques
    nom_graphique	= (char*) malloc(	sizeof(char)*	400);
    
    
	printf(" Allocation memoire CPU finie !\n");
	////////////////////////////////////////////////////
    //debug();
    
	///////////////////////////////////////////////////////////////////////////////////////////
   	// Initialisation des tableaux C
	//////////////////////////////////////////////////////////////////////////////////////////
    
	printf(" Initialisation C en cours ... \n");
    
    // Points x
    for (j = 0 ; j < N ; j++)
    {
        x[j]=hx*0.5+j*hx;
        //if(GET_DEBUG__VOIR_VTK > 3) printf("x[%d]=%f\n",j,x[j]);
    }
    
    for (i = 0 ; i < M ; i++)
    {
        y[i]=hy*0.5+i*hy;
        //if(GET_DEBUG__VOIR_VTK > 3) printf("y[%d]=%f\n",i,y[i]);
    }
    
    
    printf("**************************************************************\n");
    // Initialisation de rho
    for (i = 0 ; i < M ; i++)
    {
        for (j = 0 ; j < N ; j++)
        { switch (choix) {
            case 0:
                if (y[i]<=0.5)
                    //				if ((x[j]<=0.5)&&(y[i]<=0.5))
                {
                    rho[i*N+j]=0.125;
                }
                else
                {
                    rho[i*N+j]=1.0;
                }
                break;
            case 1:
                if (x[j]<=0.5)
                    //				if ((x[j]<=0.5)&&(y[i]<=0.5))
                {
                    rho[i*N+j]=0.125;
                }
                else
                {
                    rho[i*N+j]=1.0;
                }
                break;
            case 2:
                //rho[i*N+j] = 0.125 + 0.875*((norme_carre(x[j] - 0.5, y[i] - 0.5))<carre(0.25));
                rho[i*N+j]=0.2;
        }
        }
        /*
         if (GET_CONDITION_INITIALES == 2){	
         rho[i*N+j] = 0.125 + 0.875*((norme_carre(x[j] - 0.75, y[i] - 0.75))<carre(0.25));
         }
         if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,%d]=%f\n",i,j,rho[i*N+j]);
         }
         */
    }
    
    printf("**************************************************************\n");
    // Initialisation de u TODO : tester des nouvelles CI
    for (i = 0 ; i < M ; i++)
    {
        for (j = 0 ; j < N ; j++)
        {
			if (y[i]<=0.5)
			{
				u[i*N+j]=0.;
			}
			else
			{
				u[i*N+j]=0.;
			}
			//if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,%d]=%f\n",i,j,u[i*N+j]);
        }
    }
    
    printf("**************************************************************\n");
    // Initialisation de v
    for (i = 0 ; i < M ; i++)
    {
        for (j = 0 ; j < N ; j++)
        {
			if (y[i]<=0.5)
			{
				v[i*N+j]=0.;
			}
			else
			{
				v[i*N+j]=0.;
			}
			//if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,%d]=%f\n",i,j,v[i*N+j]);
        }
    }
    
    printf("**************************************************************\n");
    // Initialisation de p
    for (i = 0 ; i < M ; i++)
    {
        for (j = 0 ; j < N ; j++)
        {
		    switch (choix) {
                case 0:
                    if (y[i]<=0.5)
                    {
                        p[i*N+j]=0.1;
                    }
                    else
                    {
                        p[i*N+j]=1;
                    }
                    break;
                case 1:
                    if (x[j]<=0.5)
                    {
                        p[i*N+j]=0.1;
                    }
                    else
                    {
                        p[i*N+j]=1;
                    }
                    break;
                case 2:
                    p[i*N+j] = 0.2;
                    //p[i*N+j] = 0.1 + 0.9*((norme_carre(x[j] - 0.5, y[i] - 0.5))<carre(0.25));
                    break;
                    
            }
        }
    }
    
    printf("**************************************************************\n");
    // Initialisation de c
    for (i = 0 ; i < M ; i++)
    {
        for (j = 0 ; j < N ; j++)
        {
            c[i*N+j]=sqrt(1.4*p[i*N+j]/rho[i*N+j]);
			//if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,%d]=%f\n",i,j,c[i*N+j]);
        }
    }
    
    printf("**************************************************************\n");
    // Initialisation de U
    // U = [ rho ; rho u ; rho v ; rho E ]
    for (i = 0 ; i < M ; i++)
    {
        for (j = 0 ; j < N ; j++)
        {
            U[i*N+j]=rho[i*N+j];
            U[N*M+i*N+j]=rho[i*N+j]*u[i*N+j];
            U[2*N*M+i*N+j]=rho[i*N+j]*v[i*N+j];
            U[3*N*M+i*N+j]=0.5*rho[i*N+j]*(u[i*N+j]*u[i*N+j]+v[i*N+j]*v[i*N+j])+p[i*N+j]/0.4;
			
             printf("U0[%d,%d]=%f\n",i,j,U[i*N+j+0*N*M]);
             printf("U1[%d,%d]=%f\n",i,j,U[i*N+j+1*N*M]);
             printf("U2[%d,%d]=%f\n",i,j,U[i*N+j+2*N*M]);
             printf("U3[%d,%d]=%f\n",i,j,U[i*N+j+3*N*M]);
             printf("\n"); 
			
        }
    }
    
    
    
	printf(" Initialisation CPU finie \n");
	/////////////////////////////////////////
    
    
    
    
    //////////////////////////////////////////////////////////////////////////////////////////
    // DEBUT du corps de la boucle en temps
    //////////////////////////////////////////////////////////////////////////////////////////
    
	// on limite le temps, et le nombre de pas de temps (pour eviter de laisser la simulation s'emballer)
    while ((t <= T) && (it <= borne) )
    {
        
        //debugm(GET_DEBUG__VOIR_VTK, "Nouvelle boucle");
        fflush(stdout);
        
        //////////////////////////////////////////////////
        //
        //	SPLITTING SELON X
        //
        //////////////////////////////////////////////////
        
        ////////////////////////////////////////////////////////
        // Actualisation des variables aux interfaces
        
        // Calcul de Ubarx
        for (k = 0 ; k < 4 ; k++)
        {
            for (i = 0 ; i < M ; i++)
            {
                for (j = 1 ; j < N ; j++)
                {
                    Ubarx[i*(N+1)+j+k*(N+1)*M]=0.5*(U[i*(N)+j-1+k*M*N]+U[i*(N)+j+k*M*N]);	// valeur moyenne
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarx%d[%d,%d]=%f\n",k,i,j,Ubarx[i*(N+1)+j+k*(N+1)*M]);
                }
                Ubarx[i*(N+1)+0+k*(N+1)*M]=0.5*(U[i*(N)+k*M*N]+U[i*(N)+N-1+k*M*N]);		// interface extreme gauche
                Ubarx[i*(N+1)+N+k*(N+1)*M]=0.5*(U[i*(N)+k*M*N]+U[i*(N)+N-1+k*M*N]);		// interface extreme droite
				//if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarx%d[%d,%d]=%f\n",k,i,0,Ubarx[i*(N+1)+0+k*(N+1)*M]);
				//if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarx%d[%d,%d]=%f\n",k,i,0,Ubarx[i*(N+1)+N+k*(N+1)*M]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ubarx");
        
        // Calcul de ubarx
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                ubarx[i*(N+1)+j]= Ubarx[1*(N+1)*M+i*(N+1)+j]/Ubarx[i*(N+1)+j];
                /**/			// u = U[1] / U[0] = rho u / rho
                
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de ubarx");
        
        // Calcul de vbarx
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                vbarx[i*(N+1)+j]= Ubarx[2*(N+1)*M+i*(N+1)+j]/Ubarx[i*(N+1)+j];
                /**/			// u = U[1] / U[0] = rho u / rho
                //if(GET_DEBUG__VOIR_VTK > 3) printf("vbarx[%d,%d]=%f\n",i,j,vbarx[i*(N+1)+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de vbarx");
        
        // Calcul de pbarx
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                pbarx[i*(N+1)+j]= 0.4*Ubarx[3*(N+1)*M+i*(N+1)+j]-0.5*0.4*Ubarx[0*(N+1)*M+i*(N+1)+j]*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]);
                //check_negatif(pbarx[i*(N+1)+j], "pbarx", i, j);
                /**/			// p = (gamma - 1) * rho * e
                //if(GET_DEBUG__VOIR_VTK > 3) printf("pbarx[%d,%d]=%f\n",i,j,pbarx[i*(N+1)+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de pbarx");
        
        // Calcul de cbarx
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                cbarx[i*(N+1)+j]= sqrt(1.4*(pbarx[i*(N+1)+j])/Ubarx[0*(N+1)*M+i*(N+1)+j]);
                //check_negatif(cbarx[i*(N+1)+j], "cbarx", i, j);
                /**/			// c = racine(gamma * p / rho)
                //if(GET_DEBUG__VOIR_VTK > 3) printf("cbarx[%d,%d]=%f\n",i,j,cbarx[i*(N+1)+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de cbarx");
        
        // Calcul de Hbarx (variable intermediaire pour le calcul des valeurs propres)
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                Hbarx[i*(N+1)+j]= Ubarx[3*(N+1)*M+i*(N+1)+j]/Ubarx[0*(N+1)*M+i*(N+1)+j]+pbarx[i*(N+1)+j]/Ubarx[0*(N+1)*M+i*(N+1)+j];
                /**/			// H = E + p / rho = rho E / rho + p / rho
                //if(GET_DEBUG__VOIR_VTK > 3) printf("Hbarx[%d,%d]=%f\n",i,j,Hbarx[i*(N+1)+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Hbarx");
        
        
        //////////////////////////////////
        //Calcul du pas de temps
        
        double dt;			// variable fraiche
        //dt = CFL * hx / maxu(pdt,N+1);	// fonction externe definie plus bas
        //dt = pas_de_temps(ubarx, cbarx, vbary, cbary, (N+1)*M, (M+1)*N, hx, hy);
        dt = pas_de_temps2(u, c, v, N, M, hx, hy, it);
        // dt=0.00001;			// valeur test
        // getchar();
        
        t	=	t + dt;		// on actualise la valeur du temps reel (inutilisee)
        it++;				// et le compteur de boucle de temps (UTILE)
        printf("Nouvelle etape : t = %f dt = %f it = %d. \n", t, dt, it);
        // printf("\ndt = %f\t it = %d\n",dt,it);
        //debug();
        
        ////////////////////////////////////
        // Calcul des valeurs propres
        
        // On utilise les valeurs formelles connues
        //  d'apres le document trouves a l'adresse suivante : <http://www.cmla.ens-cachan.fr/fileadmin/Membres/ghidaglia/fluide1.pdf>
        //	et <http://www.cmla.ens-cachan.fr/fileadmin/Membres/ghidaglia/rapport.zip>
        //  de Jean-Michel Ghidaglia et collaborateurs (CMLA).
        
        /* Nos matrices 4x4 sont indices comme cela :
         [
         0   1   2   3
         4   5   6   7
         8   9   10  11
         12  13  14  15
         ]
         ********/
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                
                ////////////////////////////////////
                // Matrice R : valeurs propres de A
                Rx[0+	16*(i*(N+1)+j) ] = 1.0;
                Rx[1+	16*(i*(N+1)+j) ] = 1.0;
                Rx[2+	16*(i*(N+1)+j) ] = 0.0;
                Rx[3+	16*(i*(N+1)+j) ] = 1.0;
                
                Rx[4+	16*(i*(N+1)+j) ] = ubarx[i*(N+1)+j]-cbarx[i*(N+1)+j];
                Rx[5+	16*(i*(N+1)+j) ] = ubarx[i*(N+1)+j];
                Rx[6+	16*(i*(N+1)+j) ] = 0.0;
                Rx[7+	16*(i*(N+1)+j) ] = ubarx[i*(N+1)+j]+cbarx[i*(N+1)+j];
                
                Rx[8+	16*(i*(N+1)+j) ] = vbarx[i*(N+1)+j];
                Rx[9+	16*(i*(N+1)+j) ] = vbarx[i*(N+1)+j];
                Rx[10+	16*(i*(N+1)+j) ] = 1.0;
                Rx[11+	16*(i*(N+1)+j) ] = vbarx[i*(N+1)+j];
                
                Rx[12+	16*(i*(N+1)+j) ] = Hbarx[i*(N+1)+j]-ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j];
                Rx[13+	16*(i*(N+1)+j) ] = 0.5*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]);
                Rx[14+	16*(i*(N+1)+j) ] = vbarx[i*(N+1)+j];
                Rx[15+	16*(i*(N+1)+j) ] = Hbarx[i*(N+1)+j]+ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j];
                /*
                 if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 16 ; k++)
                 {
                 printf("Rx[%d,\t%d,\t%d]=%f\n",i,j,k,Rx[k+	16*(i*(N+1)+j) ]);
                 }
                 }
                 
                 debugm(GET_DEBUG__VOIR_VTK, "Calcul de Rx");
                 */
                
                ////////////////////////////////////
                // Matrice L : valeurs propres de A^t
                
                Lx[	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j]+0.4*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]-Hbarx[i*(N+1)+j])+ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j]);
                Lx[1+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*ubarx[i*(N+1)+j]-cbarx[i*(N+1)+j]);
                Lx[2+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*vbarx[i*(N+1)+j]);
                Lx[3+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*0.4;
                
                Lx[4+	16*(i*(N+1)+j) ] = 0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(Hbarx[i*(N+1)+j]-(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]));
                Lx[5+	16*(i*(N+1)+j) ] = 0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*ubarx[i*(N+1)+j];
                Lx[6+	16*(i*(N+1)+j) ] = 0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(vbarx[i*(N+1)+j]);
                Lx[7+	16*(i*(N+1)+j) ] = -0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j]);
                
                Lx[8+	16*(i*(N+1)+j) ] = -vbarx[i*(N+1)+j];
                Lx[9+	16*(i*(N+1)+j) ] = 0.0;
                Lx[10+	16*(i*(N+1)+j) ] = 1.0;
                Lx[11+	16*(i*(N+1)+j) ] = 0.0;
                
                Lx[12+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j]+0.4*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]-Hbarx[i*(N+1)+j])-ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j]);
                Lx[13+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*ubarx[i*(N+1)+j]+cbarx[i*(N+1)+j]);
                Lx[14+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*vbarx[i*(N+1)+j]);
                Lx[15+	16*(i*(N+1)+j) ] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*0.4;;
                /*
                 if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 16 ; k++)
                 {
                 printf("Lx[%d,\t%d,\t%d]=%f\n",i,j,k,Lx[k+	16*(i*(N+1)+j) ]);
                 }
                 }
                 */			
                //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Lx");
                
                
                //produitmatriciel(&Rx[16*(i*(N+1)+j)],&Lx[16*(i*(N+1)+j)],4,testid);
                //test_identite_4_4(GET_DEBUG__VOIR_VTK, testid, it, i, j);
                // un test pour verifier que R * L = Id_4
                
            }
        }
        
        
        //debug();
        
    	// Calculs de R et L : faits
        ///////////////////////////////////////////
        
        //////////////////////////////////////////////
        // Calcul de la matrice diagonale SS
        //  valeurs propres de A
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                
                SSx[	16*(i*(N+1)+j) ] = sgn(ubarx[i*(N+1)+j]-cbarx[i*(N+1)+j]);	// valeurs propres de A
                SSx[1+	16*(i*(N+1)+j) ] = 0.0;
                SSx[2+	16*(i*(N+1)+j) ] = 0.0;
                SSx[3+	16*(i*(N+1)+j) ] = 0.0;
                
                SSx[4+	16*(i*(N+1)+j) ] = 0.0;
                SSx[5+	16*(i*(N+1)+j) ] = sgn(ubarx[i*(N+1)+j]);
                SSx[6+	16*(i*(N+1)+j) ] = 0.0;
                SSx[7+	16*(i*(N+1)+j) ] = 0.0;
                
                SSx[8+	16*(i*(N+1)+j) ] = 0.0;
                SSx[9+	16*(i*(N+1)+j) ] = 0.0;
                SSx[10+	16*(i*(N+1)+j) ] = sgn(ubarx[i*(N+1)+j]);
                SSx[11+	16*(i*(N+1)+j) ] = 0.0;
                
                SSx[12+	16*(i*(N+1)+j) ] = 0.0;
                SSx[13+	16*(i*(N+1)+j) ] = 0.0;
                SSx[14+	16*(i*(N+1)+j) ] = 0.0;
                SSx[15+	16*(i*(N+1)+j) ] = sgn(ubarx[i*(N+1)+j]+cbarx[i*(N+1)+j]);
                
                /* if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 16 ; k++)
                 {
                 printf("SSx[%d,\t%d,\t%d]=%f\n",i,j,k,SSx[k+	16*(i*(N+1)+j) ]);
                 }
                 }
                 debugm(GET_DEBUG__VOIR_VTK, "Calcul de SSx");
                 */
                
            }
        }
        //debug();
        
        
        //////////////////////////////////////////////
        // Calcul du vecteur des flux F
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                F[i*(N)+j]		= rho[i*(N)+j]*u[i*(N)+j];
                F[N*M+i*(N)+j]		= rho[i*(N)+j]*u[i*(N)+j]*u[i*(N)+j]+p[i*(N)+j];
                F[2*N*M+i*(N)+j]	= rho[i*(N)+j]*v[i*(N)+j]*u[i*(N)+j];
                // F[3*N+j]	= (0.5*rho[j]*(u[j]*u[j]+v[j]*v[j])+p[j]/0.4+p[j])*u[j];
                F[3*N*M+i*(N)+j]	= (U[3*N*M+i*(N)+j]+p[i*(N)+j])*u[i*(N)+j];
                /*if(GET_DEBUG__VOIR_VTK > 3){
                 for (k = 0 ; k < 4 ; k++)
                 {
                 printf("F%d[%d,\t%d]=%f\n",k,i,j,F[k*N*M+	i*N+j ]);
                 } 
                 } */
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de F");
        
        //////////////////////////////////////////////
        // Calcul du vecteur des flux F centres
        for (k = 0 ; k < 4 ; k++)
        {
            for (i = 0 ; i < M ; i++)
            {
                for (j = 1 ; j < N ; j++)
                {
                    Fc[k*(N+1)*M+i*(N+1)+j]	= 0.5*(F[k*M*N+i*(N)+j-1]+F[k*M*N+i*(N)+j]);
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("Fc%d[%d,\t%d]=%f\n",k,i,j,Fc[k*(N+1)*M+	i*(N+1)+j ]);
                    /**/				// valeurs moyennes aux interfaces gauche et droite
                }
				Fc[k*(N+1)*M+i*(N+1)+0]	= 0.5*(F[k*(N)*M+i*(N)]+F[k*(N)*M+i*(N)+N-1]);	// interface extreme gauche
				Fc[k*(N+1)*M+i*(N+1)+N]	= 0.5*(F[k*(N)*M+i*(N)]+F[k*(N)*M+i*(N)+N-1]);	// interface extreme droite (FN !)
				//if(GET_DEBUG__VOIR_VTK > 3) printf("Fc%d[%d,\t%d]=%f *\n",k,i,0,Fc[k*(N+1)*M+	i*(N+1)+0 ]);
				//if(GET_DEBUG__VOIR_VTK > 3) printf("Fc%d[%d,\t%d]=%f *\n",k,i,N,Fc[k*(N+1)*M+	i*(N+1)+N ]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }	
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Fc");
        
        
        
        
        //////////////////////////////////////////////
        // Calcul du vecteur des differences de flux F
        for (k = 0 ; k < 4 ; k++)
        {
            for (i = 0 ; i < M ; i++)
            {
                for (j = 1 ; j < N ; j++)
                {
                    deltaF[k*(N+1)*M+i*(N+1)+j]	= F[k*M*N+i*(N)+j]-F[k*M*N+i*(N)+j-1];
                    /**/			// valeur a droite MOINS valeur a gauche
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("deltaF%d[%d,\t%d]=%f\n",k,i,j,deltaF[k*(N+1)*M+	i*(N+1)+j ]);
                }
				deltaF[k*(N+1)*M+i*(N+1)+0]	= F[k*M*N+i*(N)]-F[k*M*N+i*(N)+N-1];
				deltaF[k*(N+1)*M+i*(N+1)+N]	= F[k*M*N+i*(N)]-F[k*M*N+i*(N)+N-1];
				//if(GET_DEBUG__VOIR_VTK > 3) printf("deltaF%d[%d,\t%d]=%f *\n",k,i,0,deltaF[k*(N+1)*M+	i*(N+1)+0 ]);
				//if(GET_DEBUG__VOIR_VTK > 3) printf("deltaF%d[%d,\t%d]=%f *\n",k,i,N,Fc[k*(N+1)*M+	i*(N+1)+N ]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        //debug();	
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de deltaF");
        
        
        //////////////////////////////////////////////
        // Calcul matriciel
        /**/	// On doit calculer Phi_i = (F_i+1 + F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N+1 ; j++)
            {
                /*			IR	= malloc(	16	  * sizeof(double));*/
				// variables temporaires du produit
                df2[0]	= deltaF[i*(N+1)+j];
                df2[1]	= deltaF[(N+1)*M+i*(N+1)+j];
                df2[2]	= deltaF[2*(N+1)*M+i*(N+1)+j];
                df2[3]	= deltaF[3*(N+1)*M+i*(N+1)+j];
                
                // printf("Produits : i=%d\tj=%d\n",i,j);
                produitmatriciel(&Rx[(i*(N+1)+j)*16], &SSx[(i*(N+1)+j)*16], 4, resu);
				// resu = R * diag(sign(lamba_k))
                produitmatriciel(resu, &Lx[(i*(N+1)+j)*16], 4, SG);
				// SG = R * diag(sign(lamba_k)) * R^{-1}
                produitvecteur(SG, df2, 4, PROD);
				// PROD =  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
                
                //debug();
                
                for (k = 0 ; k < 4 ; k++)
                {
                    Phix[k*(N+1)*M+i*(N+1)+j]	= Fc[k*(N+1)*M+i*(N+1)+j]-0.5*PROD[k];
                    /**/				// On doit calculer Phi_i = (F_i+1 + F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
                }
            }
        }
        // Fin Calcul matriciel
        //////////////////////////////////////////////
        
        //debug();
        
        ////////////////////////////////////////////////////////////////
        // ACTUALISATION du vecteur des variables conservatives
        for (k = 0 ; k < 4 ; k++)
        {
            for (i = 0 ; i < M ; i++)
            {
                for (j = 0 ; j < N ; j++)
                {
                    U[k*M*N+i*N+j] -= (dt / hx)*(Phix[k*(N+1)*M+i*(N+1)+j+1]-Phix[k*(N+1)*M+i*(N+1)+j]);
                    /**/			// U_j^{n+1} = U_j^n + (dt / h) . Phi_{j+1}^n - Phi_j^n
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("U[%d,\t%d,\t%d]=%f\n",i,j,k,U[k*M*N+i*N+j]);
                }
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }	
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de U n*");
        printf(" : splitting selon X\n");
        
        
        //////////////////////////////////////////////////////////////////////////////
        // ACTUALISATION des variables conservatives en fonction du nouveau U
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                rho[i*N+j ]	= U[i*N+j];	// rho = U_0
                u[i*N+j ]	= U[1*N*M+i*N+j]/U[0*N*M+i*N+j];	// u = U_1 / U_0
                v[i*N+j ]	= U[2*N*M+i*N+j]/U[0*N*M+i*N+j];	// v = U_2 / U_0
                p[i*N+j ]	= 0.4*(U[3*N*M+i*N+j]-0.5*U[0*N*M+i*N+j]*(u[i*N+j ]*u[i*N+j ]+v[i*N+j ]*v[i*N+j ]));
				// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
				// car U_4 = rho.E
				// E =  e + 0.5*||u^{->}||^2
                c[i*N+j ]	= sqrt(1.4*p[i*N+j]/rho[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,\t%d]=%f\n",i,j,rho[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,\t%d]=%f\n",i,j,u[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,\t%d]=%f\n",i,j,v[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,\t%d]=%f\n",i,j,p[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,\t%d]=%f\n",i,j,c[i*N+j]);
                //debug();
                //check_negatif((rho[i*(N)+j]), "rho n*", i, j);
                //check_negatif((p[i*(N)+j]), "p n*", i, j);
                //check_negatif((c[i*(N)+j]), "c n*", i, j);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de rho u v p c : n*");
        
        //////////////////////////////////////////////////
        //
        //	SPLITTING SELON Y
        //
        //////////////////////////////////////////////////
        
        
        ////////////////////////////////////////////////////////
        // Actualisation des variables aux interfaces
        
        // Calcul de Ubary
        for (k = 0 ; k < 4 ; k++)
        {
            for (j = 0 ; j < N ; j++)
            {
                for (i = 1 ; i < M ; i++)
                {
                    Ubary[i*(N)+j+k*(N)*(M+1)]=0.5*(U[i*(N)+j+k*M*N]+U[(i-1)*(N)+j+k*M*N]);	// valeur moyenne
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("Ubary%d[%d,%d]=%f\n",k,i,j,Ubary[i*(N)+j+k*(N)*(M+1)]);
                }
                
                Ubary[0*N+j+k*N*(M+1)]=0.5*(U[0*(N)+j+k*M*N]+U[(M-1)*(N)+j+k*M*N]);		// interface extreme bas
				//if(GET_DEBUG__VOIR_VTK > 3) printf("Ubary%d[%d,%d]=%f\n",k,0,j,Ubary[0*(N)+j+k*(N)*(M+1)]);
                Ubary[M*N+j+k*N*(M+1)]=0.5*(U[0*(N)+j+k*M*N]+U[(M-1)*(N)+j+k*M*N]);		// interface extreme haut
				//if(GET_DEBUG__VOIR_VTK > 3) printf("Ubary%d[%d,%d]=%f\n\n",k,M,j,Ubary[M*(N)+j+k*(N)*(M+1)]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ubary");
        
        // Calcul de ubary
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                ubary[i*N+j]= Ubary[1*(N)*(M+1)+i*N+j]/Ubary[0*(N)*(M+1)+i*N+j];
                /**/			// u = U[1] / U[0] = rho u / rho
                //if(GET_DEBUG__VOIR_VTK > 3) printf("ubary[%d,%d]=%f\n",i,j,ubary[i*N+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de ubary");
        
        // Calcul de vbary
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                vbary[i*(N)+j]= Ubary[2*(N)*(M+1)+i*N+j]/Ubary[0*(N)*(M+1)+i*N+j];
                /**/			// u = U[1] / U[0] = rho u / rho
				//if(GET_DEBUG__VOIR_VTK > 3) printf("vbary[%d,%d]=%f\n",i,j,vbary[i*N+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de varby");
        
        // Calcul de pbary
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                pbary[i*(N)+j]= 0.4*Ubary[3*(N)*(M+1)+i*N+j]-0.5*0.4*Ubary[0*(N)*(M+1)+i*N+j]*(ubary[i*(N)+j]*ubary[i*(N)+j]+vbary[i*(N)+j]*vbary[i*(N)+j]);
                /**/			// p = (gamma - 1) * rho * e
                //if(GET_DEBUG__VOIR_VTK > 3) printf("pbary[%d,%d]=%f\n",i,j,pbary[i*N+j]);
                //check_negatif(pbary[i*(N)+j], "pbary", i, j);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de pbary");
        
        // Calcul de cbary
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                cbary[i*(N)+j]= sqrt(1.4*(pbary[i*(N)+j])/Ubary[0*(N)*(M+1)+i*N+j]);
                /**/			// c = racine(gamma * p / rho)
                //if(GET_DEBUG__VOIR_VTK > 3) printf("cbary[%d,%d]=%f\n",i,j,cbary[i*N+j]);
                //check_negatif(cbary[i*(N)+j], "cbary", i, j);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de cbary");
        
        // Calcul de Hbary (variable intermediaire pour le calcul des valeurs propres)
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                Hbary[i*(N)+j]= Ubary[3*(N)*(M+1)+i*N+j]/Ubary[0*(N)*(M+1)+i*N+j]+pbary[i*(N)+j]/Ubary[0*(N)*(M+1)+i*N+j];
                /**/			// H = E + p / rho = rho E / rho + p / rho
                //if(GET_DEBUG__VOIR_VTK > 3) printf("Hbary[%d,%d]=%f\n",i,j,Hbary[i*N+j]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }
        
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Hbary");
        
        
        //////////////////////////////////
        //Calcul du pas de temps
        // N'est plus a faire ici, car deja expedie dans le splitting selon x.
        
        ////////////////////////////////////
        // Calcul des valeurs propres
        
        // On utilise les valeurs formelles connues
        //  d'apres le document trouves a l'adresse suivante : <http://www.cmla.ens-cachan.fr/fileadmin/Membres/ghidaglia/fluide1.pdf>
        //	et <http://www.cmla.ens-cachan.fr/fileadmin/Membres/ghidaglia/rapport.zip>
        //  de Jean-Michel Ghidaglia et collaborateurs (CMLA).
        
        /* Nos matrices 4x4 sont indices comme cela :
         [
         0   1   2   3
         4   5   6   7
         8   9   10  11
         12  13  14  15
         ]
         ********/
        
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                
                ////////////////////////////////////
                // Matrice R : valeurs propres de A
                Ry[0+	16*(i*(N)+j) ] = 1.0;
                Ry[1+	16*(i*(N)+j) ] = 1.0;
                Ry[2+	16*(i*(N)+j) ] = 0.0;
                Ry[3+	16*(i*(N)+j) ] = 1.0;
                
                Ry[4+	16*(i*(N)+j) ] = ubary[i*(N)+j];
                Ry[5+	16*(i*(N)+j) ] = ubary[i*(N)+j];
                Ry[6+	16*(i*(N)+j) ] = 1.0;
                Ry[7+	16*(i*(N)+j) ] = ubary[i*(N)+j];
                
                Ry[8+	16*(i*(N)+j) ] = vbary[i*(N)+j]-cbary[i*(N)+j];
                Ry[9+	16*(i*(N)+j) ] = vbary[i*(N)+j];
                Ry[10+	16*(i*(N)+j) ] = 0.0;
                Ry[11+	16*(i*(N)+j) ] = vbary[i*(N)+j]+cbary[i*(N)+j];
                
                Ry[12+	16*(i*(N)+j) ] = Hbary[i*(N)+j]-vbary[i*(N)+j]*cbary[i*(N)+j];
                Ry[13+	16*(i*(N)+j) ] = 0.5*(ubary[i*(N)+j]*ubary[i*(N)+j]+vbary[i*(N)+j]*vbary[i*(N)+j]);
                Ry[14+	16*(i*(N)+j) ] = ubary[i*(N)+j];
                Ry[15+	16*(i*(N)+j) ] = Hbary[i*(N)+j]+vbary[i*(N)+j]*cbary[i*(N)+j];
                /*
                 if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 16 ; k++)
                 {
                 printf("Ry[%d,\t%d,\t%d]=%f\n",i,j,k,Ry[k+	16*(i*(N)+j) ]);
                 }
                 }
                 debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ry");
                 */
                
                ////////////////////////////////////
                // Matrice L : valeurs propres de A^t
                
                Ly[	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(cbary[i*(N)+j]*cbary[i*(N)+j]+0.4*(ubary[i*(N)+j]*ubary[i*(N)+j]+vbary[i*(N)+j]*vbary[i*(N)+j]-Hbary[i*(N)+j])+vbary[i*(N)+j]*cbary[i*(N)+j]);
                Ly[1+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(-0.4*ubary[i*(N)+j]);
                Ly[2+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(-0.4*vbary[i*(N)+j]-cbary[i*N+j]);
                Ly[3+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*0.4;
                
                Ly[4+	16*(i*(N)+j) ] = (0.4/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(Hbary[i*(N)+j]-(ubary[i*(N)+j]*ubary[i*(N)+j]+vbary[i*(N)+j]*vbary[i*(N)+j]));
                Ly[5+	16*(i*(N)+j) ] = (0.4/(cbary[i*(N)+j]*cbary[i*(N)+j]))*ubary[i*(N)+j];
                Ly[6+	16*(i*(N)+j) ] = (0.4/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(vbary[i*(N)+j]);
                Ly[7+	16*(i*(N)+j) ] = -(0.4/(cbary[i*(N)+j]*cbary[i*(N)+j]));
                
                Ly[8+	16*(i*(N)+j) ] = -ubary[i*(N)+j];
                Ly[9+	16*(i*(N)+j) ] = 1.0;
                Ly[10+	16*(i*(N)+j) ] = 0.0;
                Ly[11+	16*(i*(N)+j) ] = 0.0;
                
                Ly[12+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(cbary[i*(N)+j]*cbary[i*(N)+j]+0.4*(ubary[i*(N)+j]*ubary[i*(N)+j]+vbary[i*(N)+j]*vbary[i*(N)+j]-Hbary[i*(N)+j])-vbary[i*(N)+j]*cbary[i*(N)+j]);
                Ly[13+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(-0.4*ubary[i*(N)+j]);
                Ly[14+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*(-0.4*vbary[i*(N)+j]+cbary[i*(N)+j]);
                Ly[15+	16*(i*(N)+j) ] = (0.5/(cbary[i*(N)+j]*cbary[i*(N)+j]))*0.4;;
                /*
                 if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 16 ; k++)
                 {
                 printf("Ly[%d,\t%d,\t%d]=%f\n",i,j,k,Ly[k+	16*(i*(N)+j) ]);
                 }
                 }
                 debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ly");
                 
                 produitmatriciel(&Ry[16*(i*(N)+j)],&Ly[16*(i*(N)+j)],4,testid);
                 test_identite_4_4(GET_DEBUG__VOIR_VTK, testid, it, i, j);
                 // un test pour verifier que R * L = Id_4
                 */
            }
        }
        
        //debug();
        
        // Calculs de R et L : faits
        ///////////////////////////////////////////
        
        
        //////////////////////////////////////////////
        // Calcul de la matrice diagonale SS
        //  valeurs propres de A
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                
                SSy[	16*(i*(N)+j) ] = sgn(vbary[i*(N)+j]-cbary[i*(N)+j]);	// valeurs propres de A
                SSy[1+	16*(i*(N)+j) ] = 0.0;
                SSy[2+	16*(i*(N)+j) ] = 0.0;
                SSy[3+	16*(i*(N)+j) ] = 0.0;
                
                SSy[4+	16*(i*(N)+j) ] = 0.0;
                SSy[5+	16*(i*(N)+j) ] = sgn(vbary[i*(N)+j]);
                SSy[6+	16*(i*(N)+j) ] = 0.0;
                SSy[7+	16*(i*(N)+j) ] = 0.0;
                
                SSy[8+	16*(i*(N)+j) ] = 0.0;
                SSy[9+	16*(i*(N)+j) ] = 0.0;
                SSy[10+	16*(i*(N)+j) ] = sgn(vbary[i*(N)+j]);
                SSy[11+	16*(i*(N)+j) ] = 0.0;
                
                SSy[12+	16*(i*(N)+j) ] = 0.0;
                SSy[13+	16*(i*(N)+j) ] = 0.0;
                SSy[14+	16*(i*(N)+j) ] = 0.0;
                SSy[15+	16*(i*(N)+j) ] = sgn(vbary[i*(N)+j]+cbary[i*(N)+j]);
                /*
                 if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 16 ; k++)
                 {
                 printf("SSy[%d,\t%d,\t%d]=%f\n",i,j,k,SSy[k+	16*(i*(N)+j) ]);
                 }
                 }
                 debugm(GET_DEBUG__VOIR_VTK, "Calcul de SSy");
                 */
            }
        }
        
        
        //////////////////////////////////////////////
        // Calcul du vecteur des flux G
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                G[i*(N)+j]		= (rho[i*(N)+j])*(v[i*(N)+j]);
                G[N*M+i*(N)+j]		= rho[i*(N)+j]*v[i*(N)+j]*u[i*(N)+j];
                /*			rho[i*(N)+j]*u[i*(N)+j]*u[i*(N)+j]+p[i*(N)+j];*/
                G[2*N*M+i*(N)+j]	= rho[i*(N)+j]*v[i*(N)+j]*v[i*(N)+j]+p[i*(N)+j];
                G[3*N*M+i*(N)+j]	= (U[3*N*M+i*(N)+j]+p[i*(N)+j])*v[i*(N)+j];
                /*			
                 if(GET_DEBUG__VOIR_VTK > 3) {
                 for (k = 0 ; k < 4 ; k++)
                 {
                 printf("G%d[%d,\t%d]=%f\n",k,i,j,G[k*N*M+	i*N+j ]);
                 }
                 }
                 
                 debugm(GET_DEBUG__VOIR_VTK, "Calcul de G");
                 */
            }
        }
        
        //////////////////////////////////////////////
        // Calcul du vecteur des flux G centres
        for (k = 0 ; k < 4 ; k++)
        {
            for (j = 0 ; j < N ; j++)
            {
                for (i = 1 ; i < M ; i++)
                {
                    Gc[k*(N)*(M+1)+i*(N)+j]	= 0.5*(G[k*M*N+(i)*(N)+j]+G[k*M*N+(i-1)*(N)+j]);
                    
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("Gc%d[%d,\t%d]=%f\n",k,i,j,Gc[k*(N)*(M+1)+	i*(N)+j ]);
                    /**/				// valeurs moyennes en haut et en bas
                }
                Gc[k*(N)*(M+1)+0*(N)+j]	= 0.5*(G[k*M*N+0*(N)+j]+G[k*M*N+(M-1)*(N)+j]);	// interface extreme bas
                Gc[k*(N)*(M+1)+M*(N)+j]	= 0.5*(G[k*M*N+0*(N)+j]+G[k*M*N+(M-1)*(N)+j]);	// interface extreme haut
                //if(GET_DEBUG__VOIR_VTK > 3) printf("Gc%d[%d,\t%d]=%f\n",k,0,j,Gc[k*(N)*(M+1)+	M*(N)+j ]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("Gc%d[%d,\t%d]=%f\n",k,M,j,Gc[k*(N)*(M+1)+	0*(N)+j ]);
            }
        }
        //debug();
		
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de Gc");
        
        
        //////////////////////////////////////////////
        // Calcul du vecteur des differences de flux G
        for (k = 0 ; k < 4 ; k++)
        {
            for (j = 0 ; j < N ; j++)
            {
                for (i = 1 ; i < M ; i++)
                {
                    deltaG[k*(N)*(M+1)+i*(N)+j]	= G[k*M*N+(i)*(N)+j]-G[k*M*N+(i-1)*(N)+j];
                    /**/			// valeur en haut MOINS valeur en bas
                    //if(GET_DEBUG__VOIR_VTK > 3) printf("deltaG%d[%d,\t%d]=%f\n",k,i,j,deltaG[k*(N)*(M+1)+	i*(N)+j ]);
                }
                deltaG[k*(N)*(M+1)+0*(N)+j]	= G[k*M*N+0*(N)+j]-G[k*M*N+(M-1)*(N)+j];
                deltaG[k*(N)*(M+1)+M*(N)+j]	= G[k*M*N+0*(N)+j]-G[k*M*N+(M-1)*(N)+j];
				
                //if(GET_DEBUG__VOIR_VTK > 3) printf("deltaG%d[%d,\t%d]=%f\n",k,0,j,deltaG[k*(N)*(M+1)+	0*(N)+j ]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("deltaG%d[%d,\t%d]=%f\n",k,M,j,deltaG[k*(N)*(M+1)+	M*(N)+j ]);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }	
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de deltaG");
        
        
        //////////////////////////////////////////////
        // Calcul matriciel
        /**/	// On doit calculer Phi_i = (G_i+1 + G_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (G_i+1 - G_i)/2
        for (i = 0 ; i < M+1 ; i++)
        {
            for (j = 0 ; j < N; j++)
            {
                /*			IR	= malloc(	16	  * sizeof(double));*/
				// variables temporaires du produit
                /*			df2	=	malloc(	4	  * sizeof(double));*/
                df2[0]	= deltaG[i*(N)+j];
                df2[1]	= deltaG[(N)*(M+1)+i*(N)+j];
                df2[2]	= deltaG[2*(N)*(M+1)+i*(N)+j];
                df2[3]	= deltaG[3*(N)*(M+1)+i*(N)+j];
                
                //printf("Produits : %d\n",j);
                produitmatriciel(&Ry[(i*(N)+j)*16], &SSy[(i*(N)+j)*16], 4, resu);
				// resu = R * diag(sign(lamba_k))
                produitmatriciel(resu, &Ly[(i*(N)+j)*16], 4, SG);
				// SG = R * diag(sign(lamba_k)) * R^{-1}
                produitvecteur(SG, df2, 4, PROD);
				// PROD =  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
                
                //debug();
                
                for (k = 0 ; k < 4 ; k++)
                {
                    Phiy[k*(N)*(M+1)+i*(N)+j]	= Gc[k*(N)*(M+1)+i*(N)+j]-0.5*PROD[k];
                    /**/			// On doit calculer Phi_i = (F_i+1 + F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
                }
            }
        }
        // Fin Calcul matriciel
        //////////////////////////////////////////////
        
        //debug();
        
        
        ////////////////////////////////////////////////////////////////
        // ACTUALISATION du vecteur des variables conservatives
        for (k = 0 ; k < 4 ; k++)
        {
            for (i = 0 ; i < M ; i++)
            {
                for (j = 0 ; j < N ; j++)
                {
                    U[k*M*N+i*N+j] -= (dt / hy)*(Phiy[k*(N)*(M+1)+(i+1)*(N)+j]-Phiy[k*(N)*(M+1)+i*(N)+j]);
                    /**/			// U_j^{n+1} = U_j^n + (dt / h) . Phi_{j+1}^n - Phi_j^n
                }
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }	
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de U n+1");
        //printf(",et selon Y.\n");
        
        
        //////////////////////////////////////////////////////////////////////////////
        // ACTUALISATION des variables conservatives en fonction du nouveau U
        for (i = 0 ; i < M ; i++)
        {
            for (j = 0 ; j < N ; j++)
            {
                rho[i*N+j ]	= U[i*N+j];	// rho = U_0
                u[i*N+j ]	= U[1*N*M+i*N+j]/U[0*N*M+i*N+j];	// u = U_1 / U_0
                v[i*N+j ]	= U[2*N*M+i*N+j]/U[0*N*M+i*N+j];	// v = U_2 / U_0
                p[i*N+j ]	= 0.4*(U[3*N*M+i*N+j]-0.5*U[0*N*M+i*N+j]*(u[i*N+j ]*u[i*N+j ]+v[i*N+j ]*v[i*N+j ]));
				// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
				// car U_4 = rho.E
				// E =  e + 0.5*||u^{->}||^2
                c[i*N+j ]	= sqrt(1.4*p[i*N+j]/rho[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,\t%d]=%f\n",i,j,rho[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,\t%d]=%f\n",i,j,u[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,\t%d]=%f\n",i,j,v[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("NORME(u,v)[%d,\t%d]=%.12f\n",i,j,norme(u[i*N+j], v[i*N+j]));
                //if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,\t%d]=%f\n",i,j,p[i*N+j]);
                //if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,\t%d]=%f\n",i,j,c[i*N+j]);
                //debug();
                //check_negatif((rho[i*(N)+j]), "rho n+1", i, j);
                //check_negatif((p[i*(N)+j]), "p n+1", i, j);
                //check_negatif((c[i*(N)+j]), "c n+1", i, j);
            }
            //if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
        }	
        
                
        //debugm(GET_DEBUG__VOIR_VTK, "Calcul de rho u v p c : n+1");
        //sleep(0.8);// on attend une second
        
    }
    //////////////////////////////////////////////////////////////////////////////////////////
    // FIN de la boucle en temps
    //////////////////////////////////////////////////////////////////////////////////////////
	printf("\nDONE : euler_2D_C : simulation terminee.\nHecth & Isnard & Besson (c) et (r) 2012\n");
	printf("  Les resultats ont ete ecrits dans deux listes de fichiers VTK :\n\
           Ecrire_valeurs_Euler_2D_C_*_p_[x|y].vtk  \t : pour la pression\n\
           Ecrire_valeurs_Euler_2D_C_*_rho_[x|y].vtk\t : pour la masse volumique\n\
           Ils peuvent etre visualises via le logiciel Paraview \n\
           recuperable a l'adresse suivante : [http://www.paraview.org/paraview/resources/software.php] \n\
           ou	[sudo apt-get install paraview]	sous Linux \n\
           \n");
    
	//debug();
    /*
     /////////////////////////
     // truc de frimeur
     // On libere la memoire allouee sur le CPU
     //
     free(x); free(y); free(U); free(Ubarx); free(Ubary); free(Phix); free(Phiy);
     free(F); free(Fc); free(deltaF); 
     free(G); free(Gc); free(deltaG); 
     
     free(u); free(v); free(p); free(c); free(rho);
     free(Rx); free(Lx); free(SSx);
     free(Ry); free(Ly); free(SSy);
     free(ubarx); free(vbarx); free(pbarx); free(cbarx); free(Hbary);
     free(ubary); free(vbary); free(pbary); free(cbary); free(Hbarx);
     free(resu); free(IR); free(SG); free(PROD); free(df2);
     
     free(nom_graphique);
     //////////////////////////////////
     */    
    fin =1; // on a fini 
    // la function de calcul 
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    printf("Fin du calcul ######\n");
    pthread_exit(0);//  fin du calcul
	
}


pthread_t threadCalcul;
struct  Global {    
    // une petite classe pour stoker toutes les variables globales
    int Width , Height;              // taille de l'écran en pixel
    bool stereo;
    bool windowdump;
    R rapz;
    int nbiso;
    R * viso; // le valeu de iso value
    void * Th;
    int N;
    int M;
    R * f ;                          // solution P1 a affich\'e
    R theta,phi,coef_dist;           // coordonnée polaire de la camera
    R dtheta;                        // vitesse de rotation de la camera 
    R xmin,xmax,ymin,ymax,zmin,zmax; // borne de la scène
    R xm,ym,zm; // point regarde
    R ceyes;
    R focal;
    int xold,yold;
    int wait_ms;// 
    int state; 
    
    
} *global ;  // la variable global
typedef struct  Global  Global;

void Global_SetView(Global * this,int eyes);           // define le point de vue
void Global_DefaultView(Global *);
void Global_MoveXView(Global *,R dx,R dy);
void Global_Affiche(Global *);

Global *Global_Global(void * TTh,R *ff,int NN,int MM,int height,int width,R rpz,int nbisovalue,bool st) 
{
    int i; 
    Global *this= (Global*)malloc(sizeof(Global));
    this->Th=TTh;
    this->N=NN;
    this->M=MM;
    this->f=ff;
    this->nbiso = nbisovalue;
    this->Width= width ;
    this->Height=height;
    this->rapz=rpz;
    this->windowdump=0;
    // first compute the mesh bound
    this->xmin=0;
    this->ymin=0;
    this->zmin=0;
    this->xmax=1;
    this->ymax=1;
    this->zmax=4;
    
    if(this->nbiso>2)
    {
        this->viso = (R *) malloc(sizeof(R)*this->nbiso);
        R diso=(this->zmax-this->zmin)/(this->nbiso-1);
        for (i=0;i<this->nbiso;i++)
            this->viso[i]=this->zmin+i*diso;
    }
    else 
        this->viso=0;
    
    Global_DefaultView(this);
    this->wait_ms= 1000/25; // 25 image par second 
    this->state=0; // etat no timer ..
    return this;
}

void TimerNextPlot(int value)
{
    global->state=0; // no timer 
    glutPostRedisplay(); 
}

void Global_DefaultView(Global *this) 
{
    this->coef_dist =300;
    //this->theta=45*pi/180.;
    this->theta=0.;
    this->dtheta= pi/1800; // 0.1 degree  si trop rapide
    this->focal = 30; // en degree
    this->ceyes = 0.05; // de l'objets
    this->coef_dist=0.28;
    //this->phi = 20*pi/180.;
    this->phi = pi/2;
    this->xm = (this->xmin+this->xmax)*0.5;
    this->ym = (this->ymin+this->ymax)*0.5;
    this->zm = this->rapz*(this->zmin+this->zmax)*0.5;    
}
void Global_SetView(Global *this,int eye) 
{
    glViewport( 0, 0, this->Width, this->Height );
    
    R ratio= (double) this->Width / (double)  this->Height; 
    R dx =(this->xmax-this->xmin), dy= (this->ymax-this->ymin), dz=(this->zmax-this->zmin)*this->rapz;
    R dmax= sqrt(dx*dx+dy*dy+dz*dz);
    R dist = dmax/2/asin(this->focal/2*pi/180.)*this->coef_dist;
    R camx=this->xm+cos(this->phi)*cos(this->theta)*dist;
    R camy=this->ym+cos(this->phi)*sin(this->theta)*dist;
    R camz=this->zm+dist*sin(this->phi);  
    R znear=max(dist-dmax/2,1e-5);
    R zfare=dist+dmax/2;
    R aspect=ratio;
    if (eye)
    {
        R dmm = -dmax*this->ceyes;
        R dx = -dmm*sin(this->theta);
        R dy = dmm*cos(this->theta);
        camx += dx*eye;
        camy += dy*eye;
    } 
    //  R hx= (  ratio*dy < dx  ) ? dx : dy*ratio ;
    //  R hy= hx/ratio ;
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity(); 
    gluLookAt(camx,camy,camz,this->xm,this->ym,this->zm,0.,0.,1.);
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity(); 
    gluPerspective(this->focal,aspect,znear,zfare);
}

void Global_MoveXView(Global* this,R dx,R dy)
{
    // cout << xm << " " << ym << " " << zm << " apres " ;
    this->zm -= dy*(this->zmax-this->zmin)*this->rapz/50.;
    this->xm += dx*(this->xmax-this->xmin)*sin(this->theta)/50;
    this->ym -= dx*(this->ymax-this->ymin)*cos(this->theta)/50;   
    // cout << xm << " " << ym << " " << zm << endl;
}

void hsvToRgb (float h, float s, float v, float * r, float * g, float * b)
{
    int i;
    float aa, bb, cc, f;
    
    if (s == 0) /* Grayscale */
        *r = *g = *b = v;
    else {
        if (h == 1.0) h = 0;
        h *= 6.0;
        i =  (int) h;
        f = h - i;
        aa = v * (1 - s);
        bb = v * (1 - (s * f));
        cc = v * (1 - (s * (1 - f)));
        switch (i) {
            case 0: *r = v;  *g = cc; *b = aa; break;
            case 1: *r = bb; *g = v;  *b = aa; break;
            case 2: *r = aa; *g = v;  *b = cc; break;
            case 3: *r = aa; *g = bb; *b = v;  break;
            case 4: *r = cc; *g = aa; *b = v;  break;
            case 5: *r = v;  *g = aa; *b = bb; break;
        }
    }
}

void   SetColor(R f)
{
    float r=0,g=0,b=0; 
    assert(global);
    R fmin=global->zmin; // borne de la fonction
    R fmax=global->zmax;
    
    hsvToRgb(0.99*(f-fmin)/(fmax-fmin),1,1,&r,&g,&b);
    glColor3f(r,g,b);
}

bool WindowDump(int width,int height)
{
	int i,j;
	FILE *fptr;
	static int counter = 0;
	char fname[32];
	unsigned char *image;
	
	/* Allocate our buffer for the image */
	if ((image = (unsigned char *) malloc(3*width*height)) == NULL){
		fprintf(stderr,"WindowDump - Failed to allocate memory for image\n");
		return(0);
	}
	
	/* Open the file */
	sprintf(fname,"L_%04d.ppm",counter);
	if ((fptr = fopen(fname,"w")) == NULL) {
		fprintf(stderr,"WindowDump - Failed to open file for window dump\n");
		return(0);
	}
	printf(" WindowDump in %s \n", fname);// << endl;
	/* Copy the image into our buffer */
	glReadBuffer(GL_FRONT);
	glReadPixels(0,0,width,height,GL_RGB,GL_UNSIGNED_BYTE,image);
	
	/* Write the PPM file */
	fprintf(fptr,"P6\n%d %d\n255\n",width,height);
	for (j=height-1;j>=0;j--) {
		for (i=0;i<width;i++) {
			fputc(image[3*j*width+3*i+0],fptr);
			fputc(image[3*j*width+3*i+1],fptr);
			fputc(image[3*j*width+3*i+2],fptr);
		}
	}
	fclose(fptr);
    
    free( image);
	counter++;
	return(1);
}


void Clean() 
{
    glClearColor(1.0, 1.0, 1.0, 0.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
}

static void Reshape( int width, int height )
{   
    global->Width  = width;
    global->Height = height;  
    glutPostRedisplay();
}
/*
int pickCase(int x,int y )
{
    
    GLdouble modelMatrix[16];
    GLdouble projMatrix[16];
    GLint viewport[4];  
    
    glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);
    ShowGlerror(" Get PM");
    glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);
    ShowGlerror(" Get MV");
    glGetIntegerv(GL_VIEWPORT,viewport);
    ShowGlerror(" Get VP");
    //int Width , Height; 
    GLdouble mx=x,my=global->Height-y,mz=0.0;
    GLdouble xx,yy,zz;
    
    GLint ok= gluUnProject( mx,my,mz,modelMatrix,projMatrix,viewport,&xx,&yy,&zz);
    if( (xx >=0) && (xx <1) && (yy >=0) && (yy <1) )
    {
        int i = floor(xx*global->N);
        int j = floor(yy*global->M);
        int k= i*global->N+j;

        if(1) 
            printf(" pick %d, %d ->  %g %g %g , %d\n",x,y, xx,yy,zz,k);
        return k;
    }
    else return -1; 
    
    
    
}
*/
int pickCase(int x,int y )
{
    GLdouble modelMatrix[16];
    GLdouble projMatrix[16];
    GLint viewport[4];  
    // for picking  ... 
    glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);
    ShowGlerror(" Get PM");
    glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);
    ShowGlerror(" Get MV");
    glGetIntegerv(GL_VIEWPORT,viewport);
    ShowGlerror(" Get VP");
    
    y = global->Height-y; 
    
    
    GLdouble mx=x,my=y,mz=1;
    GLdouble xx,yy,zz;
    GLdouble ox,oy,oz;
    GLdouble vx0,vy0,vz0;
    GLdouble vx1,vy1,vz1;
    GLdouble vx2,vy2,vz2;
    GLdouble vx3,vy3,vz3;
    GLint ok; 
    //if(1)
    {
        ok= gluProject( ox=0,oy=0,oz=0,modelMatrix,projMatrix,viewport,&vx0,&vy0,&vz0);
        ok= gluProject( ox=1,oy=0,oz=0,modelMatrix,projMatrix,viewport,&vx1,&vy1,&vz1);
        ok= gluProject( ox=0,oy=1,oz=0,modelMatrix,projMatrix,viewport,&vx2,&vy2,&vz2);
        ok= gluProject( ox=1,oy=1,oz=0,modelMatrix,projMatrix,viewport,&vx3,&vy3,&vz3);
        
        /*  printf(" pick 00->  %g %g %g   \n", vx0,vy0,vz0);
         printf(" pick 10 ->  %g %g %g  \n", vx1,vy1,vz1);
         printf(" pick 01 ->  %g %g %g  \n", vx2,vy2,vz2);
         printf(" pick 11 ->  %g %g %g  \n", vx3,vy3,vz3);
         */
        // resolution du systeme 2x2 : 
        double x0=x-vx0, x1=y-vy0;
        double u0 = vx1-vx0, u1=vy1-vy0;
        double v0 = vx2-vx0, v1=vy2-vy0;
        double det= u0*v1-v0*u1;
        xx  = (x0*v1-x1*v0)/det;
        yy  = (-x0*u1+x1*u0)/det;
        printf(" picking %f %f \n" , xx,yy);
        
    }
    //ok= gluUnProject( mx,my,mz,modelMatrix,projMatrix,viewport,&xx,&yy,&zz);
    //printf(" pick   %d, %d ->  %g %g %g   %d\n",x,y, xx,yy,zz,ok);
    // kkkkk =-1; 
    if( (xx >=0) && (xx <1) && (yy >=0) && (yy <1) )
    {
        int i = floor(xx*global->N);
        int j = floor(yy*global->M);
        int k= i*global->N+j;
        // kkkkk=k; 
        if(1) 
            printf(" pick %d, %d ->  %g %g %g , %d :: %d\n",x,y, xx,yy,zz,k,ok);
        return k;
    }
    else return -1; 
    
    
    
}

static void Key( unsigned char key, int x, int y )
{
    switch (key) {
        case 27: // esc char
            if(fin==1) {
                pthread_join(threadCalcul,0) ; // o attent la fin du calcul      
                exit(0);
            }                break;
        case '+':  
            global->coef_dist /= 1.2;
            break;
        case '-':  
            global->coef_dist *= 1.2;
            break;
        case 'g':  
            global->theta += pi/180.;
            break;
        case 'd':  
            global->theta -= pi/180.;
            break;
        case 'h':  
            global->phi += pi/180.;
            break;
        case 'b':  
            global->phi -= pi/180.;
            break;
        case 'a':
            global->dtheta = global->dtheta ? 0 : pi/1800.;
            break;	
        case 'A':
            global->dtheta *=1.2;
            break;	
        case 's':
            global->dtheta /= 1.2;
            break;	
        case 'f':
            global->focal /= 1.2 ;
            break;	
        case 'F':
            global->focal *= 1.2 ;
            break;	
        case 'e':
            global->ceyes /= 1.2 ;
            break;	
        case 'E':
            global->ceyes *= 1.2 ;
            break;
       // case 'p':
            //   cout << " ceyes = " << global->ceyes << " " << " focal = " << global->focal 
            // << " coef_dist " << global->coef_dist  << endl;
         //   break;
        case 'p':
        {
            int i,j;
            int kcase=pickCase(x,y);
            int j0 = kcase % (global->N);
            int i0 = (kcase -j0)/(global->N);
            if ( kcase = i0*global->N+j0) printf("YES \n");
            
            if (kcase >=0) {
                for (i=0; i< global->N; i++) {
                    for (j=0; j< global->M; j++) {
                        
                        if (norme_carre((i-i0)/(100.), (j-j0)/(100.))<carre(0.05))
                        {global->f[3*global->N*global->M+i*global->N+j] += 0.1/0.4;
                            global->f[i*global->N+j] -= 0.1;
                        }
                    }
                }
            } 
        }
            case 'w':
            global->windowdump=1;
            break;
        case '=':
            Global_DefaultView(global);
            break;
        case '?':
            printf( "coef_dist = %f \n focal %f \n Theta %f %d %d %d \n",global->coef_dist,global->focal,global->theta,x,y,pickCase(x,y));
            break;
        default:
            printf( " Key Character %d %c \n", (int) key , key) ; // << " " << key << endl;  
            printf(
                   "=  defautl \n"
                   "Ee  +/- ecart en les oeils\n"
                   "fF  +/ focal \n"
                   "s   slow \n"
                   "A    accelere \n"
                   "a   animation ou arret\n"
                   "bhgd  bas haut gauche droite \n"
                   "w     window dump \n"
                   "+-   zoom \n"
                   "(ESC) FIN \n")
            ;
            
            
    }
    glutPostRedisplay();
}

void Global_Affiche(Global*this)
{
   //glRenderMode( GL_SELECT ); 
    //glNewList(1,GL_COMPILE_AND_EXECUTE);
    glPolygonMode(GL_FRONT,GL_FILL); // mode affichage des polygones
    
    int NX = this->N, NY =this->M; // nb de pas en x et y 
    double x0 = 0, y0=0;
    double x1 =1, y1=1;
    double dx0 = (x1-x0)/NX; 
    double dy0 = (y1-y0)/NY; 
    double P[4][3]; 
    int ip[4] = { 0,1,1,0};
    int jp[4] = { 0,0,1,1};
    int i,j,k;
    for ( i=0;i<NX;++i)
        for (j=0;j<NY;++j)
        {
            for( k=0;k<4;++k)
            {
                P[k][0] = x0 + (i+ip[k]) * dx0;
                P[k][1] = y0 + (j+jp[k]) * dy0;
                P[k][2] = this->f[i*NX+j];
            }
            glLoadName( i*NX+j );
            glBegin(GL_POLYGON);
            for (k=0; k<4; ++k)
            {
                SetColor(this->f[i*NX+j]);
                glVertex3f(P[k][0],P[k][1],P[k][2]);
            }
            glEnd();
            
        }
}

void Display(void)
{ 
    
    if (global->stereo)
    {
        glClearColor(1.0, 1.0, 1.0, 0.0);
        glDrawBuffer(GL_BACK_RIGHT);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        Global_SetView(global,-1);
        //      glCallList(TheDrawList);    
        Global_Affiche(global);
        glClearColor(1.0, 1.0, 1.0, 0.0);
        glDrawBuffer(GL_BACK_LEFT);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        Global_SetView(global, +1);
        //glCallList(TheDrawList);
        Global_Affiche(global);
        glFlush();
        glutSwapBuffers();
    }
    else 
    {
        Clean();
        Global_SetView(global,0);
        //glCallList(TheDrawList);    
        Global_Affiche(global);
        glFlush();
        glutSwapBuffers();
        if ( global->windowdump)
            WindowDump(global->Width,global->Height);
        global->windowdump=0;
    }
    if(global->state==0) 
    {
        global->state=1; // a timer is load .. 
        glutTimerFunc(global->wait_ms,TimerNextPlot,1);
        
    }
}

int  ShowGlerror(const char *s)
{
    GLint error = glGetError();
    if ( error != GL_NO_ERROR )
	printf("Attention %s erreur : %x \n",s,error);   
    return error;   
}

static void Mouse( int button,int state,int x,int y )
{
    // state up or down 
    if (state == GLUT_DOWN) { global->xold=x,global->yold=y;return;}
    // cout << "Mouse " << button<< " " << state << " " << x-global->xold << " " << y-global->yold << endl;
    //  x gauche -> droite
    //  y  haut -> bas`
    //global->phi   += (y-global->yold)/(2.*180.);
    //global->theta -= (x-global->xold)/(2*180.);
    int i,j;
    int kcase=pickCase(x,y);
    int j0 = kcase % (global->N);
    int i0 = (kcase -j0)/(global->N);
    if ( kcase = i0*global->N+j0) printf("YES \n");
    
    if (kcase >=0) {
        for (i=0; i< global->N; i++) {
            for (j=0; j< global->M; j++) {
                
                if (norme_carre((i-i0)/(100.), (j-j0)/(100.))<carre(0.04))
                {global->f[3*global->N*global->M+i*global->N+j] += 0.5/0.4;
                //global->f[i*global->N+j] += 0.5;
                }
            }
        }
    } 
    
    glutPostRedisplay(); 
}
static void MotionMouse(int x,int y )
{
    // cout << " MotionMouse " << " " << x << " " << y << endl;
    //  GLuint gtime = glutGet(GLUT_ELAPSED_TIME); //   
    global->phi   += (y-global->yold)/(2.*180.);
    global->theta -= (x-global->xold)/(2*180.);
    global->xold=x;
    global->yold=y;
    int i,j;
    int kcase=pickCase(x,y);
    int j0 = kcase % (global->N);
    int i0 = (kcase -j0)/(global->N);
    if (kcase >=0) {
        for (i=0; i< global->N; i++) {
            for (j=0; j< global->M; j++) {
                
                if (norme_carre((i-i0)/(100.), (j-j0)/(100.))<carre(0.04))
                {global->f[3*global->N*global->M+i*global->N+j] += 0.3/0.4;
                    global->f[i*global->N+j] += 0.3;
                }
            }
        }
    } 

    glutPostRedisplay();
}
void SpecialKey(int key, int x, int y)
{
    // cout << " SpecialKey " << key << " " << x << " " << y << " : ";
    R dx=0.,dy=0.;
    switch (key) {
        case  GLUT_KEY_LEFT:   dx = -1; break;
        case  GLUT_KEY_RIGHT:  dx = +1; break;
        case  GLUT_KEY_DOWN:   dy = -1; break;
        case  GLUT_KEY_UP:     dy = +1; break;
    }
    // calcul du deplacement de xm,ym,zm;
    // cout << " " << dx << " " << dy << endl;
    Global_MoveXView(global,dx,dy);
    glutPostRedisplay();
    
}

pthread_attr_t tid;

    

	/////////////////////////////////////////////////////
	//
	//	Fin de la fonction Main_calcul
	//
	//////////////////////////////////////////////////////



//////////////////////////////////////
////   Fonction main
//////////////////////////////////////

int main(int argc, char** argv)
{
    bool stereo=0;
    bool fullscreen = 1;
    int k,kas=0;
    R rapz=1;
    int nbiso=20;
    
    
    
    for ( k=1;k < argc;k++)
    {
        if( strcmp("-s",argv[k])==0) 
        { stereo=1;}
        else if (strcmp("-f",argv[k])==0) 
        { fullscreen=1;}
        
    }
    
    
    glutInit(&argc, argv);
    rho = (double*) malloc(sizeof(R)*N*M);
    p = (double*) malloc(sizeof(R)*N*M);
    U	= (double*) malloc(	4*N*M	  * sizeof(double));
    
    fprintf(stderr," #### f = %p \n",U);
    
    
    int R = pthread_create(&threadCalcul,NULL,&main_calcul,U);
    if(R != 0)printf("Thread::Start: Thread could not be created %d\n",R);
    
    if(stereo)
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STEREO);
    else  
        glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
    int Height = 512;
    int Width = 512; 
    
    
    glutInitWindowSize(Width , Height);
    glutInitWindowPosition(100, 100);
    
    glutCreateWindow("gl plot iso ??? ");
    glutPushWindow();
    if (fullscreen)
        glutFullScreen();
    void * Th=0;
    
    global= Global_Global(Th,U,N,M,Height,Width,rapz,nbiso,stereo);
    
    glSelectBuffer( PICK_BUFFER_SIZE, PickBuffer );// setup the picking buffer
    glEnable(GL_DEPTH_TEST); 
    glutReshapeFunc( Reshape ); // pour changement de fenetre 
    glutKeyboardFunc( Key );    // pour les evenements clavier
    glutSpecialFunc(SpecialKey);
    glutMouseFunc(Mouse);       // pour les evenements sourie
    glutMotionFunc(MotionMouse); // les mouvements  de la sourie 
    glutDisplayFunc( Display ); // l'affichage
    glutMainLoop(); 
    return 0;
}



/* FIN
 *    Normalement, ce code marche sur n'importe quel machine.
 *
 *	Pour compiler : $> gcc -O3 -o euler_2D_C euler_2D_C.c -lm
 *	Pour executer : $> ./euler_2D_C <N> <M> <Borne>	// N : nb de points sur x, M : nb de points sur y, Borne : nb d'etapes max
 *	Pour visualiser : $> paraview
 *		Puis : ouvrir -> Ecrire_valeurs_Euler_2D_C_*_[p ? rho ?].vtk -> APPLY
 			-> Onglet Filters -> Data Analysis -> Plot Over Line
 			-> AXE X -> APPLY -> Play
 *
 *	Pour toutes questions, corrections, ou remarques, contactez nous :
 *	 maxime.isnard@ens-cachan.fr | lilian.besson@ens-cachan.fr | sophie.hecht@ens-cachan.fr
 */

////////////////////////////////////////////////////////////////
//	Maxime Isnard & Sophie Hecht & Lilian BESSON
//		 ENS de Cachan & LSV & CMLA - Mars-Avril 2012
//			CMLA
////////////////////////////////////////////////////////////////
