//////////////////////////////////////////////////
//  Equation d'Euler compressible 2D	CUDA
//	et différentes conditions aux limites
//////////////////////////////////////////////////

// version jeudi 28 juin 18h : version FINALE ! [tous les tests sont ok ?]
#define VERSION ("jeudi 28 juin 18h")

/* Ces codes sont librement distribués sous licence GPL
 * disponible ici <http://www.gnu.org/copyleft/gpl.html>
 */


////////////////////////////////////////////////////////////////
//	Maxime Isnard & Sophie Hecht & Lilian BESSON
//		 ENS de Cachan & LSV & CMLA - Juin 2012
////////////////////////////////////////////////////////////////

// Remarques et notations :
// 	- Les lignes ou paragraphes entourées de "//////////////////////////" sont pour structurer les étapes;
//	- Les paragraphes entre "/*" et "*/" sont importants;
//	- Les lignes commençant par "/**/" sont des justifications mathématiques correspondant aux calculs effectués juste avant;
//	- Les commentaires en fin de lignes commençant par "// " sont des explications simples;

/* DEBUT
 *    Normalement, ce code marche sur n'importe quel machine possédant une carte graphique nVidia et une installation valide de CUDA (v4.0+).
 *
 *	Pour compiler : $> nvcc -o Euler_2D_CUDA Euler_2D_CUDA.cu -lm
 *	Pour executer : $> ./Euler_2D_CUDA <N> <M> <Borne> **	// N : nb de points selon x, M : nb de points selon y, Borne : nb d'etapes max
 *	Pour visualiser : $> paraview (ou mpirun -np 1 paraview, qui permet d'utiliser plusieurs cœurs)
 *		Puis : ouvrir -> Euler_2DCUDA_**_*_[p ? rho ?]_[x ? y ? circ ?].vtk -> APPLY
 			-> Play
 *
 *	Pour toutes questions, corrections, ou remarques, contactez nous :
 *	 lilian.besson@ens-cachan.fr
 */

			///////////////////////////////
			// # include <bibliotheque.h>
#include <stdio.h>		// librairie standard d'input & d'output
#include <stdlib.h>		// librairie standard de C

#include <math.h>		// librairie pour utiliser les fonctions mathematiques.
#include <string.h>		// librairie pour utiliser les fonctions "plus elaborees" pour la gestion de chaines de caracteres
#include <time.h>		// librairie pour utiliser des fonctions de gestions du temps.

#include <assert.h>		// librairie qui fournit assert(booleen) pour faire du debug

#include <cuda.h>		// librairie de base du CUDA
#include <cuda_runtime.h>	// comme time mais pour les mesures en CUDA

			///////////////////////////////////
			// #define <macro>

// fonctions macro utiles
#define iMIN(a,b) 		( ((a) <= (b)) ? (a) : (b))	// min
#define iMAX(a,b) 		( ((a) <= (b)) ? (b) : (a))	// max
#define NORME(x,y) 		(sqrt((x)*(x)+(y)*(y)))

/* Coefficient CFL pour notre pas de temps. TODO : il faut que tout fonctionne avec une CFL de 0.95 et que tout diverge dès que CFL > 1.01 ! */
#define CFL 0.99

/* Coefficient GAMMA du gaz parfait modélisé */
#define GAMMA 1.4

/* Coefficient g du terme source */
#define coefficient_g 9.81

// savoir si on calcul le pas de temps à chaque étape (!=0) ou si on le fait une seule fois (=0).
#define CALCUL_DU_PAS_DE_TEMPS (1)

// pour une mini interface utilisateur (et une analyse des arguments de l'exécutable produit)
#define NOMBRE_MIN_ARGUMENTS		2
#define NOMBRE_MAX_ARGUMENTS		4

#define USAGE		("\n\
\033[01;31m\
Usage: ./Euler_2D_CUDA <N>, <M>, <borne>, <direction>, <limites_x>, <limites_y>, [<controle>]\n\
	effectue une simulation de l'equation d'Euler 1D avec \n\
		sur <N> points selon x, \n\
		sur <M> points selon y, \n\
		et en se limitant a <borne> etape de temps.\n\
		si <direction> est présent : les conditions initiales sont orientées selon x si direction=1 selon y si direction=0, circulairement si direction>=2.\n\
		si <direction> est présent, et si <limite_x> est présent et non nul, les conditions limites horizontales sont orientées : vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.\n\
		si <limite_x> est présent, et si <limite_y> est présent et non nul, les conditions limites verticales sont orientées : vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.\n\
		si <controle> est present et non nul, le systeme est en mode debug.\n\
		\n\
\n\
Le programme calcule une simulation de l'equation d'Euler compressible 2D, par la methode des volumes finis, via le langage CUDA;\n\
!!	Les conditions limites sont périodiques.\n\
\n\
Les resultats numeriques sont ecrits dans des fichiers VTK.\
\n\
par Maxime Isnard & Sophie Hecht & Lilian Besson © 2012 - ENS de Cachan & LSV & CMLA - Naereen Corp.\n\
\033[01;37m")	// il s'agit du «mode d'emplois»

/* ******************************** */
/* Valeurs par defaut des arguments */
#define	N__Cst	(40)	// Nombre de points selon x, par defaut = 5.
#define M__Cst	(40)	// Nombre de points selon y, par defaut = 5.
#define BORNE__Cst (50)	// Nombre d'etape de calcul, par defaut = 150.
#define DEBUG__VOIR_VTK__Cst (0)
				// vaut 1 si on veut afficher a l'ecran les valeurs ecrites dans les fichiers VTK,
				// et attendre l'utilisateur a chaque ecriture. 0 par defaut

#define CONDITION_PERIODIQUES_x__Cst (0) // selon x
		// vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 
		// 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.

#define CONDITION_PERIODIQUES_y__Cst (0) // selon y
		// vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 
		// 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.

/* Orientation des conditions initiales . TODO tester d'autres ! */
#define CONDITION_INITIALES__Cst	(2)
	/* Vaut 1 si selon x, 0 si selon y, circulaire baroque si = 2 */

			//////////////////////////////////////////////////
			//		Pour les arguments		//
			//////////////////////////////////////////////////

/* N */
//#ifdef __APPLE__
//#define GET_N() (N__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_N() (N__Cst)
//#else
#define GET_N() ( (argc > 1)? (atoi (argv[1])) : N__Cst )
//#endif

/* M */
//#ifdef __APPLE__
//#define GET_M() (M__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_M() (M__Cst)
//#else
#define GET_M() ( (argc > 2)? (atoi (argv[2])) : M__Cst )
//#endif

/* Borne */
//#ifdef __APPLE__
//#define GET_Borne() (BORNE__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_Borne() (BORNE__Cst)
//#else
#define GET_Borne() ( (argc > 3)? (atoi (argv[3])) : BORNE__Cst )
//#endif

/* Orientation des conditions initiales */
//#ifdef __APPLE__
//#define GET_CONDITION_INITIALES (CONDITION_INITIALES__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_CONDITION_INITIALES (CONDITION_INITIALES__Cst)
//#else
#define GET_CONDITION_INITIALES ( (argc > 4)? (atoi (argv[4])) : CONDITION_INITIALES__Cst )
//#endif

/* Parametre des conditions limites horizontales */
//#ifdef __APPLE__
//#define GET_CONDITION_PERIODIQUES_x (CONDITION_PERIODIQUES_x__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_CONDITION_PERIODIQUES_x (CONDITION_PERIODIQUES_x__Cst)
//#else
#define GET_CONDITION_PERIODIQUES_x ( (argc > 5)? (atoi (argv[5])) : CONDITION_PERIODIQUES_x__Cst )
//#endif

/* Parametre des conditions limites verticales */
//#ifdef __APPLE__
//#define GET_CONDITION_PERIODIQUES_y (CONDITION_PERIODIQUES_y__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_CONDITION_PERIODIQUES_y (CONDITION_PERIODIQUES_y__Cst)
//#else
#define GET_CONDITION_PERIODIQUES_y ( (argc > 6)? (atoi (argv[6])) : CONDITION_PERIODIQUES_y__Cst )
//#endif

/* Parametre de debug */
//#ifdef __APPLE__
//#define GET_DEBUG__VOIR_VTK (DEBUG__VOIR_VTK__Cst)
//#else
//#ifdef __WINDOWS__
//#define GET_DEBUG__VOIR_VTK (DEBUG__VOIR_VTK__Cst)
//#else
#define GET_DEBUG__VOIR_VTK ( (argc > 7)? (atoi (argv[7])) : DEBUG__VOIR_VTK__Cst )
//#endif

/* Variables de la parallélisation */
#define THREADS_X ( (argc > 8)? (atoi (argv[8])) : 16)
#define THREADS_Y ( (argc > 8)? (atoi (argv[8])) : 16)
//#define THREADS_Z ( (argc > 8)? (atoi (argv[8])) : 16)
#define THREADS_Z 1

#define GRIDS_X ( (argc > 9)? (atoi (argv[9])) : 32)
#define GRIDS_Y ( (argc > 9)? (atoi (argv[9])) : 32)
//#define GRIDS_Z ( (argc > 9)? (atoi (argv[9])) : 32)
#define GRIDS_Z 1

/* Parametre pour l'écriture des résultats : vaut 0 si on écrit pas, 1 si on écrit rho, 2 pour p, 3 pour rho et p */
//#ifdef __APPLE__
//#define GET_IMPRIMEVTK (0)
//#else
//#ifdef __WINDOWS__
//#define GET_IMPRIMEVTK (0)
//#else
#define GET_IMPRIMEVTK ( (argc > 10)? (atoi (argv[10])) : 0 )
//#endif

	///////////////////////////////////////////////////
	/* Formatage de texte dans la sortie en terminal */
#ifdef __APPLE__
#define VERT	("")
#define BLANC	("")
#define ROUGE	("")
#define BLEU	("")
#endif
#ifdef __WINDOWS__
#define VERT	("")
#define BLANC	("")
#define ROUGE	("")
#define BLEU	("")
#else
#define VERT	("\033[01;32m")
#define BLANC	("\033[01;37m")
#define ROUGE	("\033[01;31m")
#define BLEU	("\033[01;34m")
#endif
			//////////////////////////////////
			//	Fonctions CPU		//
			//////////////////////////////////

__host__ void print_vert(char* s){ printf(	"%s%s%s", VERT, s, BLANC); }
__host__ void print_rouge(char* s){ printf(	"%s%s%s", ROUGE, s, BLANC); }
__host__ void print_bleu(char* s){ printf(	"%s%s%s", BLEU, s, BLANC); }
__host__ void print_couleur(char* s, int codecouleurANSI){ printf(	"\033[01m;%d%s%s", codecouleurANSI, s, BLANC); }

__host__ void fprint_vert(FILE* file, char* s){ fprintf(file,	"%s%s%s", VERT, s, BLANC); }
__host__ void fprint_rouge(FILE* file, char* s){ fprintf(file,	"%s%s%s", ROUGE, s, BLANC); }
__host__ void fprint_bleu(FILE* file, char* s){ fprintf(file,	"%s%s%s", BLEU, s, BLANC); }
__host__ void fprint_couleur(FILE* file, char* s, int codecouleurANSI){ fprintf(file,	"\033[01m;%d%s%s", codecouleurANSI, s, BLANC); }

__host__ void print_endline_vert(char* s){ printf(	"%s%s%s\n", VERT, s, BLANC); }
__host__ void print_endline_rouge(char* s){ printf(	"%s%s%s\n", ROUGE, s, BLANC); }
__host__ void print_endline_bleu(char* s){ printf(	"%s%s%s\n", BLEU, s, BLANC); }
__host__ void print_endline_couleur(char* s, int codecouleurANSI){ printf(	"\033[01m;%d%s%s\n", codecouleurANSI, s, BLANC); }

__host__	char* nom_conditions_initiales(int direction){
		if (direction == 0) return "y";
		if (direction == 1) return "x";
		if (direction == 2) return "circ";
		return "Unknown";
}

__host__	char* nom_conditions_limites(int direction){
		if (direction == 0) return "periodiques";
		if (direction == 1) return "absorbantes";
		if (direction == 2) return "murPauvre";
		if (direction == 3) return "mur_paspauvre";
		if (direction == 4) return "mur_miroir";
		return "Unknown";
}

// Pour calculer l'indice d'une matrice.
//  Pour les conventions d'indicages, voir la doc.
__host__ int I2D(int i, int j, int taille){	return (i*taille + j);}

// Petite fonction pour mettre un point d'arrêt dans le programme
__host__ void debugm(int debugint, const char* message) {
	if(debugint > 5){
		printf("**************************************************************\n");
		printf("%s ? (-> Enter) ", message);
		getchar();
	}
}

			////////////////////////////////////////////////////////////////////////
			//	Prototypes de fonctions (les codes sont a la fin du programme)

// Fonction norme
__host__ float norme(float x, float y){	return sqrt(x*x + y*y);}

// Fonction carré
__host__ float carre(float x){	return (x*x);}

// Fonction norme au carré
__host__ float norme_carre(float x, float y){	return (x*x + y*y);}

	// Macro pour ecrire nos donnees dans un fichier .vtk [code a la fin]
__host__ void WriteVtk2D(char *nom, float *f, int N, int M, float hx, float hy, char* nom_variable, int debugint);

////////////////////////////////////////
// Un test qui affiche un vecteur
__host__ void test_en_affichant(int argc, const char * argv[], float* a, int N, int M, float hx, float hy, int it, char* NOMV){

	float* a_cpu;
	char* nom_graphique;

	a_cpu = (float*) malloc(N * M * sizeof(float));
	nom_graphique = (char*) malloc(400 * sizeof(char));
	cudaMemcpy( a_cpu, a, N*M	* sizeof(float), cudaMemcpyDeviceToHost );

	// On imprime a
	sprintf(nom_graphique, "Ecrire_valeurs_Euler_2D_cuda_%d_%s_%s.vtk", it, NOMV, ((GET_CONDITION_INITIALES == 0)? ("y") : ((GET_CONDITION_INITIALES == 1)? ("x") : ("circ"))));
	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [autres]...\n", it, nom_graphique);
	WriteVtk2D(nom_graphique, a_cpu, N, M, hx, hy, NOMV, GET_DEBUG__VOIR_VTK);
	fflush(stdout);
	free(a_cpu);
}

	//////////////////////////////////
	//	KERNELS CUDA		//
	//////////////////////////////////

// Fonction norme [Macro]
#define norme_cuda(x,y) (sqrt( (x)*(x) + (y)*(y) ))

// Fonction carré [Macro]
#define carre_cuda(x)	( (x)*(x) )

// Fonction norme au carré [Macro]
#define norme_carre_cuda(x,y) ( (x)*(x) + (y)*(y) )

// Fonction signe [Macro]
#define sgn_dev(x)	( ((x)<(0)) ? (-1.0) : ( ((x)>(0)) ? (1.0) : (0.0) ) )

			//////////////////////////////////////////////////////////
			// 		Calcul matriciel en CUDA		//
			//	est utilisé seulement pour des matrices 4*4	//
			//////////////////////////////////////////////////////////

// Pour calculer l'indice d'une matrice.
#define I2Ddev(i,j,taille) ( ( (i) * (taille) ) + (j) )

///////////////////////////////
// Produit Matrice * Matrice
//
// direction est une matrice *carree* de taille taille * taille
// A une matrice carree de taille taille * taille
__device__ void produitmatriciel_cuda(float* A, float* B, int taille, float* direction){
	int kloc, iloc, jloc;
//	iloc = threadIdx.y + blockIdx.y * blockDim.y;
//	jloc = threadIdx.x + blockIdx.x * blockDim.x;
//	if ((iloc < taille) && (jloc < taille)){		// cet essaie ne fonctionne pas (fonction __device__ appellées depuis le GPU, le threadIdx a déja été 'consommé')
	for(iloc = 0; iloc < taille; iloc++){
		for(jloc = 0; jloc < taille; jloc++){
//			direction[I2Ddev(iloc ,jloc ,taille)] = 0.0;
//			for(kloc = 0; kloc < taille; kloc++){
//				direction[I2Ddev(iloc ,jloc ,taille)] += (A[I2Ddev(iloc ,kloc ,taille)])*(B[I2Ddev(kloc ,jloc ,taille)]);
//			}
/* ! */			direction[I2Ddev(iloc ,jloc ,taille)] = (A[I2Ddev(iloc ,0 ,taille)])*(B[I2Ddev(0 ,jloc ,taille)]) + (A[I2Ddev(iloc ,1 ,taille)])*(B[I2Ddev(1 ,jloc ,taille)]) + (A[I2Ddev(iloc ,2 ,taille)])*(B[I2Ddev(2 ,jloc ,taille)]) + (A[I2Ddev(iloc ,3 ,taille)])*(B[I2Ddev(3 ,jloc ,taille)]) ;
		}
	}
}

///////////////////////////////
// Produit Matrice * Vecteur
//
// direction est un vecteur de taille taille
// A une matrice *carree* de taille taille * taille
__device__ void produitvecteur_cuda(float* A, float* X, int taille, float* direction){
	int iloc, jloc;
//	iloc = threadIdx.x + blockIdx.x * blockDim.x;
//	if (iloc < taille){
	for(iloc = 0; iloc < taille; iloc++){
//		direction[iloc] = 0.0;
//		for(jloc = 0; jloc < taille; jloc++){
//			direction[iloc] += (A[I2Ddev(iloc ,jloc ,taille)])*(X[jloc]);
//		}
/* ! */		direction[iloc] = (A[I2Ddev(iloc ,0 ,taille)])*(X[0]) + (A[I2Ddev(iloc ,1 ,taille)])*(X[1]) + (A[I2Ddev(iloc ,2 ,taille)])*(X[2]) + (A[I2Ddev(iloc ,3 ,taille)])*(X[3]);
	}
}
			/////////////////////////////////////////////////
			// 	FIN	Calcul matriciel CUDA
			/////////////////////////////////////////////////

///////////////////////////////////////
// Actualise le pas de temps.
// Selon la formule dt = CFL * min(hx, hy) * MAX ( |w| + c)
//	w = (u, v)
//	N * M est la taille de u,c,v,
/* sur le GPU mais en séquentiel */
__global__ void pas_de_temps_cuda_seq(float* pointeur_dt, float* u, float* c, float* v, int N, int M, float hx, float hy, int it){
	// 0.9 * h / max(abs(ubar)+cbar);
	float maxcalc, pasdetemps;
	int indice;

	indice = 0;
	maxcalc = 0.0;
	pasdetemps = 0.0002;

	while (indice < N*M ){
		if (maxcalc < ((norme_cuda(u[indice], v[indice])) + c[indice])) {
			maxcalc = ((norme_cuda(u[indice], v[indice])) + c[indice]);
		}
		indice += 1;
	 __syncthreads();	// au besoin
	}
	if (maxcalc != 0.0){pasdetemps = (CFL * (iMIN(hx,hy))) / maxcalc;}	// on se protège.
	else{
		pasdetemps = 0.0;
	}

	pointeur_dt[0] = pasdetemps;
		// pas de valeurs retournées par un kernel !
}

////////////////////////////////////////////////
// On utilise celui la : en séquentiel, pour ne pas être confronté aux problèmes de réduction
//
/* En CPU séquentiel0 */
__host__ float pas_de_temps_seq(float* u, float* c, float* v, int N, int M, float hx, float hy, int it){
	// CFL * h / max(abs(Wbar)+cbar);
	float maxcalc, pasdetemps;
	int indice, indice_i, indice_j;
	char reponse = '0';

//	printf("Calcul du pas de temps en séquentiel. it = %d. N = %d, M = %d.\n", it, N, M);
	float *u_cpu, *c_cpu, *v_cpu;
//	printf("Calcul de dt. 1.\t");

	u_cpu = (float*) malloc(N * M  * sizeof(float));
	c_cpu = (float*) malloc(N * M  * sizeof(float));
	v_cpu = (float*) malloc(N * M  * sizeof(float));
//	printf("Calcul de dt. 2.\t");

	cudaMemcpy( u_cpu, u, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
	cudaMemcpy( c_cpu, c, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
	cudaMemcpy( v_cpu, v, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
//	printf("Calcul de dt. 3.\t");

	indice = 0;
	maxcalc = 0.0;
	pasdetemps = 0.0002;
//	printf("Calcul de dt. 4.\nTapez 1 pour voir les détails, autre chose pour ne pas les voir : ?");

//	reponse = getchar();
	for (indice_i = 0 ; indice_i < M; indice_i ++) {
		for (indice_j = 0 ; indice_j < N ; indice_j ++){
			indice = (indice_i * N) + indice_j;
if (reponse == '1')			printf("Calcul de dt. 5 i=%d j=%d, indice = %d.\t", indice_i, indice_j, indice);
			if (maxcalc < ((norme_cuda(u_cpu[indice], v_cpu[indice])) + c_cpu[indice])) {
				maxcalc = ((norme_cuda(u_cpu[indice], v_cpu[indice])) + c_cpu[indice]);
			}
if (reponse == '1')			printf("Calcul de dt. 6 maxcalc = %f.\n", maxcalc);
		}
	}
	free(u_cpu);
	free(c_cpu);
	free(v_cpu);
	if (maxcalc != 0.0)	{	pasdetemps = (CFL * (iMIN(hx,hy))) / maxcalc;}	// on se protège.
	else			{	pasdetemps = 0.0;
		fprintf(stderr, "\nPas de temps foireux ! it=%d et maxcalc=%f\n", it, maxcalc);
		fflush(stderr);
		exit(2);
		}
//	printf("Calcul de dt finis .\n");
//	getchar();
	return pasdetemps;
}

////////////////////////////////////////////////////
// Vérifications Rx.Lx = I4_4
//
__host__ void	test_verif_I(int debugint, float* R, float* L, int it){

	int iloc, jloc, kloc;
	int taille = 4;
if(debugint > 2)		printf("***************************************************************\n TEST R*L = ? I_4*4\n");
	float *R_cpu, *L_cpu, *I_cpu;

	R_cpu = (float*) malloc(taille * taille  * sizeof(float));
	L_cpu = (float*) malloc(taille * taille  * sizeof(float));
	I_cpu = (float*) malloc(taille * taille  * sizeof(float));

	cudaMemcpy( R_cpu, R, taille * taille	* sizeof(float), cudaMemcpyDeviceToHost );
	cudaMemcpy( L_cpu, L, taille * taille	* sizeof(float), cudaMemcpyDeviceToHost );

	for(iloc = 0; iloc < taille; iloc++){
		for(jloc = 0; jloc < taille; jloc++){
			I_cpu[I2Ddev(iloc ,jloc ,taille)] = 0.0;
			for(kloc = 0; kloc < taille; kloc++){
				I_cpu[I2Ddev(iloc ,jloc ,taille)] += (R_cpu[I2Ddev(iloc ,kloc ,taille)])*(L_cpu[I2Ddev(kloc ,jloc ,taille)]);
			}	}	}
	for(iloc = 0; iloc < taille; iloc++){
		for(jloc = 0; jloc < taille; jloc++){
if(debugint > 2)	printf("I[%d,\t%d]=%f\n",iloc, jloc, I_cpu[I2Ddev(iloc, jloc, taille)]);
			if (iloc == jloc) {assert((1.1 > I_cpu[I2Ddev(iloc, jloc, taille)])&&(0.9 < I_cpu[I2Ddev(iloc, jloc, taille)]));};
			if (iloc != jloc) {assert((0.1 > I_cpu[I2Ddev(iloc, jloc, taille)])&&(-0.1 < I_cpu[I2Ddev(iloc, jloc, taille)]));};
//			assert((1.01 > I_cpu[I2Ddev(iloc, jloc, taille)])&&(-1.01 < I_cpu[I2Ddev(iloc, jloc, taille)]));
		}
if(debugint > 2)	printf("\n");
	}
if(debugint > 2)		printf("***************************************************************\n");
//	getchar();
	free(R_cpu);
	free(L_cpu);
	free(I_cpu);
}
/////////////////////////////////////////////////////

			//////////////////////////
			// Splitting en X	//
			//////////////////////////

////////////////////////////////////////////////////////
	// Actualisation des variables aux interfaces

__global__ void	calcul_Ubarx(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubarx, float* ubarx, float* vbarx, float* pbarx, float* cbarx, float* Hbarx, float* Phix, float* Rx, float* Lx, float* testid, float* SSx, float* F, float* Fc, float* deltaF, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j;
	int k;

	// Calcul de Ubarx
	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	j ++;

	while (i < M ){
		while (j < N ){
//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				Ubarx[i*(N+1)+(j)+k*(N+1)*M]=0.5*(U[i*N+(j)-1+k*M*N]+U[i*N+(j)+k*M*N]);	// valeur moyenne
			}
			j += blockDim.x * gridDim.x;
		}
//		if(k<4){
		for (k = 0 ; k < 4 ; k++){
			Ubarx[i*(N+1)+0+k*(N+1)*M]=0.5*(U[i*N+k*M*N]+U[i*N+N-1+k*M*N]);		// interface extreme gauche
			Ubarx[i*(N+1)+N+k*(N+1)*M]=0.5*(U[i*N+k*M*N]+U[i*N+N-1+k*M*N]);		// interface extreme droite
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}

__global__ void	calcul_uvpcH_barx_RLSSx(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubarx, float* ubarx, float* vbarx, float* pbarx, float* cbarx, float* Hbarx, float* Phix, float* Rx, float* Lx, float* testid, float* SSx, float* F, float* Fc, float* deltaF, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j;
	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	while (i < M ){
		while (j < N+1 ){
			ubarx[i*(N+1)+j]= Ubarx[1*(N+1)*M+i*(N+1)+j]/Ubarx[i*(N+1)+j];
/**/			// u = U[1] / U[0] = rho u / rho
			vbarx[i*(N+1)+j]= Ubarx[2*(N+1)*M+i*(N+1)+j]/Ubarx[i*(N+1)+j];
/**/			// v = U[2] / U[0] = rho v / rho
			pbarx[i*(N+1)+j]= 0.4*Ubarx[3*(N+1)*M+i*(N+1)+j]-0.5*0.4*Ubarx[0*(N+1)*M+i*(N+1)+j]*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]);
/**/			// p = (gamma - 1) * rho * e
			cbarx[i*(N+1)+j]= sqrt(1.4*(pbarx[i*(N+1)+j])/Ubarx[0*(N+1)*M+i*(N+1)+j]);
/**/			// c = racine(gamma * p / rho)
			Hbarx[i*(N+1)+j]= Ubarx[3*(N+1)*M+i*(N+1)+j]/Ubarx[0*(N+1)*M+i*(N+1)+j]+pbarx[i*(N+1)+j]/Ubarx[0*(N+1)*M+i*(N+1)+j];
/**/			// H = E + p / rho = rho E / rho + p / rho
/*******/
// TODO : optimiser le nombre d'acces mémoire de cette partie là : pour le moment ce n'est vraiment pas très propre !
/*******/
			////////////////////////////////////
			// Matrice R : valeurs propres de A
			Rx[0+	16*(i*(N+1)+j)] = 1.0;
			Rx[1+	16*(i*(N+1)+j)] = 1.0;
			Rx[2+	16*(i*(N+1)+j)] = 0.0;
			Rx[3+	16*(i*(N+1)+j)] = 1.0;

			Rx[4+	16*(i*(N+1)+j)] = ubarx[i*(N+1)+j]-cbarx[i*(N+1)+j];
			Rx[5+	16*(i*(N+1)+j)] = ubarx[i*(N+1)+j];
			Rx[6+	16*(i*(N+1)+j)] = 0.0;
			Rx[7+	16*(i*(N+1)+j)] = ubarx[i*(N+1)+j]+cbarx[i*(N+1)+j];

			Rx[8+	16*(i*(N+1)+j)] = vbarx[i*(N+1)+j];
			Rx[9+	16*(i*(N+1)+j)] = vbarx[i*(N+1)+j];
			Rx[10+	16*(i*(N+1)+j)] = 1.0;
			Rx[11+	16*(i*(N+1)+j)] = vbarx[i*(N+1)+j];

			Rx[12+	16*(i*(N+1)+j)] = Hbarx[i*(N+1)+j]-ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j];
			Rx[13+	16*(i*(N+1)+j)] = 0.5*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]);
			Rx[14+	16*(i*(N+1)+j)] = vbarx[i*(N+1)+j];
			Rx[15+	16*(i*(N+1)+j)] = Hbarx[i*(N+1)+j]+ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j];

			////////////////////////////////////
			// Matrice L : valeurs propres de A^t
			Lx[	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j]+0.4*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]-Hbarx[i*(N+1)+j])+ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j]);
			Lx[1+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*ubarx[i*(N+1)+j]-cbarx[i*(N+1)+j]);
			Lx[2+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*vbarx[i*(N+1)+j]);
			Lx[3+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*0.4;

			Lx[4+	16*(i*(N+1)+j)] = 0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(Hbarx[i*(N+1)+j]-(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]));
			Lx[5+	16*(i*(N+1)+j)] = 0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*ubarx[i*(N+1)+j];
			Lx[6+	16*(i*(N+1)+j)] = 0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(vbarx[i*(N+1)+j]);
			Lx[7+	16*(i*(N+1)+j)] = -0.4/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j]);

			Lx[8+	16*(i*(N+1)+j)] = -vbarx[i*(N+1)+j];
			Lx[9+	16*(i*(N+1)+j)] = 0.0;
			Lx[10+	16*(i*(N+1)+j)] = 1.0;
			Lx[11+	16*(i*(N+1)+j)] = 0.0;

			Lx[12+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j]+0.4*(ubarx[i*(N+1)+j]*ubarx[i*(N+1)+j]+vbarx[i*(N+1)+j]*vbarx[i*(N+1)+j]-Hbarx[i*(N+1)+j])-ubarx[i*(N+1)+j]*cbarx[i*(N+1)+j]);
			Lx[13+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*ubarx[i*(N+1)+j]+cbarx[i*(N+1)+j]);
			Lx[14+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*(-0.4*vbarx[i*(N+1)+j]);
			Lx[15+	16*(i*(N+1)+j)] = 0.5/(cbarx[i*(N+1)+j]*cbarx[i*(N+1)+j])*0.4;
    	// Calculs de R et L : faits
///////////////////////////////////////////

//////////////////////////////////////////////
	// Calcul de la matrice diagonale SS
	//  valeurs propres de A
			SSx[	16*(i*(N+1)+j)] = sgn_dev(ubarx[i*(N+1)+j]-cbarx[i*(N+1)+j]);	// valeurs propres de A
			SSx[1+	16*(i*(N+1)+j)] = 0.0;
			SSx[2+	16*(i*(N+1)+j)] = 0.0;
			SSx[3+	16*(i*(N+1)+j)] = 0.0;

			SSx[4+	16*(i*(N+1)+j)] = 0.0;
			SSx[5+	16*(i*(N+1)+j)] = sgn_dev(ubarx[i*(N+1)+j]);
			SSx[6+	16*(i*(N+1)+j)] = 0.0;
			SSx[7+	16*(i*(N+1)+j)] = 0.0;

			SSx[8+	16*(i*(N+1)+j)] = 0.0;
			SSx[9+	16*(i*(N+1)+j)] = 0.0;
			SSx[10+	16*(i*(N+1)+j)] = sgn_dev(ubarx[i*(N+1)+j]);
			SSx[11+	16*(i*(N+1)+j)] = 0.0;

			SSx[12+	16*(i*(N+1)+j)] = 0.0;
			SSx[13+	16*(i*(N+1)+j)] = 0.0;
			SSx[14+	16*(i*(N+1)+j)] = 0.0;
			SSx[15+	16*(i*(N+1)+j)] = sgn_dev(ubarx[i*(N+1)+j]+cbarx[i*(N+1)+j]);

			j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
	// fin RLSSx
}

//////////////////////////////////////////////
	// Calcul du vecteur des flux F
__global__ void	calcul_F(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubarx, float* ubarx, float* vbarx, float* pbarx, float* cbarx, float* Hbarx, float* Phix, float* Rx, float* Lx, float* testid, float* SSx, float* F, float* Fc, float* deltaF, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j;
	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	while (i < M ){
		while (j < N ){
			F[i*N+j]		= rho[i*N+j]*u[i*N+j];
			F[N*M+i*N+j]		= rho[i*N+j]*u[i*N+j]*u[i*N+j]+p[i*N+j];
			F[2*N*M+i*N+j]		= rho[i*N+j]*v[i*N+j]*u[i*N+j];
			F[3*N*M+i*N+j]		= (U[3*N*M+i*N+j]+p[i*N+j])*u[i*N+j];
			j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}

//////////////////////////////////////////////
	// Calcul du vecteur des flux F centres
__global__ void	calcul_Fc_deltaF(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubarx, float* ubarx, float* vbarx, float* pbarx, float* cbarx, float* Hbarx, float* Phix, float* Rx, float* Lx, float* testid, float* SSx, float* F, float* Fc, float* deltaF, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt, int condition_limites_x){

	int i; int j; int k;
	float pression_speciale;

	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	j ++;

	while (i < M ){
		while (j < N ){
//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				Fc[k*(N+1)*M+i*(N+1)+(j)]	= 0.5*(F[k*M*N+i*N+(j)-1]+F[k*M*N+i*N+(j)]);
				deltaF[k*(N+1)*M+i*(N+1)+(j)]	= F[k*M*N+i*N+(j)]-F[k*M*N+i*N+(j)-1];
			}
/**/				// valeurs moyennes aux interfaces gauche et droite
			j += blockDim.x * gridDim.x;
		}
//		if(k<4){
		for (k = 0 ; k < 4 ; k++){
		
/* NEW : on a le choix de nombreuses conditions limites */
			switch (condition_limites_x)
			{
			case 0:		// limites périodiques
			{
				Fc[k*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[k*N*M + i*N + 0] + F[k*N*M + i*N + (N-1)]);	// interface extreme gauche
				Fc[k*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[k*N*M + i*N + 0] + F[k*N*M + i*N + (N-1)]);	// interface extreme droite (FN !)
				deltaF[k*(N + 1)*M + i*(N + 1) + 0]	= F[k*N*M + i*N + 0]-F[k*N*M + i*N + (N-1)];
				deltaF[k*(N + 1)*M + i*(N + 1) + N]	= F[k*N*M + i*N + 0]-F[k*N*M + i*N + (N-1)];
				break;
			}
			case 1:		// limites absorbantes
			{
				Fc[k*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[k*N*M + i*N + 0] + F[k*N*M + i*N + 0]);
				Fc[k*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[k*N*M + i*N + (N-1)] + F[k*N*M + i*N + (N-1)]);
				
				deltaF[k*(N + 1)*M + i*(N + 1) + 0]	= F[k*N*M + i*N + 0]-F[k*N*M + i*N + 0];	// oui ca vaut 0
				deltaF[k*(N + 1)*M + i*(N + 1) + N]	= F[k*N*M + i*N + (N-1)]-F[k*N*M + i*N + (N-1)];	// oui ca vaut 0
				break;
			}
			case 2:		// limites mur stratégie pauvre
			{
				pression_speciale = p[i*N + 0];
					
					Fc[0*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[0*N*M + i*N + 0] + 0.0);
					Fc[1*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[1*N*M + i*N + 0] + pression_speciale);
					Fc[2*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[2*N*M + i*N + 0] + 0.0);
					Fc[3*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[3*N*M + i*N + 0] + 0.0);

					deltaF[0*(N + 1)*M + i*(N + 1) + 0]	= F[0*N*M + i*N + 0] - 0.0;
					deltaF[1*(N + 1)*M + i*(N + 1) + 0]	= F[1*N*M + i*N + 0] - pression_speciale;
					deltaF[2*(N + 1)*M + i*(N + 1) + 0]	= F[2*N*M + i*N + 0] - 0.0;
					deltaF[3*(N + 1)*M + i*(N + 1) + 0]	= F[3*N*M + i*N + 0] - 0.0;

				pression_speciale = p[i*N + (N-1)];

					Fc[0*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[0*N*M + i*N + (N-1)] + 0.0);
					Fc[1*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[1*N*M + i*N + (N-1)] + pression_speciale);
					Fc[2*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[2*N*M + i*N + (N-1)] + 0.0);
					Fc[3*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[3*N*M + i*N + (N-1)] + 0.0);

					deltaF[0*(N + 1)*M + i*(N + 1) + N]	= - F[0*N*M + i*N + (N-1)] + 0.0;
					deltaF[1*(N + 1)*M + i*(N + 1) + N]	= - F[1*N*M + i*N + (N-1)] + pression_speciale;
					deltaF[2*(N + 1)*M + i*(N + 1) + N]	= - F[2*N*M + i*N + (N-1)] + 0.0;
					deltaF[3*(N + 1)*M + i*(N + 1) + N]	= - F[3*N*M + i*N + (N-1)] + 0.0;

				break;
			}
			case 3:		// limites mur stratégie pas pauvre (papier JM.Ghidaglia F.Pascal, page 58 (10.54))
			{
				pression_speciale = p[i*N + 0]*(1 + (u[i*N + 0]*GAMMA)/(c[i*N + 0] - (GAMMA-1)*u[i*N + 0]));
					
					Fc[0*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[0*N*M + i*N + 0] + 0.0);
					Fc[1*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[1*N*M + i*N + 0] + pression_speciale);
					Fc[2*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[2*N*M + i*N + 0] + 0.0);
					Fc[3*(N + 1)*M + i*(N + 1) + 0]	= 0.5*(F[3*N*M + i*N + 0] + 0.0);

					deltaF[0*(N + 1)*M + i*(N + 1) + 0]	= F[0*N*M + i*N + 0] - 0.0;
					deltaF[1*(N + 1)*M + i*(N + 1) + 0]	= F[1*N*M + i*N + 0] - pression_speciale;
					deltaF[2*(N + 1)*M + i*(N + 1) + 0]	= F[2*N*M + i*N + 0] - 0.0;
					deltaF[3*(N + 1)*M + i*(N + 1) + 0]	= F[3*N*M + i*N + 0] - 0.0;
				
				pression_speciale = p[i*N + (N-1)]*(1 + (u[i*N + (N-1)]*GAMMA)/(c[i*N + (N-1)] - (GAMMA-1)*u[i*N + (N-1)]));

					Fc[0*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[0*N*M + i*N + (N-1)] + 0.0);
					Fc[1*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[1*N*M + i*N + (N-1)] + pression_speciale);
					Fc[2*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[2*N*M + i*N + (N-1)] + 0.0);
					Fc[3*(N + 1)*M + i*(N + 1) + N]	= 0.5*(F[3*N*M + i*N + (N-1)] + 0.0);

					deltaF[0*(N + 1)*M + i*(N + 1) + N]	= - F[0*N*M + i*N + (N-1)] + 0.0;
					deltaF[1*(N + 1)*M + i*(N + 1) + N]	= - F[1*N*M + i*N + (N-1)] + pression_speciale;
					deltaF[2*(N + 1)*M + i*(N + 1) + N]	= - F[2*N*M + i*N + (N-1)] + 0.0;
					deltaF[3*(N + 1)*M + i*(N + 1) + N]	= - F[3*N*M + i*N + (N-1)] + 0.0;
				break;
			}
			}
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}



//////////////////////////////////////////////
	// Calcul matriciel
/**/	// On doit calculer Phi_i = (F_i+1 + F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
__global__ void	calcul_Phix(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubarx, float* ubarx, float* vbarx, float* pbarx, float* cbarx, float* Hbarx, float* Phix, float* Rx, float* Lx, float* testid, float* SSx, float* F, float* Fc, float* deltaF, float* df2x, float* resux, float* SGx, float* PRODx, int  N, int  M, float hx, float hy, float dt){

	int i; int j; int k;

	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	while (i < M ){
		while (j < N+1 ){
			df2x[0+(i*(N+1)+j)*4]	= deltaF[i*(N+1)+j];
			df2x[1+(i*(N+1)+j)*4]	= deltaF[(N+1)*M+i*(N+1)+j];
			df2x[2+(i*(N+1)+j)*4]	= deltaF[2*(N+1)*M+i*(N+1)+j];
			df2x[3+(i*(N+1)+j)*4]	= deltaF[3*(N+1)*M+i*(N+1)+j];

			produitmatriciel_cuda(&Rx[(i*(N+1)+j)*16], &SSx[(i*(N+1)+j)*16], 4, &resux[(i*(N+1)+j)*16]);
				// resu = R * diag(sign(lamba_k))
			produitmatriciel_cuda(&resux[(i*(N+1)+j)*16], &Lx[(i*(N+1)+j)*16], 4, &SGx[(i*(N+1)+j)*16]);
				// SG = R * diag(sign(lamba_k)) * R^{-1}
			produitvecteur_cuda(&SGx[(i*(N+1)+j)*16], &df2x[(i*(N+1)+j)*4], 4, &PRODx[(i*(N+1)+j)*4]);
				// PROD =  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2

//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				Phix[k*(N+1)*M+i*(N+1)+j]	= Fc[k*(N+1)*M+i*(N+1)+j]-0.5*PRODx[k+(i*(N+1)+j)*4];
/**/				// On doit calculer Phi_i = (F_i+1 + F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
			}
			j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}
	// Fin Calcul matriciel
//////////////////////////////////////////////

////////////////////////////////////////////////////////////////
	// ACTUALISATION du vecteur des variables conservatives
__global__ void	calcul_U_x_rhouvpc_x(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubarx, float* ubarx, float* vbarx, float* pbarx, float* cbarx, float* Hbarx, float* Phix, float* Rx, float* Lx, float* testid, float* SSx, float* F, float* Fc, float* deltaF, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j; int k;

	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	while (i < M ){
		while (j < N ){
//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				U[k*M*N+i*N+j] -= (dt / hx)*(Phix[k*(N+1)*M+i*(N+1)+j+1]-Phix[k*(N+1)*M+i*(N+1)+j]);
			}
/**/			// U_j^{n+1} = U_j^n -0 (dt / h) . Phi_{j+1}^n - Phi_j^n
			rho[i*N+j]	= U[0*N*M+i*N+j];	// rho = U_0
			u[i*N+j]	= U[1*N*M+i*N+j]/U[0*N*M+i*N+j];	// u = U_1 / U_0
			v[i*N+j]	= U[2*N*M+i*N+j]/U[0*N*M+i*N+j];	// v = U_2 / U_0
			p[i*N+j]	= 0.4*(U[3*N*M+i*N+j]-0.5*U[0*N*M+i*N+j]*(u[i*N+j]*u[i*N+j]+v[i*N+j]*v[i*N+j]));
				// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
				// car U_4 = rho.E
				// E =  e + 0.5*||u^{->}||^2
			c[i*N+j]	= sqrt(1.4*p[i*N+j]/rho[i*N+j]);
			j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}
////////////////////////////////////////////////////

////////////////////////////////////////////////////
// SPLITTING en Y

	////////////////////////////////////////////////////////
	// Actualisation des variables aux interfaces

	// Calcul de Ubary
__global__ void	calcul_Ubary(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubary, float* ubary, float* vbary, float* pbary, float* cbary, float* Hbary, float* Phiy, float* Ry, float* Ly, float* testid, float* SSy, float* G, float* Gc, float* deltaG, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j; int k;

	j = threadIdx.x + blockIdx.x * blockDim.x;
	i = threadIdx.y + blockIdx.y * blockDim.y;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	i++;

	while (j < N){
		while (i < M ){
//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				Ubary[(i)*N+j+k*N*(M+1)]=0.5*(U[(i)*N+j+k*M*N]+U[((i)-1)*N+j+k*M*N]);	// valeur moyenne
			}
			i += blockDim.y * gridDim.y;
			}
//		if(k<4){
		for (k = 0 ; k < 4 ; k++){
			Ubary[0*N+j+k*N*(M+1)]=0.5*(U[0*N+j+k*M*N]+U[(M-1)*N+j+k*M*N]);		// interface extreme bas
			Ubary[M*N+j+k*N*(M+1)]=0.5*(U[0*N+j+k*M*N]+U[(M-1)*N+j+k*M*N]);		// interface extreme haut
		}
		j += blockDim.x * gridDim.x;
//		i = threadIdx.y + blockIdx.y * blockDim.y;
	}
}

	// Calcul de ubary
__global__ void	calcul_uvpcH_bary_RLSSy(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubary, float* ubary, float* vbary, float* pbary, float* cbary, float* Hbary, float* Phiy, float* Ry, float* Ly, float* testid, float* SSy, float* G, float* Gc, float* deltaG, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j;
	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	while (i < M+1){
		while (j < N ){
			ubary[i*N+j]= Ubary[1*N*(M+1)+i*N+j]/Ubary[0*N*(M+1)+i*N+j];
/**/			// u = U[1] / U[0] = rho u / rho
			vbary[i*N+j]= Ubary[2*N*(M+1)+i*N+j]/Ubary[0*N*(M+1)+i*N+j];
/**/			// v = U[2] / U[0] = rho v / rho
			pbary[i*N+j]= 0.4*Ubary[3*N*(M+1)+i*N+j]-0.5*0.4*Ubary[0*N*(M+1)+i*N+j]*(ubary[i*N+j]*ubary[i*N+j]+vbary[i*N+j]*vbary[i*N+j]);
/**/			// p = (gamma - 1) * rho * e
			cbary[i*N+j]= sqrt(1.4*pbary[i*N+j]/Ubary[i*N+j]);
/**/			// c = racine(gamma * p / rho)
			Hbary[i*N+j]= Ubary[3*N*(M+1)+i*N+j]/Ubary[0*N*(M+1)+i*N+j]+pbary[i*N+j]/Ubary[0*N*(M+1)+i*N+j];
/**/			// H = E + p / rho = rho E / rho + p / rho
			////////////////////////////////////
			// Matrice R : valeurs propres de A
			Ry[0+	16*(i*N+j)] = 1.0;
			Ry[1+	16*(i*N+j)] = 1.0;
			Ry[2+	16*(i*N+j)] = 0.0;
			Ry[3+	16*(i*N+j)] = 1.0;

			Ry[4+	16*(i*N+j)] = ubary[i*N+j];
			Ry[5+	16*(i*N+j)] = ubary[i*N+j];
			Ry[6+	16*(i*N+j)] = 1.0;
			Ry[7+	16*(i*N+j)] = ubary[i*N+j];

			Ry[8+	16*(i*N+j)] = vbary[i*N+j]-cbary[i*N+j];
			Ry[9+	16*(i*N+j)] = vbary[i*N+j];
			Ry[10+	16*(i*N+j)] = 0.0;
			Ry[11+	16*(i*N+j)] = vbary[i*N+j]+cbary[i*N+j];

			Ry[12+	16*(i*N+j)] = Hbary[i*N+j]-vbary[i*N+j]*cbary[i*N+j];
			Ry[13+	16*(i*N+j)] = 0.5*(ubary[i*N+j]*ubary[i*N+j]+vbary[i*N+j]*vbary[i*N+j]);
			Ry[14+	16*(i*N+j)] = ubary[i*N+j];
			Ry[15+	16*(i*N+j)] = Hbary[i*N+j]+vbary[i*N+j]*cbary[i*N+j];

			////////////////////////////////////
			// Matrice L : valeurs propres de A^t
			Ly[	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*(cbary[i*N+j]*cbary[i*N+j]+0.4*(ubary[i*N+j]*ubary[i*N+j]+vbary[i*N+j]*vbary[i*N+j]-Hbary[i*N+j])+vbary[i*N+j]*cbary[i*N+j]);
			Ly[1+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*(-0.4*ubary[i*N+j]);
			Ly[2+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*(-0.4*vbary[i*N+j]-cbary[i*N+j]);
			Ly[3+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*0.4;

			Ly[4+	16*(i*N+j)] = (0.4/(cbary[i*N+j]*cbary[i*N+j]))*(Hbary[i*N+j]-(ubary[i*N+j]*ubary[i*N+j]+vbary[i*N+j]*vbary[i*N+j]));
			Ly[5+	16*(i*N+j)] = (0.4/(cbary[i*N+j]*cbary[i*N+j]))*ubary[i*N+j];
			Ly[6+	16*(i*N+j)] = (0.4/(cbary[i*N+j]*cbary[i*N+j]))*(vbary[i*N+j]);
			Ly[7+	16*(i*N+j)] = -(0.4/(cbary[i*N+j]*cbary[i*N+j]));

			Ly[8+	16*(i*N+j)] = -ubary[i*N+j];
			Ly[9+	16*(i*N+j)] = 1.0;
			Ly[10+	16*(i*N+j)] = 0.0;
			Ly[11+	16*(i*N+j)] = 0.0;

			Ly[12+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*(cbary[i*N+j]*cbary[i*N+j]+0.4*(ubary[i*N+j]*ubary[i*N+j]+vbary[i*N+j]*vbary[i*N+j]-Hbary[i*N+j])-vbary[i*N+j]*cbary[i*N+j]);
			Ly[13+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*(-0.4*ubary[i*N+j]);
			Ly[14+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*(-0.4*vbary[i*N+j]+cbary[i*N+j]);
			Ly[15+	16*(i*N+j)] = (0.5/(cbary[i*N+j]*cbary[i*N+j]))*0.4;
	// Calculs de R et L : faits
///////////////////////////////////////////

//////////////////////////////////////////////
	// Calcul de la matrice diagonale SS
	//  valeurs propres de A
			SSy[	16*(i*N+j)] = sgn_dev(vbary[i*N+j]-cbary[i*N+j]);	// valeurs propres de A
			SSy[1+	16*(i*N+j)] = 0.0;
			SSy[2+	16*(i*N+j)] = 0.0;
			SSy[3+	16*(i*N+j)] = 0.0;

			SSy[4+	16*(i*N+j)] = 0.0;
			SSy[5+	16*(i*N+j)] = sgn_dev(vbary[i*N+j]);
			SSy[6+	16*(i*N+j)] = 0.0;
			SSy[7+	16*(i*N+j)] = 0.0;

			SSy[8+	16*(i*N+j)] = 0.0;
			SSy[9+	16*(i*N+j)] = 0.0;
			SSy[10+	16*(i*N+j)] = sgn_dev(vbary[i*N+j]);
			SSy[11+	16*(i*N+j)] = 0.0;

			SSy[12+	16*(i*N+j)] = 0.0;
			SSy[13+	16*(i*N+j)] = 0.0;
			SSy[14+	16*(i*N+j)] = 0.0;
			SSy[15+	16*(i*N+j)] = sgn_dev(vbary[i*N+j]+cbary[i*N+j]);

		 	j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}

//////////////////////////////////////////////
	// Calcul du vecteur des flux G
__global__ void	calcul_G(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubary, float* ubary, float* vbary, float* pbary, float* cbary, float* Hbary, float* Phiy, float* Ry, float* Ly, float* testid, float* SSy, float* G, float* Gc, float* deltaG, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j;
	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	while (i < M ){
		while (j < N ){
			G[i*N+j]		= rho[i*N+j]*v[i*N+j];
			G[N*M+i*N+j]		= rho[i*N+j]*v[i*N+j]*u[i*N+j];
			G[2*N*M+i*N+j]		= rho[i*N+j]*v[i*N+j]*v[i*N+j]+p[i*N+j];
			G[3*N*M+i*N+j]		= (U[3*N*M+i*N+j]+p[i*N+j])*v[i*N+j];
		 	j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}

//////////////////////////////////////////////
	// Calcul du vecteur des flux G centres
__global__ void	calcul_Gc_deltaG(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubary, float* ubary, float* vbary, float* pbary, float* cbary, float* Hbary, float* Phiy, float* Ry, float* Ly, float* testid, float* SSy, float* G, float* Gc, float* deltaG, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt, int condition_limites_y){

	int i; int j; int k;
	float pression_speciale;

	j = threadIdx.x + blockIdx.x * blockDim.x;
	i = threadIdx.y + blockIdx.y * blockDim.y;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	i++;

	while (j < N){
		while (i < M ){
//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				Gc[k*N*(M+1)+(i)*N+j]	= 0.5*(G[k*M*N+(i)*N+j]+G[k*M*N+((i)-1)*N+j]);
				deltaG[k*N*(M+1)+(i)*N+j]	= G[k*M*N+(i)*N+j]-G[k*M*N+((i)-1)*N+j];
			}
/**/				// valeurs moyennes en haut et en bas
	 		i += blockDim.y * gridDim.y;
		}
//		if(k<4){
		for (k = 0 ; k < 4 ; k++){
/* NEW : on a le choix de nombreuses conditions limites */
			switch (condition_limites_y)
			{
			case 0:		// limites périodiques
			{
				Gc[k*N*(M + 1) + 0*N + j]	= 0.5*(G[k*N*M + 0*N + j] + G[k*N*M + (M-1)*N + j]);	// interface extreme bas
				Gc[k*N*(M + 1) + M*N + j]	= 0.5*(G[k*N*M + 0*N + j] + G[k*N*M + (M-1)*N + j]);	// interface extreme haut (FN !)
				deltaG[k*N*(M + 1) + 0*N + j]	= G[k*N*M + 0*N + j] - G[k*N*M + (M-1)*N + j];
				deltaG[k*N*(M + 1) + M*N + j]	= G[k*N*M + 0*N + j] - G[k*N*M + (M-1)*N + j];
				break;
			}
			case 1:		// limites absorbantes
			{
				Gc[k*N*(M + 1) + 0*N + j]	= 0.5*(G[k*N*M + 0*N + j] + G[k*N*M + 0*N + j]);	// interface extreme bas
				Gc[k*N*(M + 1) + M*N + j]	= 0.5*(G[k*N*M + (M-1)*N + j] + G[k*N*M + (M-1)*N + j]);	// interface extreme haut (FN !)
				deltaG[k*N*(M + 1) + 0*N + j]	= G[k*N*M + 0*N + j] - G[k*N*M + 0*N + j];	// oui ca vaut 0
				deltaG[k*N*(M + 1) + M*N + j]	= G[k*N*M + (M-1)*N + j] - G[k*N*M + (M-1)*N + j];	// oui ca vaut 0
				break;
			}
			case 2:		// limites mur stratégie pauvre
			{
				pression_speciale = p[0*N + j];
					
					Gc[0*N*(M + 1) + 0*N + j]	= 0.5*(G[0*N*M + 0*N + j] + 0.0);
					Gc[1*N*(M + 1) + 0*N + j]	= 0.5*(G[1*N*M + 0*N + j] + 0.0);
					Gc[2*N*(M + 1) + 0*N + j]	= 0.5*(G[2*N*M + 0*N + j] + pression_speciale);
					Gc[3*N*(M + 1) + 0*N + j]	= 0.5*(G[3*N*M + 0*N + j] + 0.0);

					deltaG[0*N*(M + 1) + 0*N + j]	= G[0*N*M + 0*N + j] - 0.0;
					deltaG[1*N*(M + 1) + 0*N + j]	= G[1*N*M + 0*N + j] - 0.0;
					deltaG[2*N*(M + 1) + 0*N + j]	= G[2*N*M + 0*N + j] - pression_speciale;
					deltaG[3*N*(M + 1) + 0*N + j]	= G[3*N*M + 0*N + j] - 0.0;

				pression_speciale = p[(M-1)*N + j];

					Gc[0*N*(M + 1) + M*N + j]	= 0.5*(G[0*N*M + (M-1)*N + j] + 0.0);
					Gc[1*N*(M + 1) + M*N + j]	= 0.5*(G[1*N*M + (M-1)*N + j] + 0.0);
					Gc[2*N*(M + 1) + M*N + j]	= 0.5*(G[2*N*M + (M-1)*N + j] + pression_speciale);
					Gc[3*N*(M + 1) + M*N + j]	= 0.5*(G[3*N*M + (M-1)*N + j] + 0.0);

					deltaG[0*N*(M + 1) + M*N + j]	= - G[0*N*M + (M-1)*N + j] + 0.0;
					deltaG[1*N*(M + 1) + M*N + j]	= - G[1*N*M + (M-1)*N + j] + 0.0;
					deltaG[2*N*(M + 1) + M*N + j]	= - G[2*N*M + (M-1)*N + j] + pression_speciale;
					deltaG[3*N*(M + 1) + M*N + j]	= - G[3*N*M + (M-1)*N + j] + 0.0;

				break;
			}
			case 3:		// limites mur stratégie pas pauvre (papier JM.Ghidaglia F.Pascal, page 58 (10.54))
			{
				pression_speciale = p[(M-1)*N + j]*(1 + (v[(M-1)*N + j]*GAMMA)/(c[(M-1)*N + j] - (GAMMA-1)*v[(M-1)*N + j]));
					
					Gc[0*N*(M + 1) + 0*N + j]	= 0.5*(G[0*N*M + 0*N + j] + 0.0);
					Gc[1*N*(M + 1) + 0*N + j]	= 0.5*(G[1*N*M + 0*N + j] + 0.0);
					Gc[2*N*(M + 1) + 0*N + j]	= 0.5*(G[2*N*M + 0*N + j] + pression_speciale);
					Gc[3*N*(M + 1) + 0*N + j]	= 0.5*(G[3*N*M + 0*N + j] + 0.0);

					deltaG[0*N*(M + 1) + 0*N + j]	= G[0*N*M + 0*N + j] - 0.0;
					deltaG[1*N*(M + 1) + 0*N + j]	= G[1*N*M + 0*N + j] - 0.0;
					deltaG[2*N*(M + 1) + 0*N + j]	= G[2*N*M + 0*N + j] - pression_speciale;
					deltaG[3*N*(M + 1) + 0*N + j]	= G[3*N*M + 0*N + j] - 0.0;
				
				pression_speciale = p[(M-1)*N + j]*(1 + (v[(M-1)*N + j]*GAMMA)/(c[(M-1)*N + j] - (GAMMA-1)*v[(M-1)*N + j]));

					Gc[0*N*(M + 1) + M*N + j]	= 0.5*(G[0*N*M + (M-1)*N + j] + 0.0);
					Gc[1*N*(M + 1) + M*N + j]	= 0.5*(G[1*N*M + (M-1)*N + j] + 0.0);
					Gc[2*N*(M + 1) + M*N + j]	= 0.5*(G[2*N*M + (M-1)*N + j] + pression_speciale);
					Gc[3*N*(M + 1) + M*N + j]	= 0.5*(G[3*N*M + (M-1)*N + j] + 0.0);

					deltaG[0*N*(M + 1) + M*N + j]	= - G[0*N*M + (M-1)*N + j] + 0.0;
					deltaG[1*N*(M + 1) + M*N + j]	= - G[1*N*M + (M-1)*N + j] + 0.0;
					deltaG[2*N*(M + 1) + M*N + j]	= - G[2*N*M + (M-1)*N + j] + pression_speciale;
					deltaG[3*N*(M + 1) + M*N + j]	= - G[3*N*M + (M-1)*N + j] + 0.0;
				break;
			}
			}
		}
 		j += blockDim.x * gridDim.x;
//		i = threadIdx.y + blockIdx.y * blockDim.y;
 	}
}

//////////////////////////////////////////////
	// Calcul matriciel
/**/	// On doit calculer Phi_i = (G_i+1 + G_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (G_i+1 - G_i)/2
__global__ void	calcul_Phiy(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubary, float* ubary, float* vbary, float* pbary, float* cbary, float* Hbary, float* Phiy, float* Ry, float* Ly, float* testid, float* SSy, float* G, float* Gc, float* deltaG, float* df2y, float* resuy, float* SGy, float* PRODy, int  N, int  M, float hx, float hy, float dt){

	int i; int j;
	int k;

	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	while (i < M+1 ){
		while (j < N ){
			df2y[0+(i*N+j)*16]	= deltaG[i*N+j];
			df2y[1+(i*N+j)*16]	= deltaG[N*(M+1)+i*N+j];
			df2y[2+(i*N+j)*16]	= deltaG[2*N*(M+1)+i*N+j];
			df2y[3+(i*N+j)*16]	= deltaG[3*N*(M+1)+i*N+j];

			produitmatriciel_cuda(&Ry[(i*N+j)*16], &SSy[(i*N+j)*16], 4, &resuy[(i*N+j)*16]);
				// resu = R * diag(sign(lamba_k))
			produitmatriciel_cuda(&resuy[(i*N+j)*16], &Ly[(i*N+j)*16], 4, &SGy[(i*N+j)*16]);
				// SG = R * diag(sign(lamba_k)) * R^{-1}
			produitvecteur_cuda(&SGy[(i*N+j)*16], &df2y[(i*N+j)*16], 4, &PRODy[(i*N+j)*4]);

//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				Phiy[k*N*(M+1)+i*N+j]	= Gc[k*N*(M+1)+i*N+j]-0.5*PRODy[k+(i*N+j)*4];
/**/			// On doit calculer Phi_i = (F_i+1 + F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i+1 - F_i)/2
			}
			j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}
	// Fin Calcul matriciel
//////////////////////////////////////////////

////////////////////////////////////////////////////////////////
	// ACTUALISATION du vecteur des variables conservatives
__global__ void	calcul_U_y_rhouvpc_y(float* U, float* rho, float* u, float* v, float* c, float* p, float* Ubary, float* ubary, float* vbary, float* pbary, float* cbary, float* Hbary, float* Phiy, float* Ry, float* Ly, float* testid, float* SSy, float* G, float* Gc, float* deltaG, float* df2, float* resu, float* SG, float* PROD, int  N, int  M, float hx, float hy, float dt){

	int i; int j; int k;

	i = threadIdx.y + blockIdx.y * blockDim.y;
	j = threadIdx.x + blockIdx.x * blockDim.x;
	k = threadIdx.z + blockIdx.z * blockDim.z;
	while (i < M ){
		while (j < N ){
//			if(k<4){
			for (k = 0 ; k < 4 ; k++){
				U[k*M*N+i*N+j] -= (dt / hy)*(Phiy[k*N*(M+1)+(i+1)*N+j]-Phiy[k*N*(M+1)+i*N+j]);
			}
/**/			// U_j^{n+1} = U_j^n + (dt / h) . Phi_{j+1}^n - Phi_j^n
			rho[i*N+j]	= U[i*N+j];	// rho = U_0
			u[i*N+j]	= U[1*N*M+i*N+j]/U[0*N*M+i*N+j];	// u = U_1 / U_0
			v[i*N+j]	= U[2*N*M+i*N+j]/U[0*N*M+i*N+j];	// v = U_2 / U_0
			p[i*N+j]	= 0.4*(U[3*N*M+i*N+j]-0.5*U[0*N*M+i*N+j]*(u[i*N+j]*u[i*N+j]+v[i*N+j]*v[i*N+j]));
				// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
				// car U_4 = rho.E
				// E =  e + 0.5*||u^{->}||^2
			c[i*N+j]	= sqrt(1.4*p[i*N+j]/rho[i*N+j]);
			j += blockDim.x * gridDim.x;
		}
		i += blockDim.y * gridDim.y;
//		j = threadIdx.x + blockIdx.x * blockDim.x;
	}
}
////////////////////////////////////////////////////

	//////////////////////////////////
	//				//
	//	Fonction main		//
	//				//
	//////////////////////////////////

__host__ int main (int argc, const char * argv[])
{

	if ((argc < NOMBRE_MIN_ARGUMENTS)||(argc > NOMBRE_MAX_ARGUMENTS)) //donne l'usage si on l'appelle sans rentrer le bon nombre d'arguments
	{
		fprintf (stderr, USAGE);
		fflush (stderr);
		if (strcmp(argv[1], "-h") == 0) exit(0);
	}
	print_vert("\nInitialisation\n\tSimulation numerique des equations d'Euler compressible, en 2D, en CUDA ...\n");
	printf("\n Programme du %s.\n", VERSION);

	// variables sur le GPU et le CPU
	///////////////////////
	printf(" Declarations variables GPU et CPU en cours ... ");

/*************************************/
// entiers, flottants scalaires : CPU
		int	N;	// nombre de points de la discretisation en espace, selon x
		int	M;	// selon y,
		float	hx;	// pas de temps selon x,
		float	hy;	// selon y,

/*****************/
// vecteurs : CPU
		float	*x;	// points centraux de nos cellules
		float	*y;	// points centraux de nos cellules
		float	*U_cpu;		// vecteurs des variables conservatives
		float	*rho_cpu;	// masse volumique du fluide
		float	*u_cpu;		// vitesse SELON X
		float	*v_cpu;		// SELON Y
		float	*p_cpu;		// pression P
		float	*c_cpu;		// vitesse du son c

/******************/
// vecteurs : GPU
		float	*U;	// vecteurs des variables conservatives
		float	*Ubarx;	// valeurs aux interfaces de U
		float	*Ubary;
// Flux
		float	*Phix;	// flux numeriques
		float	*Phiy;	// flux numeriques
		float	*F;	// termes F(U) de l'EDP
		float	*Fc;	// flux centres
		float	*G;	// termes G(U) de l'EDP
		float	*Gc;	// flux centres
		float	*deltaF;// differences des flux
		float	*deltaG;// differences des flux
// variables conservatives
		float	*rho;	// masse volumique du fluide
		float	*u;	// vitesse SELON X
		float	*v;	// SELON Y
		float	*p;	// pression P
		float	*c;	// vitesse du son c
// matrices propres
		float	*Sx;	// vecteur intermediaire des valeurs propres de A (cf equation)
		float	*Sy;	// vecteur intermediaire des valeurs propres de A (cf equation)
		float	*Rx;	// matrice des vecteurs propres de A
		float	*Ry;	// matrice des vecteurs propres de A
		float	*Lx;	// L^t = R^{-1} matrice des vecteurs propres de A^t
		float	*Ly;	// L^t = R^{-1} matrice des vecteurs propres de A^t
// interfaces
		float	*ubarx;	// valeurs aux interfaces selon x
		float	*ubary;	// valeurs aux interfaces selon y : vitesse u
		float	*vbarx;
		float	*vbary; // vitesse v
		float	*pbarx;
		float	*pbary; // pression p
		float	*cbarx;
		float	*cbary; // vitesse du son c
		float	*Hbarx;
		float	*Hbary; // entropie macroscopique H
// temporaires de calculs
		float	*pdt;	// vecteur intermediaire pour le calcul du pas de temps
		float	*SSx;	// matrice diagonale des vecteurs propres selon x
		float	*SSy;	// matrice diagonale des vecteurs propres selon y

		float	*IR;
		float	*resu;	// matrices temporaires pour les produits matriciels
		float	*SG;	// idem
		float	*PROD;	// idem
		float	*df2;	// vecteur intermediaire

		float	*resux;	// matrices temporaires pour les produits matriciels x
		float	*SGx;	// idem
		float	*PRODx;	// idem
		float	*df2x;	// vecteur intermediaire

		float	*resuy;	// matrices temporaires pour les produits matriciels y
		float	*SGy;
		float	*PRODy;	// idem
		float	*df2y;	// vecteur intermediaire

/**********************************************/
// variables de boucles et pour le temps : CPU
		int	i;	// variable de boucle
		int	j;	// souvent, k bouge de 0 a 3, j de 0 a N
		//int k;	// et i de 0 a M.
			// FIXME : expliquer le coup des threadIdx.x * ... dans les kernels !
		float	t	= 0.0;	// T temps maximum & t temps courant
		float	dt	= 0.000001;	// pas de temps par défaut
		int	it	= 0;	//nb d'iterations
		int	borne;	//nb max d'iterations
		//int	debug_valeur_negative;	// un compteur si p ou c deviens negatif.
		float	T	= 1000.0;	/* TODO : a modifier si besoin*/

		/* Nouvelles variables */
		int	condition_limites_x;	// pour savoir quelle condition limite chosir, selon x
		int	condition_limites_y;	// pour savoir quelle condition limite chosir, selon y

/*******************************/
// Chaines de caracteres : CPU
		char*	nom_graphique;	// titre du fichier dans lequel on imprime les resultats
		float	*testid;

/************************************/
// Variables pour les kernels CUDA : (hautement experimental !)
	//Avant le premier appel a un Kernel CUDA, on va definir deux variables spécifiques a CUDA, pour appeler ce kernel avec le bon nombre de threads et de blocs

//	dim3	grids(128,	128);
//	dim3	threads(16,	16);
		// On utilise les valeurs définies plus haut
	dim3	grids(GRIDS_X, GRIDS_Y, GRIDS_Z);
	dim3	threads(THREADS_X, THREADS_Y, THREADS_Z);

// Un essai pour paralléliser les opérations avec k entre 0 et 3
//	dim3	grids_3D_avec_k(128, 128, 32);
//	dim3	threads_3D_avec_k(16, 16, 4);

	// on utilise les memes pour tous les kernels : mais cela impose de vérifier dans chaque kernel que les indices i et j soient bien bornés par leur tailles max ...

	// Initialisation des variables qui seront fixes :
	/////////////////////////////////////////////////

// FIXME : Selon les plateformes, Sophie ou Maxime compilent avec un IDE, donc sans donner d'arguments
//		et Lilian compile dans un terminal, et peut donner des arguments
	N	= GET_N();	//nb de points du decoupage du segment [0,1] horizontal : argument de notre executable
	M	= GET_M();	//nb de points du decoupage du segment [0,1] vertical : argument de notre executable

	hx	= 1. / N;	//pas de la discretisation
	hy	= 1. / M;

	borne 	= GET_Borne();	//nombre maximum d'iteration : argument de notre executable
	// types de conditions limites selon les trois directions
	condition_limites_x	= GET_CONDITION_PERIODIQUES_x;	// FIXME : pour Maxime ou Sophie, mettre ici la constante voulue (0,1,2,3,4) 
	condition_limites_y	= GET_CONDITION_PERIODIQUES_y;

	int debugint;
	debugint = GET_DEBUG__VOIR_VTK;

	//Messages d'erreurs en cas de conditions non respectees sur les arguments de l'executable.
	if (borne <= 0 ){	fprintf (stderr, "ERROR - ./Euler_2D_cuda : La borne doit etre un entier positif.\n"); fflush (stderr); exit (4);  }
	if (N < 0 ){		fprintf (stderr, "ERROR - ./Euler_2D_cuda : Le nombre de points <N> selon x doit etre un entier >= 0.\n"); fflush (stderr); exit (5);}
	if (M < 0 ){		fprintf (stderr, "ERROR - ./Euler_2D_cuda : Le nombre de points <M> selon y doit etre un entier >= 0.\n"); fflush (stderr); exit (6);}

	//Ecrire les parametres de la simulation :
	printf ("\nNombre de points de la simulation selon x : N = \033[01;32m%d\033[01;37m\n", N);
	printf ("\t\tPas de la simulation selon x : hx = \033[01;32m%f\033[01;37m\n", hx);
	printf ("Nombre de points de la simulation selon y : M = \033[01;32m%d\033[01;37m\n", M);
	printf ("\t\tPas de la simulation selon y : hy = \033[01;32m%f\033[01;37m\n", hy);

	printf ("Nombre d'etapes maximum de la simulation : borne = \033[01;32m%d\033[01;37m\n", borne);
	printf ("Orientation des conditions initiales de pression (P) et de masse (rho) : %s.\n\n", nom_conditions_initiales(GET_CONDITION_INITIALES));
	printf ("Indice de débug debugint = \033[01;32m%d\033[01;37m.\n", debugint);

	printf ("Coefficient de Courant-Friedrich-Levy : CFL = \033[01;32m%.2f\033[01;37m;\n", CFL);
	printf ("Coefficient gamma du gaz modèle : GAMMA = \033[01;32m%.2f\033[01;37m.\n\n", GAMMA);
	printf("Type de condition aux limites : \033[01;32m%s\033[01;37m selon x, et \033[01;32m%s\033[01;37m selon y.\n\n", nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));

	printf("Variables de la parallélisation : THREADS_X=\033[01;32m%d\033[01;37m\t, THREADS_Y=\033[01;32m%d\033[01;37m\t, THREADS_Z=\033[01;32m%d\033[01;37m\t, GRIDS_X=\033[01;32m%d\033[01;37m\t, GRIDS_Y=\033[01;32m%d\033[01;37m\t, GRIDS_Z=\033[01;32m%d\033[01;37m.\n", THREADS_X, THREADS_Y,THREADS_Z, GRIDS_X, GRIDS_Y, GRIDS_Z);

debugm(0, "Je me lance");

printf("**************************************************************\n");
	printf(" Declarations variables CPU et GPU finie !\n");

    	// allocations de memoires CPU
    	///////////////////////////////
	printf(" Allocation memoire CPU et GPU en cours ... ");

/**********************************************************************************************
	Il y a plusieurs sortes de tableaux : CPU et GPU.
$ CPU	:
=========

	seulement u_cpu, v_cpu, c_cpu, p_cpu, rho_cpu et U_cpu

$ GPU	:
=========

taille N*M : valeurs moyennes sur les cellules (u, v, p, c, rho; et x)
taille (N+1)*M : valeurs aux interfaces selon x (ubarx, vbarx, pbarx, cbarx, etc)
taille N*(M+1) : valeurs aux interfaces selon y (ubary, vbary, pbary, cbary, etc)
taille 4*N*M : quatres informations en chaque cellule (U, F, G)
taille 4*(N+1)*M : quatres informations en chaque interface selon x (Ubarx, Fcx, deltaFx, Phix etc)
taille 4*N*(M+1) : quatres informations en chaque interface selon y (Ubary, Fcy, deltaFy, Phiy etc)
taille 4*4 : matrice 4x4 temporaire pour le calcul des flux (methode VFFC)
taille 4*4*(N+1)*M : matrices 4x4 en chaque interfaces pour le calcul des flux en x (Lx et Rx et SSx)
taille 4*4*N*(M+1) : matrices 4x4 en chaque interfaces pour le calcul des flux en y (Ly et Ry et SSy)

***********************************************************************************************/
/* CPU */
	x	= (float*) malloc(	N	* sizeof(float));
	y	= (float*) malloc(	M	* sizeof(float));
	rho_cpu	= (float*) malloc(	N*M	* sizeof(float));
	u_cpu	= (float*) malloc(	N*M	* sizeof(float));
	v_cpu	= (float*) malloc(	N*M	* sizeof(float));
	p_cpu	= (float*) malloc(	N*M	* sizeof(float));
	c_cpu	= (float*) malloc(	N*M	* sizeof(float));
	U_cpu	= (float*) malloc(	4*N*M	* sizeof(float));

/* GPU */
	// ex :	cudaMalloc( (void**)&rho_dev	, nlsd*ncsd * 			sizeof(float));
	cudaMalloc( (void**)&U		, 4*N*M	*	sizeof(float));
	cudaMalloc( (void**)&Ubarx	, 4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&Ubary	, 4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&Phix	, 4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&Phiy	, 4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&F		, 4*N*M	*	sizeof(float));
	cudaMalloc( (void**)&Fc		, 4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&deltaF	, 4*(N+1)*M	*	sizeof(float));

	cudaMalloc( (void**)&G		, 4*N*M	*	sizeof(float));
	cudaMalloc( (void**)&Gc		, 4*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&deltaG	, 4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&rho	, N*M	*	sizeof(float));
	cudaMalloc( (void**)&u		, N*M	*	sizeof(float));
	cudaMalloc( (void**)&v		, N*M	*	sizeof(float));
	cudaMalloc( (void**)&p		, N*M	*	sizeof(float));
	cudaMalloc( (void**)&c		, N*M	*	sizeof(float));

	cudaMalloc( (void**)&Sx		, 4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&Sy		, 4*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&Rx		, 4*4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&Ry		, 4*4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&ubarx	, 1*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&ubary	, 1*N*(M+1)*	sizeof(float));
	cudaMalloc( (void**)&vbarx	, 1*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&vbary	, 1*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&pbarx	, 1*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&pbary	, 1*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&cbarx	, 1*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&cbary	, 1*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&Hbarx	, 1*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&Hbary	, 1*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&pdt	, 1*(N+1)	*	sizeof(float));
	cudaMalloc( (void**)&SSx	, 4*4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&SSy	, 4*4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&resu	, 4*4	*	sizeof(float));
	cudaMalloc( (void**)&IR		, 4*4	*	sizeof(float));
	cudaMalloc( (void**)&SG		, 4*4	*	sizeof(float));
	cudaMalloc( (void**)&PROD	, 4	*	sizeof(float));
	cudaMalloc( (void**)&df2	, 4	*	sizeof(float));

	cudaMalloc( (void**)&resux	, 4*4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&SGx		, 4*4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&PRODx	, 4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&df2x	, 4*(N+1)*M	*	sizeof(float));

	cudaMalloc( (void**)&resuy	, 4*4*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&SGy		, 4*4*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&PRODy	, 4*N*(M+1)	*	sizeof(float));
	cudaMalloc( (void**)&df2y	, 4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&Lx		, 4*4*(N+1)*M	*	sizeof(float));
	cudaMalloc( (void**)&Ly		, 4*4*N*(M+1)	*	sizeof(float));

	cudaMalloc( (void**)&testid	, 4*4	*	sizeof(float));

// Nom du fichier texte dans lequel on ecrit les donnees numeriques
	nom_graphique	= (char*) malloc(	sizeof(char)*	400);


	printf(" Allocation memoire CPU et GPU finie !\n");
	////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
   	//		Initialisation des tableaux C		//
	//////////////////////////////////////////////////////////

	printf(" Initialisation CPU en cours ... discretisation sur [0, 1] x [0, 1] \n");

		// Points x : on travaille sur [0, 1]
		for (j = 0 ; j < N ; j++){
			x[j]=hx*0.5+j*hx;
			if(GET_DEBUG__VOIR_VTK > 3) printf("x[%d]=%f\n",j,x[j]);
		}

		// Points y : on travaille sur [0, 1]
		for (i = 0 ; i < M ; i++){
			y[i]=hy*0.5+i*hy;
			if(GET_DEBUG__VOIR_VTK > 3) printf("y[%d]=%f\n",i,y[i]);
		}


printf("**************************************************************\n");
		// Initialisation de rho
		for (i = 0 ; i < M ; i++){
			for (j = 0 ; j < N ; j++){
				rho_cpu[i*N+j] = 1.0;
				if (GET_CONDITION_INITIALES == 0){
					if (y[i]<=0.5)
//					if ((x[j]<=0.5)&&(y[i]<=0.5))
					{
						rho_cpu[i*N+j]=0.125;
					}
					else
					{
						rho_cpu[i*N+j]=1.0;
					}
				}

				if (GET_CONDITION_INITIALES == 1){
					if (x[j]<=0.5)
//					if ((x[j]<=0.5)&&(y[i]<=0.5))
					{
						rho_cpu[i*N+j]=0.125;
					}
					else
					{
						rho_cpu[i*N+j]=1.0;
					}
				}

				if (GET_CONDITION_INITIALES == 2){
						rho_cpu[i*N+j] = 0.125 + 0.875*((norme_carre(x[j] - 0.25, y[i] - 0.25))<carre(0.125));
				}

				if (GET_CONDITION_INITIALES == 3){
						rho_cpu[i*N+j] = 1.0;
				}
//				rho_cpu[i*N+j] += 15.0;
					if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,%d]=%f\n",i,j,rho_cpu[i*N+j]);
// Initialisation de u TODO : tester des nouvelles CI
				u_cpu[i*N+j]=0.1;
				if (y[i]<=0.5)
				{
					u_cpu[i*N+j]=0.0;
				}
				else
				{
					u_cpu[i*N+j]=0.0;
				}
	//			u_cpu[i*N+j]=20.0;
					if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,%d]=%f\n",i,j,u_cpu[i*N+j]);
// Initialisation de v
				v_cpu[i*N+j]=0.1;
				if (y[i]<=0.5)
				{
					v_cpu[i*N+j]=0.0;
				}
				else
				{
					v_cpu[i*N+j]=0.0;
				}
	//			v_cpu[i*N+j]=20.0;
					if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,%d]=%f\n",i,j,v_cpu[i*N+j]);
// Initialisation de p
			    p_cpu[i*N+j] = 2.0;
			    if (GET_CONDITION_INITIALES == 0){
				if (y[i]<=0.5)
				{
					p_cpu[i*N+j]=0.1;
				}
				else
				{
					p_cpu[i*N+j]=1;
				}
			    }
			    if (GET_CONDITION_INITIALES == 1){
				if (x[j]<=0.5)
				{
					p_cpu[i*N+j]=0.1;
				}
				else
				{
					p_cpu[i*N+j]=1;
				}
			    }

			   if (GET_CONDITION_INITIALES == 2){
					p_cpu[i*N+j] = 0.1 + 0.9*((norme_carre(x[j] - 0.25, y[i] - 0.25))<carre(0.125));
			   }

			   if (GET_CONDITION_INITIALES == 3){
					p_cpu[i*N+j] = 2.0;
			   }
//				p_cpu[i*N+j] = 20.0;
				if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,%d]=%f\n",i,j,p_cpu[i*N+j]);
// Initialisation de c
				c_cpu[i*N+j]=sqrt(1.4*p_cpu[i*N+j]/rho_cpu[i*N+j]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,%d]=%f\n",i,j,c_cpu[i*N+j]);
		// Initialisation de U
		// U = [ rho ; rho u ; rho v ; rho E]
				U_cpu[i*N+j]		= rho_cpu[i*N+j];
				U_cpu[N*M+i*N+j]	= rho_cpu[i*N+j]*u_cpu[i*N+j];
				U_cpu[2*N*M+i*N+j]	= rho_cpu[i*N+j]*v_cpu[i*N+j];
				U_cpu[3*N*M+i*N+j]	= 0.5*rho_cpu[i*N+j]*(u_cpu[i*N+j]*u_cpu[i*N+j]+v_cpu[i*N+j]*v_cpu[i*N+j])+p_cpu[i*N+j]/0.4;
				if(GET_DEBUG__VOIR_VTK > 3) {
					printf("U0[%d,%d]=%f\n",i,j,U_cpu[i*N+j+0*N*M]);
					printf("U1[%d,%d]=%f\n",i,j,U_cpu[i*N+j+1*N*M]);
					printf("U2[%d,%d]=%f\n",i,j,U_cpu[i*N+j+2*N*M]);
					printf("U3[%d,%d]=%f\n",i,j,U_cpu[i*N+j+3*N*M]);
					printf("\n");
					}
			}
		}

//	dt = pas_de_temps_seq(u, c, v, N, M, hx, hy, it);

	printf(" Initialisation CPU finie \n");
	/////////////////////////////////////////


	//////////////////////////////////////////////////////////
   	//		Initialisation des tableaux GPU		//
	//////////////////////////////////////////////////////////

	printf(" Initialisation GPU en cours ... discretisation sur [0, 1] x [0, 1] \n");

	cudaMemcpy( rho, rho_cpu, N*M	* sizeof(float), cudaMemcpyHostToDevice );	// sens CPU --> GPU
	if(debugint > 2)	test_en_affichant(argc, argv, rho, N, M, hx, hy, it, "rho");
	cudaMemcpy( u, u_cpu, N*M	* sizeof(float), cudaMemcpyHostToDevice );
	if(debugint > 2)	test_en_affichant(argc, argv, u, N, M, hx, hy, it, "u");
	cudaMemcpy( v, v_cpu, N*M	* sizeof(float), cudaMemcpyHostToDevice );
	if(debugint > 2)	test_en_affichant(argc, argv, v, N, M, hx, hy, it, "v");
	cudaMemcpy( c, c_cpu, N*M	* sizeof(float), cudaMemcpyHostToDevice );
	if(debugint > 2)	test_en_affichant(argc, argv, c, N, M, hx, hy, it, "c");
	cudaMemcpy( p, p_cpu, N*M	* sizeof(float), cudaMemcpyHostToDevice );
	if(debugint > 2)	test_en_affichant(argc, argv, p, N, M, hx, hy, it, "p");

	cudaMemcpy( U, U_cpu, 4*N*M	* sizeof(float), cudaMemcpyHostToDevice );
	if(debugint > 2)	test_en_affichant(argc, argv, rho, N, M, hx, hy, it, "rho");

	printf(" Initialisation GPU finie\n");
	/////////////////////////////////////////

	// On va sauvegarder les valeurs du pas de temps pour les étudier après
	sprintf(nom_graphique, "Euler2DCUDA_dt_N%d_M%d_CLD_%d_rho_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
	FILE *filedt=fopen(nom_graphique, "w");	// On ouvre en ecriture 'w'
	assert(filedt != NULL);
	fprintf(filedt,"# vtk DataFile Version 3.0\n#EULER_2D_CUDA dt\nASCII\nDATASET STRUCTURED_POINTS\n");
	fprintf(filedt,"DIMENSIONS %d %d %d\n",	borne,	1,	1);
	fprintf(filedt,"ORIGIN %f %f %f\n",	0.0,	0.0,	0.0);
	fprintf(filedt,"SPACING %f %f %f\n",	1.0,	1.0,	1.0);
	fprintf(filedt,"POINT_DATA %d\n",		borne);
	fprintf(filedt,"SCALARS dt float\n");
	fprintf(filedt,"LOOKUP_TABLE default\n");
	printf("\033[01;34mOuverture du fichier <%s> pour l'écriture des valeurs de dt.\n\033[01;37m", nom_graphique);

////////////////////////////////////////////////////////////////
	// Ecriture des donnees numeriques dans les fichiers VTK
	//  Pour plus de details sur la syntaxe de tels fichiers, voir les rapport du mois d'avril.
	//	ou les fonctions en fin de fichier. (⇓)

	// On imprime Rho
	sprintf(nom_graphique, "Euler2DCUDA_N%d_M%d_CLD_%d_rho_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [RHO]...\n",0, nom_graphique);
	WriteVtk2D(nom_graphique, rho_cpu, N, M, hx, hy, "Rho", GET_DEBUG__VOIR_VTK);

	// On imprime P
//	sprintf(nom_graphique, "Euler2DCUDA_N%d_M%d_CLD_%d_p_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
//	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [P]...\n", 0, nom_graphique);
//	WriteVtk2D(nom_graphique, p_cpu, N, M, hx, hy, "P", GET_DEBUG__VOIR_VTK);

debugm(-22 + GET_DEBUG__VOIR_VTK, "Initialisation faite : je me lance");

	float *pointeur_dt, *pointeur_dt_cpu;

	pointeur_dt_cpu = (float*) malloc( 1 * sizeof(float));
	cudaMalloc( (void**)&pointeur_dt	, 1	*	sizeof(float));

//////////////////////////////////////////////////////////////////////////
//			DEBUT du corps de la boucle en temps		//
//////////////////////////////////////////////////////////////////////////

	// on limite le temps, et le nombre de pas de temps (pour eviter de laisser la simulation s'emballer)
while ((t <= T) && (it < borne) )
{

	//Calcul du pas de temps
	/* Deux solutions : le faire sur le GPU mais en séquentiel, ou transporter tous les tableaux, et le faire en séquentiel sur le CPU */
//printf("Calcul du pas de temps :\n");
//if(debugint > 2)	test_en_affichant(argc, argv, c, N, M, hx, hy, it, "c");
//if(debugint > 2)	test_en_affichant(argc, argv, u, N, M, hx, hy, it, "u");
//if(debugint > 2)	test_en_affichant(argc, argv, v, N, M, hx, hy, it, "v");

//	pas_de_temps_cuda_seq<<<1, 1>>>(pointeur_dt, u, c, v, N, M, hx, hy, it);
//	cudaMemcpy( pointeur_dt_cpu, pointeur_dt, 1	* sizeof(float), cudaMemcpyDeviceToHost );
//	dt = pointeur_dt_cpu[0];

if(CALCUL_DU_PAS_DE_TEMPS == 1)	dt = pas_de_temps_seq(u, c, v, N, M, hx, hy, it);

	it ++;		// actualisation du compteur de boucle (utile !)
	t += dt;	// actualisation du temps courant (valeur réelle : inutilisée)
	printf("\033[01;31m Nouvelle etape : t = %f dt = %f it = %d.\n \033[01;37m", t, dt, it);
//printf("dt ***********************************************************\n");
	fprintf(filedt, "%f\n", dt);

//////////////////////////////////////////
//	SPLITTING SELON X		//
//////////////////////////////////////////

	calcul_Ubarx<<<grids, threads>>>(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de Ubarx");
if(debugint > 2)	test_en_affichant(argc, argv, Ubarx, N+1, 4*M, hx, hy, it, "Ubarx");

	calcul_uvpcH_barx_RLSSx<<<grids, threads>>>(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de ubarx");
if(debugint > 2)	test_en_affichant(argc, argv, ubarx, N+1, M, hx, hy, it, "ubarx");
debugm(debugint, "Calcul de vbarx");
if(debugint > 2)	test_en_affichant(argc, argv, vbarx, N+1, M, hx, hy, it, "vbarx");
debugm(debugint, "Calcul de pbarx");
if(debugint > 2)	test_en_affichant(argc, argv, pbarx, N+1, M, hx, hy, it, "pbarx");
debugm(debugint, "Calcul de cbarx");
if(debugint > 2)	test_en_affichant(argc, argv, cbarx, N+1, M, hx, hy, it, "cbarx");
debugm(debugint, "Calcul de Hbarx");
if(debugint > 2)	test_en_affichant(argc, argv, Hbarx, N+1, M, hx, hy, it, "Hbarx");
debugm(debugint, "Calcul de RLSSx");

if(debugint > 2){
		for (i = 0 ; i < M ; i++){
			for (j = 0 ; j < N+1 ; j++){
				test_verif_I(debugint, &Rx[16*(i*(N+1)+j)], &Lx[16*(i*(N+1)+j)], it);
			}
		}
	}
debugm(debugint, "TEST");

	calcul_F<<<grids, threads>>>(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de F");
if(debugint > 2)	test_en_affichant(argc, argv, F, N, 4*M, hx, hy, it, "F");

	calcul_Fc_deltaF<<<grids, threads>>>(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2, resu, SG, PROD, N, M, hx, hy, dt, condition_limites_x);
debugm(debugint, "Calcul de Fc");
if(debugint > 2)	test_en_affichant(argc, argv, Fc, N+1, 4*M, hx, hy, it, "Fc");
debugm(debugint, "Calcul de deltaF");
if(debugint > 2)	test_en_affichant(argc, argv, deltaF, N+1, 4*M, hx, hy, it, "deltaF");

/* Deux solutions */
	calcul_Phix<<<grids, threads>>>(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2x, resux, SGx, PRODx, N, M, hx, hy, dt);
//	calcul_Phix2(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de Phix");
if(debugint > 2)	test_en_affichant(argc, argv, Phix, N, 4*M, hx, hy, it, "Phix");

	calcul_U_x_rhouvpc_x<<<grids, threads>>>(U, rho, u, v, c, p, Ubarx, ubarx, vbarx, pbarx, cbarx, Hbarx, Phix, Rx, Lx, testid, SSx, F, Fc, deltaF, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de U_x");
if(debugint > 2)	test_en_affichant(argc, argv, U, N, 4*M, hx, hy, it, "U_x");
debugm(debugint, "Calcul de rhouvpc_x");
if(debugint > 2)	test_en_affichant(argc, argv, rho, N, M, hx, hy, it, "rho_x");
if(debugint > 2)	test_en_affichant(argc, argv, u, N, M, hx, hy, it, "u_x");
if(debugint > 2)	test_en_affichant(argc, argv, v, N, M, hx, hy, it, "v_x");
if(debugint > 2)	test_en_affichant(argc, argv, p, N, M, hx, hy, it, "p_x");
if(debugint > 2)	test_en_affichant(argc, argv, c, N, M, hx, hy, it, "c_x");

if(debugint > 2)	printf("\033[01;34m Splitting en X : effectue !\n\033[01;37m");
if(debugint > 2)printf("\033[01;34m X *************************************************************\n\033[01;37m");

//////////////////////////////////////////
//	SPLITTING SELON Y		//
//////////////////////////////////////////

	calcul_Ubary<<<grids, threads>>>(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de Ubary");
if(debugint > 2)	test_en_affichant(argc, argv, Ubary, N, 4*(M+1), hx, hy, it, "Ubary");

	calcul_uvpcH_bary_RLSSy<<<grids, threads>>>(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de ubary");
if(debugint > 2)	test_en_affichant(argc, argv, ubary, N, M+1, hx, hy, it, "ubary");
debugm(debugint, "Calcul de vbary");
if(debugint > 2)	test_en_affichant(argc, argv, vbary, N, M+1, hx, hy, it, "vbary");
debugm(debugint, "Calcul de pbary");
if(debugint > 2)	test_en_affichant(argc, argv, pbary, N, M+1, hx, hy, it, "pbary");
debugm(debugint, "Calcul de cbary");
if(debugint > 2)	test_en_affichant(argc, argv, cbary, N, M+1, hx, hy, it, "cbary");
debugm(debugint, "Calcul de Hbary");
if(debugint > 2)	test_en_affichant(argc, argv, Hbary, N, M+1, hx, hy, it, "Hbary");
debugm(debugint, "Calcul de RLSSy");

if(debugint > 2){
		for (i = 0 ; i < M+1 ; i++){
			for (j = 0 ; j < N ; j++){
				test_verif_I(debugint, &Ry[16*(i*N+j)], &Ly[16*(i*N+j)], it);
			}
		}
	}
debugm(debugint, "TEST");

	calcul_G<<<grids, threads>>>(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de G");
if(debugint > 2)	test_en_affichant(argc, argv, G, 4*N, M, hx, hy, it, "G");

	calcul_Gc_deltaG<<<grids, threads>>>(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2, resu, SG, PROD, N, M, hx, hy, dt, condition_limites_y);
debugm(debugint, "Calcul de Gc");
if(debugint > 2)	test_en_affichant(argc, argv, Gc, N, 4*(M+1), hx, hy, it, "Gc");
debugm(debugint, "Calcul de deltaG");
if(debugint > 2)	test_en_affichant(argc, argv, deltaG, N, 4*(M+1), hx, hy, it, "deltaG");

/* Deux solutions */
	calcul_Phiy<<<grids, threads>>>(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2y, resuy, SGy, PRODy, N, M, hx, hy, dt);
//	calcul_Phiy2(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de Phiy");
if(debugint > 2)	test_en_affichant(argc, argv, Phiy, N, 4*M, hx, hy, it, "Phiy");

	calcul_U_y_rhouvpc_y<<<grids, threads>>>(U, rho, u, v, c, p, Ubary, ubary, vbary, pbary, cbary, Hbary, Phiy, Ry, Ly, testid, SSy, G, Gc, deltaG, df2, resu, SG, PROD, N, M, hx, hy, dt);
debugm(debugint, "Calcul de U_y");
if(debugint > 2)	test_en_affichant(argc, argv, U, N, 4*M, hx, hy, it, "U_y");
debugm(debugint, "Calcul de rhouvpc_y");
if(debugint > 2)	test_en_affichant(argc, argv, rho, N, M, hx, hy, it, "rho_y");
if(debugint > 2)	test_en_affichant(argc, argv, u, N, M, hx, hy, it, "u_y");
if(debugint > 2)	test_en_affichant(argc, argv, v, N, M, hx, hy, it, "v_y");
if(debugint > 2)	test_en_affichant(argc, argv, p, N, M, hx, hy, it, "p_y");
if(debugint > 2)	test_en_affichant(argc, argv, c, N, M, hx, hy, it, "c_y");

if(debugint > 2)	printf("\033[01;34m Splitting en Y : effectue !\n\033[01;37m");
if(debugint > 2)printf("\033[01;34m Y *************************************************************\n\033[01;37m");

/////////////////////////////////////////////////////////////////////////////////////////////
	// Copie des valeurs des tableaux GPU vers la mémoire CPU
	// on ramene vers le CPU (note : pour etre efficace, il faut limiter AU MAXIMUM ces echanges)
	// on ramène seulement ceux qu'on désire imprimer :/

//	cudaMemcpy( U_cpu, U, 4*N*M	* sizeof(float), cudaMemcpyDeviceToHost );	// Sens GPU --> CPU
//	cudaMemcpy( u_cpu, u, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
//	cudaMemcpy( v_cpu, v, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
//	cudaMemcpy( c_cpu, c, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
	cudaMemcpy( rho_cpu, rho, N*M	* sizeof(float), cudaMemcpyDeviceToHost );
	cudaMemcpy( p_cpu, p, N*M	* sizeof(float), cudaMemcpyDeviceToHost );

/////////////////////////////////////////////////////////////////
	// Ecriture des donnees numeriques dans les fichiers VTK
	//  Pour plus de details sur la syntaxe de tels fichiers, voir les rapport du mois d'avril.

	switch (GET_IMPRIMEVTK){
		case 0:{
			// rien faire
		break;
		}
		case 1:{
			// On imprime Rho
			sprintf(nom_graphique, "Euler2DCUDA_N%d_M%d_CLD_%d_rho_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
			printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [RHO]...\n", it, nom_graphique);
			WriteVtk2D(nom_graphique, rho_cpu, N, M, hx, hy, "Rho", GET_DEBUG__VOIR_VTK);
		break;
		}
		case 2:{
			// On imprime P
			sprintf(nom_graphique, "Euler2DCUDA_N%d_M%d_CLD_%d_p_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
			printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [P]...\n", it, nom_graphique);
			WriteVtk2D(nom_graphique, p_cpu, N, M, hx, hy, "P", GET_DEBUG__VOIR_VTK);
		break;
		}
		case 3:{
			// On imprime Rho
			sprintf(nom_graphique, "Euler2DCUDA_N%d_M%d_CLD_%d_rho_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
			printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [RHO]...\n", it, nom_graphique);
			WriteVtk2D(nom_graphique, rho_cpu, N, M, hx, hy, "Rho", GET_DEBUG__VOIR_VTK);
			// On imprime P
			sprintf(nom_graphique, "Euler2DCUDA_N%d_M%d_CLD_%d_p_%s_X%s_Y%s.vtk", N, M, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));
			printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [P]...\n", it, nom_graphique);
			WriteVtk2D(nom_graphique, p_cpu, N, M, hx, hy, "P", GET_DEBUG__VOIR_VTK);
		break;
		}
	}


}
//////////////////////////////////////////////////
//		FIN de la boucle en temps	//
//////////////////////////////////////////////////
	printf("\033[01;34m");
	// On libere la memoire allouee sur le CPU
	free(x); free(y);
	free(u_cpu); free(v_cpu); free(p_cpu); free(c_cpu); free(rho_cpu);
	free(nom_graphique);
	printf("Mémoire CPU : libérée !\n");

	// On libère la mémoire allouée sur le GPU
	cudaFree(U); cudaFree(Ubarx); cudaFree(Ubary); cudaFree(Phix); cudaFree(Phiy);
	cudaFree(F); cudaFree(Fc); cudaFree(deltaF);
	cudaFree(G); cudaFree(Gc); cudaFree(deltaG);

	cudaFree(u); cudaFree(v); cudaFree(p); cudaFree(c); cudaFree(rho);
	cudaFree(Rx); cudaFree(Lx); cudaFree(SSx);
	cudaFree(Ry); cudaFree(Ly); cudaFree(SSy);
	cudaFree(ubarx); cudaFree(vbarx); cudaFree(pbarx); cudaFree(cbarx); cudaFree(Hbary);
	cudaFree(ubary); cudaFree(vbary); cudaFree(pbary); cudaFree(cbary); cudaFree(Hbarx);
	cudaFree(resu); cudaFree(IR); cudaFree(SG); cudaFree(PROD); cudaFree(df2);
	printf("Mémoire GPU : libérée !\n");
	
	//Ecrire les parametres de la simulation :
	printf ("\nNombre de points de la simulation selon x : N = \033[01;32m%d\033[01;37m\n", N);
	printf ("\t\tPas de la simulation selon x : hx = \033[01;32m%f\033[01;37m\n", hx);
	printf ("Nombre de points de la simulation selon y : M = \033[01;32m%d\033[01;37m\n", M);
	printf ("\t\tPas de la simulation selon y : hy = \033[01;32m%f\033[01;37m\n", hy);

	printf ("Nombre d'etapes maximum de la simulation : borne = \033[01;32m%d\033[01;37m\n", borne);
	printf ("Orientation des conditions initiales de pression (P) et de masse (rho) : %s.\n\n", nom_conditions_initiales(GET_CONDITION_INITIALES));
	printf ("Indice de débug debugint = \033[01;32m%d\033[01;37m.\n", debugint);

	printf ("Coefficient de Courant-Friedrich-Levy : CFL = \033[01;32m%.2f\033[01;37m;\n", CFL);
	printf ("Coefficient gamma du gaz modèle : GAMMA = \033[01;32m%.2f\033[01;37m.\n\n", GAMMA);
	printf("Type de condition aux limites : \033[01;32m%s\033[01;37m selon x, et \033[01;32m%s\033[01;37m selon y.\n\n", nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y));

	printf("Variables de la parallélisation : THREADS_X=\033[01;32m%d\033[01;37m\t, THREADS_Y=\033[01;32m%d\033[01;37m\t, THREADS_Z=\033[01;32m%d\033[01;37m\t, GRIDS_X=\033[01;32m%d\033[01;37m\t, GRIDS_Y=\033[01;32m%d\033[01;37m\t, GRIDS_Z=\033[01;32m%d\033[01;37m.\n", THREADS_X, THREADS_Y,THREADS_Z, GRIDS_X, GRIDS_Y, GRIDS_Z);

	printf("\nDONE : euler_2D_cuda : simulation terminee.\nHecth & Isnard & Besson (c) et (r) 2012\n");
	printf("  Les resultats ont ete ecrits dans deux listes de fichiers VTK :\n\
			Ecrire_valeurs_Euler_2D_cuda_*_p_[x|y|circ].vtk  \t : pour la pression\n\
			Ecrire_valeurs_Euler_2D_cuda_*_rho_[x|y|circ].vtk\t : pour la masse volumique\n\
		Ils peuvent etre visualises via le logiciel Paraview \n\
			recuperable a l'adresse suivante : [http://www.paraview.org/paraview/resources/software.php] \n\
		 ou	[sudo apt-get install paraview]	sous Linux \n\
		Pour toutes questions, remarques ou corrections : <lilian.besson@ens-cachan.fr>.\n\
		\n");
	printf("\033[01;37m");

	return 0;
}
	//////////////////////////////////////////////////////////
	//							//
	//	Fin de la fonction Main				//
	//							//
	//////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////
// Pour ecrire notre matrice rho ou P dans un fichier VTK
__host__ void WriteVtk2D(char *nom, float *f, int N, int M, float hx, float hy, char* nom_variable, int debugint)
{
	int i,j;
	FILE *file=fopen(nom, "w");	// On ouvre en ecriture 'w'
//	assert(file != NULL);
	fprintf(file,"# vtk DataFile Version 3.0\n#EULER_2D_CUDA %s\nASCII\nDATASET STRUCTURED_POINTS\n", nom_variable);
	fprintf(file,"DIMENSIONS %d %d 1\n",	N,	M);
	fprintf(file,"ORIGIN %f %f 0.0\n",	0.5*hx,	0.5*hy);
	fprintf(file,"SPACING %f %f 1.0\n",	hx,	hy);
	fprintf(file,"POINT_DATA %d\n",		N*M);
	fprintf(file,"SCALARS %s float\n", 	nom_variable);
	fprintf(file,"LOOKUP_TABLE default\n");

	for (i=0; i<M; i++){
		for (j=0; j<N; j++){
			fprintf(file, "%f\n",f[i*N+j]);
		}
	}
//	fflush(file);
	fclose(file);	// TODO : Tres important !!
			// Le nombre de stream ouvert est limite, il faut donc les fermer des qu'on a plus besoin d'ecrire.
			// /!\ Cause de nombreuses 'seg fault' inexplicables !
	fflush(stdout);
}

//{
//	int i,j;
//	char* nom_destination;

//	nom_destination = (char*) malloc(800 * sizeof(char));

//	sprintf(nom_destination, "%s", nom);
//if (debugint != 2){
//	FILE *file=fopen(nom_destination, "w");	// On ouvre en ecriture 'w'
//	assert(file != NULL);
//	fprintf(file,"# vtk DataFile Version 3.0\n#EULER_2D_CUDA %s\nASCII\nDATASET STRUCTURED_POINTS\n", nom_variable);
//	fprintf(file,"DIMENSIONS %d %d %d\n",	N,	M,	1);
//	fprintf(file,"ORIGIN %f %f %f\n",	0.5*hx,	0.5*hy,	0.0);
//	fprintf(file,"SPACING %f %f %f\n",	hx,	hy,	1.0);
//	fprintf(file,"POINT_DATA %d\n",		N*M);
//	fprintf(file,"SCALARS %s float\n", 	nom_variable);
//	fprintf(file,"LOOKUP_TABLE default\n");

//	if (debugint){
//		printf("* ECRIRE RESULTATS : \n* Variable %s.\tFichier %s.\tN=%d\tM=%d\thx=%f\thy=%f\n", nom_variable, nom_destination, N, M, hx, hy);
//		printf("*********\n");
//		}
//	for (i=0; i<M; i++){
//		for (j=0; j<N; j++){
//			if (debugint) { printf("%s\t[%d\t,%d\t] = %f\n",nom_variable, i, j, f[i*N+j]); }
//			fprintf(file, "%f\n",f[i*N+j]);
//		}
//	}
//	if (debugint){
//		printf("* ECRIRE RESULTATS : \n* Variable %s.\tFichier %s.\tN=%d\tM=%d\thx=%f\thy=%f\n", nom_variable, nom_destination, N, M, hx, hy);
//		printf("*********\n");
//		debugm(debugint, "Valeurs ecrites : je continue");
//	}
//	fflush(file);
//	fclose(file);	// TODO : Tres important !!
//			// Le nombre de stream ouvert est limite, il faut donc les fermer des qu'on a plus besoin d'ecrire.
//			// /!\ Cause de nombreuses 'seg fault' inexplicables !
//	free(nom_destination);
//	fflush(stdout);
//}}

//////////////////////////////////////////////////////////////////
//	Maxime Isnard & Sophie Hecht & Lilian BESSON		//
//		 ENS de Cachan & LSV & CMLA - Juin 2012		//
//////////////////////////////////////////////////////////////////
