//////////////////////////////////////////////////
//  Equation d'Euler compressible 3D	C
//	avec terme source
//	et différentes conditions aux limites
//////////////////////////////////////////////////

// version jeudi 28 juin : version FINALE ! [tous les tests sont ok]

/* Ces codes sont librement distribués sous licence GPL
 * disponible ici <http://www.gnu.org/copyleft/gpl.html>
 */

////////////////////////////////////////////////////////////////
//	Maxime Isnard & Sophie Hecht & Lilian Besson
//		 ENS de Cachan & LSV & CMLA - Juin 2012
////////////////////////////////////////////////////////////////

// Remarques et notations :
// 	- Les lignes ou paragraphes entourées de "//////////////////////////" sont pour structurer les étapes;
//	- Les paragraphes entre "/*" et "*/" sont importants;
//	- Les lignes commençant par "/**/" sont des justifications mathématiques correspondant aux calculs effectués juste avant;
//	- Les commentaires en fin de lignes commençant par "// " sont des explications simples;


/* DEBUT
 *    Normalement, ce code marche sur n'importe quel machine.
 *
 *	Pour compiler : $> gcc -O3 -o Euler_3D_C** Euler_3D_C.c** -lm
 *	Pour executer : $> ./Euler_3D_C** <N> <M> <O> <Borne> **	// N : nb de points selon x, M : nb de points selon y, O : nb de points selon z, Borne : nb d'etapes max
 *	Pour visualiser : $> paraview
 *		Puis : ouvrir -> Ecrire3D_*_[p ? rho ?]_[x/y/z/circ].vtk -> APPLY
 			-> Onglet Filters -> Data Analysis -> Plot Over Line
 			-> AXE X -> APPLY -> Play
 *
 *	Pour toutes questions, corrections, ou remarques, contactez nous :
 *	 maxime.isnard@ens-cachan.fr | lilian.besson@ens-cachan.fr | sophie.hecht@ens-cachan.fr
 */

	///////////////////////////////
	// # include <bibliotheque.h>

#include <stdio.h>		// librairie standard d'input & d'output
#include <stdlib.h>		// librairie standard de C

#include <math.h>		// librairie pour utiliser les fonctions mathematiques.
#include <string.h>		// librairie pour utiliser les fonctions "plus elaborees" pour la gestion de chaines de caracteres
#include <time.h>		// librairie pour utiliser des fonctions de gestions du temps.

#include <assert.h>		// librairie qui fournit assert(booleen) pour faire du debug


	///////////////////////////////////
	// #define <macro>

		// fonctions macro utiles
#define min(a,b) ((a)<=(b) ? (a) : (b))

/* Coefficient CFL pour notre pas de temps. TODO : changer un peu pour voir ! */
/* A CHANGER ICI */
#define CFL 0.99

/* Coefficient GAMMA du gaz parfait modélisé */
#define GAMMA 1.4

/* Coefficient g du terme source */
#define coefficient_g 9.81

// pour une mini interface utilisateur :
#define NOMBRE_MIN_ARGUMENTS		3
#define NOMBRE_MAX_ARGUMENTS		5
#define USAGE		("\n\
\033[01;31m\
Usage: ./euler_3D_C <N>, <M>, <O>, <borne>, <direction>, <limites_x>, <limites_y>, <limites_z>, [<controle>]\n\
	effectue une simulation de l'equation d'Euler 1D avec \n\
		sur <N> points selon x, \n\
		sur <M> points selon y, \n\
		sur <O> points selon z, \n\
		et en se limitant a <borne> etape de temps.\n\
		si <direction> est présent : les conditions initiales sont orientées selon x si direction=1 selon y si direction=0, selon z si direction=2, circulairement si direction>=3.\n\
		si <direction> est présent, et si <limite_x> est présent et non nul, les conditions limites horizontales sont orientées : vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.\n\
		si <limite_x> est présent, et si <limite_y> est présent et non nul, les conditions limites verticales sont orientées : vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.\n\
		si <limite_y et x> sont présents, et si <limite_z> est présent et non nul, les conditions limites selon z sont orientées : vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.\n\
		si <controle> est present et non nul, le systeme est en mode debug.\n\
		\n\
\n\
Le programme calcule une simulation de l'equation d'Euler compressible 3D, par la methode des volumes finis, via le langage C;\n\
!!	Les conditions limites sont périodiques.\n\
\n\
Les resultats numeriques sont ecrits dans des fichiers VTK.\
\n\
par Maxime Isnard & Sophie Hecht & Lilian Besson © 2012 - ENS de Cachan & LSV & CMLA - Naereen Corp.\n\
\033[01;37m\
")	// il s'agit du «mode d'emplois»


/* ******************************** */
/* Valeurs par defaut des arguments */
#define	N__Cst	(50)
	// Nombre de points selon x, par defaut = 5.
#define M__Cst	(50)
	// Nombre de points selon y, par defaut = 5.
#define O__Cst	(1)
	// Nombre de points selon z, par defaut = 5.
#define BORNE__Cst (500)
	// Nombre d'etape de calcul, par defaut = 150.
#define DEBUG__VOIR_VTK__Cst (0)
				// vaut 1 si on veut afficher a l'ecran les valeurs ecrites dans les fichiers VTK,
				// et attendre l'utilisateur a chaque ecriture. 0 par defaut

#define CONDITION_PERIODIQUES_x__Cst (0) // selon x
		// vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 
		// 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.

#define CONDITION_PERIODIQUES_y__Cst (0) // selon y
		// vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 
		// 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.
		
#define CONDITION_PERIODIQUES_z__Cst (0) // selon z
		// vaut 0 si on veut du limite périodique, 1 si on veut du limite apériodique, 
		// 2 si on veut des conditions de mur stratégie pauvre, 3 stratégie pas pauvre, 4 stratégie Florian de Vuyst.

/* Orientation des conditions initiales . TODO tester d'autres ! */
#define CONDITION_INITIALES__Cst	(3)
	/* Vaut 1 si selon x, 0 si selon y, 2 si selon z, circulaire baroque si = 3 */

#define SOURCE_PRESENTE	(0)	// détermine si on utilise ou pas le terme source (force de gravité). 0 : on ne le met pas, != 0 : on le met !

// Pour les arguments
/* N */
/*#ifdef __APPLE__*/
/*#define GET_N() (N__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_N() (N__Cst)*/
/*#else*/
#define GET_N() ( (argc > 1)? (atoi (argv[1])) : N__Cst )
/*#endif*/

/* M */
/*#ifdef __APPLE__*/
/*#define GET_M() (M__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_M() (M__Cst)*/
/*#else*/
#define GET_M() ( (argc > 2)? (atoi (argv[2])) : M__Cst )
/*#endif*/

/* O */
/*#ifdef __APPLE__*/
/*#define GET_O() (O__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_O() (O__Cst)*/
/*#else*/
//#define GET_O() ( atoi (argv[2]))
#define GET_O() ( (argc > 3)? (atoi (argv[3])) : O__Cst )
/*#endif*/

/* Borne */
/*#ifdef __APPLE__*/
/*#define GET_Borne() (BORNE__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_Borne() (BORNE__Cst)*/
/*#else*/
#define GET_Borne() ( (argc > 4)? (atoi (argv[4])) : BORNE__Cst )
/*#endif*/

/* Orientation des conditions initiales */
/*#ifdef __APPLE__*/
/*#define GET_CONDITION_INITIALES (CONDITION_INITIALES__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_CONDITION_INITIALES (CONDITION_INITIALES__Cst)*/
/*#else*/
#define GET_CONDITION_INITIALES ( (argc > 5)? (atoi (argv[5])) : CONDITION_INITIALES__Cst )
/*#endif*/

/* Parametre des conditions limites horizontales */
/*#ifdef __APPLE__*/
/*#define GET_CONDITION_PERIODIQUES_x (CONDITION_PERIODIQUES_x__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_CONDITION_PERIODIQUES_x (CONDITION_PERIODIQUES_x__Cst)*/
/*#else*/
#define GET_CONDITION_PERIODIQUES_x ( (argc > 6)? (atoi (argv[6])) : CONDITION_PERIODIQUES_x__Cst )
/*#endif*/

/* Parametre des conditions limites verticales */
/*#ifdef __APPLE__*/
/*#define GET_CONDITION_PERIODIQUES_y (CONDITION_PERIODIQUES_y__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_CONDITION_PERIODIQUES_y (CONDITION_PERIODIQUES_y__Cst)*/
/*#else*/
#define GET_CONDITION_PERIODIQUES_y ( (argc > 7)? (atoi (argv[7])) : CONDITION_PERIODIQUES_y__Cst )
/*#endif*/

/* Parametre des conditions limites selon z */
/*#ifdef __APPLE__*/
/*#define GET_CONDITION_PERIODIQUES_z (CONDITION_PERIODIQUES_z__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_CONDITION_PERIODIQUES_z (CONDITION_PERIODIQUES_z__Cst)*/
/*#else*/
#define GET_CONDITION_PERIODIQUES_z ( (argc > 8)? (atoi (argv[8])) : CONDITION_PERIODIQUES_z__Cst )
/*#endif*/

/* Parametre de debug */
/*#ifdef __APPLE__*/
/*#define GET_DEBUG__VOIR_VTK (DEBUG__VOIR_VTK__Cst)*/
/*#endif*/
/*#ifdef __WINDOWS__*/
/*#define GET_DEBUG__VOIR_VTK (DEBUG__VOIR_VTK__Cst)*/
/*#else*/
#define GET_DEBUG__VOIR_VTK ( (argc > 9)? (atoi (argv[9])) : DEBUG__VOIR_VTK__Cst )
/*#endif*/
	/////////////////////////////////////////////////
	// 		Calcul matriciel
	/////////////////////////////////////////////////

// Pour calculer l'indice d'une matrice.
//  Pour les conventions d'indicages, voir la doc (notre rapport).
int I2D(int i, int j, int taille){	return (i*taille  +  j);}

//////////////////////
// Produit Matriciel
//
// direction est une matrice *carree* de taille taille * taille
// A une matrice carree de taille taille * taille
void produitmatriciel(double* A, double* B, int taille, double* direction){

	int kloc, iloc, jloc;
	for(iloc = 0; iloc < taille; iloc ++){
		for(jloc = 0; jloc < taille; jloc ++){
			direction[I2D(iloc ,jloc ,taille)] = 0.0;
			for(kloc = 0; kloc < taille; kloc ++){
				direction[I2D(iloc ,jloc ,taille)]  += (A[I2D(iloc ,kloc ,taille)])*(B[I2D(kloc ,jloc ,taille)]);
			}	}	}
}


////////////////////
// Somme Matricielle
//
//
// direction est une matrice *carree* de taille taille * taille
// A une matrice carree de taille taille * taille
void sommematriciel(double* A, double* B, int taille, double* direction){
	int iloc, jloc;

	for(iloc = 0; iloc < taille; iloc ++){
		for(jloc = 0; jloc < taille; jloc ++){
			direction[I2D(iloc ,jloc ,taille)] = (A[I2D(iloc ,jloc ,taille)]) + (B[I2D(iloc ,jloc ,taille)]);
		}	}
}

/////////////////////////////
// Produit Matrice * Vecteur
//
//
// direction est un vecteur de taille taille
// A une matrice *carree* de taille taille * taille
void produitvecteur(double* A, double* X, int taille, double* direction){
	int iloc, jloc;

	for(iloc = 0; iloc < taille; iloc ++){
		direction[iloc] = 0.0;
		for(jloc = 0; jloc < taille; jloc ++){
			direction[iloc]  += (A[I2D(iloc ,jloc ,taille)])*(X[jloc]);
		}	}
}


/////////////////////////////
// Test I = I_5x5
//
void test_identite_5_5(int debugint, double* matriceI, int it, int i, int j, int l){
	int iloc, jloc;
	int result = 0;

	for(iloc = 0; iloc < 5; iloc ++){
		for(jloc = 0; jloc < 5; jloc ++){
			result  += ((iloc == jloc) == (matriceI[I2D(iloc ,jloc ,5)]));
		if(debugint > 3)	printf("    R×L [%d\t,%d\t] = %f\n", iloc, jloc, (matriceI[I2D(iloc ,jloc ,5)]));
		}
		if(debugint > 3)	printf("\n");
	}
	if (result != 25){
		if(debugint > 3){
			fprintf(stderr, "Le calcul de R × L ne donne pas I_5x5 : i=%d\tj=%d et\tit=%d\n", i, j, l, it);
			fflush(stderr);
		}
	}
	else {if(debugint > 3) printf("R × L donne I_5x5 : i=%d\tj=%d et\tit=%d\n", i, j, l, it);}
}


	/////////////////////////////////////////////////
	// 	FIN	Calcul matriciel
	/////////////////////////////////////////////////

// Petite fonction pour mettre un point d'arrêt dans le programme
double debugm(int debugint, const char* message) {
		if(debugint > 3){
			printf("**************************************************************\n");
			printf("%s ? (-> Enter) ", message);
			getchar();
		}
}

	/////////////////////////////////////////
	//	Prototypes de fonctions

	// deux formules differentes pour calculer le pas de temps
double pas_de_temps3(double* u, double* c, double* v, double* w, int N, int M, int O, double hx, double hy, double hz, int it);

	// Fonction signe mathematique (usuelle)
double sgn(double x);

////////////////////////////////////////
// Fonction norme
double norme(double x, double y){	return sqrt((x*x)  +  (y*y));}

////////////////////////////////////////
// Fonction norme pour trois points
double norme3(double x, double y, double z){	return sqrt((x*x)  +  (y*y)  +  (z*z));}


////////////////////////////////////////
// Fonction carré
double carre(double x){	return (x*x);}


////////////////////////////////////////
// Fonction norme au carré
double norme_carre(double x, double y){	return ((x*x)  +  (y*y));}

////////////////////////////////////////
// Fonction norme au carré pour trois points
double norme_carre3(double x, double y, double z){	return ((x*x)  +  (y*y)  +  (z*z));}

	// Macro pour ecrire nos donnees dans un fichier .vtk
	// définie plus bas
void WriteVtk3D(char *nom, double *f, int N, int M, int O, float hx, float hy, float hz, char* nom_variable, int debugint);

	/////////////////////////////////////////
	//
	//	Fonction main
	//
	/////////////////////////////////////////

int main (int argc, const char * argv[])
{

	if ((argc < NOMBRE_MIN_ARGUMENTS)||(argc > NOMBRE_MAX_ARGUMENTS)) //donne l'usage si on l'appelle sans rentrer le bon nombre d'arguments
	{
		fprintf (stderr, USAGE);	fflush (stderr);
		if (strcmp(argv[1], "-h") == 0) exit(0);
	}
	printf("\nInitialisation\n\tSimulation numerique des equations d'Euler compressible, en 3D, en C ...\n");

	// variables sur le CPU
	///////////////////////
	printf(" Declarations variables en cours ... \n");

/************/
// entiers, flottants scalaires :
// a changer quand on veut
		int	N;	// nombre de points de la discretisation en espace, selon x
		int	M;	// selon y
		int	O;	// selon z
		double	hx;	// pas de temps, selon x
		double	hy;	// selon y
		double	hz;	// selon z

/************/
// vecteurs :
		double	*x;	// points centraux de nos cellules, selon x
		double	*y;	// selon y
		double	*z;	// selon z
		double	*U;	// vecteurs des variables conservatives
		double	*Ubarx;	// valeurs aux interfaces de U, selon x
		double	*Ubary;	// selon y
		double	*Ubarz;	// selon z
/*
TODO : Notations : pour un vecteur A, Abar[x|y|z] represente les valeurs moyennes aux interfaces selon [x|y|z] (de dimension : dim(Abar.n^{->}) = dim(A.n^{->})  +  1)
*/
		double	*Phix;	// flux numeriques, selon x
		double	*Phiy;	// selon y
		double	*Phiz;	// selon y

		double	*F;	// termes F(U) de l'EDP, selon x
		double	*Fc;	// flux centres
		double	*deltaF;// differences des flux
		
		double	*G;	// G(U) selon y
		double	*Gc;
		double	*deltaG;
		
/* /!\ différent de Hbar, entropie /!\  */
		double	*H;	// H(U) selon z
		double	*Hc;
		double	*deltaH;

// FIXME Vecteur source S
		double *Source;

// variables conservatives
		double	*rho;	// masse volumique du fluide
		double	*u;	// vitesse SELON X
		double	*v;	// SELON Y
		double	*w;	// SELON Z
		double	*p;	// pression P
		double	*c;	// vitesse du son c
/* S : FIXME inutile */
		double	*Sx;	// vecteur intermediaire des valeurs propres de A (cf equation) selon x
		double	*Sy;	// selon y
		double	*Sz;	// selon z

		double	*Rx;	// matrice des vecteurs propres de A selon x
		double	*Ry;	// selon y
		double	*Rz;	// selon z

		double	*Lx;	// L^t = R^{-1} matrice des vecteurs propres de A^t, selon x
		double	*Ly;	// selon y
		double	*Lz;	// selon z


// variables de boucles et pour le temps
		int	i;	// variable de boucle
		int	j;	// souvent, k bouge de 0 a 3, j de 0 a N, i de 0 a M, l de 0 a O.
		int	l;
		int	k;

		double	t	= 0.0;	// T temps maximum & t temps courant
		int	it	= 0;	//nb d'iterations
		double	dt;	// pas de temps
		int	borne;		//nb max d'iterations
		int	debug_valeur_negative;	// un compteur si p ou c deviens negatif. 
		double	T	= 1000;	/* TODO : a modifier si besoin*/

		/* Nouvelles variables */
		int	condition_limites_x;	// pour savoir quelle condition limite chosir, selon x
		int	condition_limites_y;	// pour savoir quelle condition limite chosir, selon y
		int	condition_limites_z;	// pour savoir quelle condition limite chosir, selon z
		
		float	pression_speciale = 0.0;	// valeur temporaire pour le calcul des flux aux extrémités

/************/
// suite des vecteurs :
		double	*ubarx;	// valeurs de u aux interfaces selon x
		double	*ubary;	// selon y
		double	*ubarz;	// selon z

		double	*vbarx;	// valeurs de v aux interfaces selon x
		double	*vbary;	// selon y
		double	*vbarz;	// selon z

		double	*wbarx;	// valeurs de w aux interfaces selon x
		double	*wbary;	// selon y
		double	*wbarz;	// selon z

		double	*pbarx;	// valeurs de p aux interfaces selon x
		double	*pbary;	// selon y
		double	*pbarz;	// selon z

		double	*cbarx;	// valeurs de c aux interfaces selon x
		double	*cbary;	// selon y
		double	*cbarz;	// selon z

		double	*Hbarx;	// valeurs de H aux interfaces selon x
		double	*Hbary;	// selon y
		double	*Hbarz;	// selon z

		double	*SSx;	// matrice diagonale des vecteurs propres, selon x
		double	*SSy;	// selon y
		double	*SSz;	// selon y

		double	*resu;	// matrices temporaires pour les produits matriciels
		double	*IR;
		double	*SG;
		double	*PROD;
		double	*df2;	
		double	*testid;

/************/
// Chaines de caracteres
		char*	nom_graphique;	// titre du fichier dans lequel on imprime les resultats

	//Initialisation des variables qui seront fixes :
	/////////////////////////////////////////////////
// FIXME : Selon les plateformes, Sophie ou Maxime compilent avec un IDE, donc sans donner d'arguments
//		et Lilian compile dans un terminal, et peut donner des arguments
	N	= GET_N();	//nb de points du decoupage du segment [0,1] horizontal : argument de notre executable
	M	= GET_M();	//nb de points du decoupage du segment [0,1] vertical : argument de notre executable
	O	= GET_O();	//nb de points du decoupage du segment [0,1] en profondeur : argument de notre executable

	borne = GET_Borne();	//nombre maximum d'iteration : argument de notre executable
	
	// types de conditions limites selon les trois directions
	condition_limites_x	= GET_CONDITION_PERIODIQUES_x;	// FIXME : pour Maxime ou Sophie, mettre ici la constante voulue (0,1,2,3,4) 
	condition_limites_y	= GET_CONDITION_PERIODIQUES_y;
	condition_limites_z	= GET_CONDITION_PERIODIQUES_z;

	//Messages d'erreurs en cas de conditions non respectees sur les arguments de l'executable.
	if (borne <= 0 )
	{
		fprintf (stderr, "ERROR - ./euler_3D_C : La borne doit etre un entier positif.\n"); fflush (stderr); exit (4);
	}
	if (N <= 0 )
	{
		fprintf (stderr, "ERROR - ./euler_3D_C : Le nombre de points <N> selon x doit etre un entier > 0.\n"); fflush (stderr); exit (5);
	}
	if (M <= 0 )
	{
		fprintf (stderr, "ERROR - ./euler_3D_C : Le nombre de points <M> selon y doit etre un entier > 0.\n"); fflush (stderr); exit (6);
	}
	if (O <= 0 )
	{
		fprintf (stderr, "ERROR - ./euler_3D_C : Le nombre de points <O> selon z doit etre un entier > 0.\n"); fflush (stderr); exit (7);
	}

	hx = 1. / N;	// pas de la discretisation, selon x
	hy = 1. / M;	// selon y
	hz = 1. / O;	// selon z

	
	/*********************************/
	debug_valeur_negative = 0;
	void check_negatif3(double a, const char* nom_variable, int i, int j, int l){ 
		debug_valeur_negative  += (a < 0); 
		if(a < 0) {
			fprintf(stderr, "\n/!\\ Une valeur a ete trouvee negative pour la variable %s, valant %f, en i=%d, j=%d et l=%d. C'est l'erreur numero = %d.", nom_variable, a, i, j, l, debug_valeur_negative);
			fflush(stderr);
			if (GET_DEBUG__VOIR_VTK) {debugm(GET_DEBUG__VOIR_VTK, "Ohoh problème !");}
		}
	}
	/*********************************/

	/*********************************/
	char* nom_conditions_initiales(int direction){
		if (direction == 0) return "y";
		if (direction == 1) return "x";
		if (direction == 2) return "z";
		if (direction == 3) return "circ";
	}
	/*********************************/
	/*********************************/
	char* nom_conditions_limites(int direction){
		if (direction == 0) return "periodiques";
		if (direction == 1) return "absorbantes";
		if (direction == 2) return "murPauvre";
		if (direction == 3) return "mur_paspauvre";
		if (direction == 4) return "mur_miroir";
	}
	/*********************************/

	//Ecrire les parametres de la simulation :
	printf ("Nombre de points de la simulation selon x : N = %d;\n", N);
	printf ("\t\tPas de la simulation hx selon x = %f;\n", hx);
	printf ("Nombre de points de la simulation selon y : M = %d;\n", M);
	printf ("\t\tPas de la simulation hy selon y = %f;\n", hy);
	printf ("Nombre de points de la simulation selon z : O = %d;\n", O);
	printf ("\t\tPas de la simulation hz selon z = %f;\n", hz);

	printf ("Nombre d'etapes maximum de la simulation = %d;\n", borne);
	printf ("Orientation des conditions initiales de pression (P) et de masse (rho) : %s;\n\n", nom_conditions_initiales(GET_CONDITION_INITIALES));
	printf ("Coefficient de Courant-Friedrich-Levy : CFL = %f;\n", CFL);
	printf ("Coefficient gamma du gaz modèle : GAMMA = %f.\n\n", GAMMA);
	printf("Type de condition aux limites : %s selon x, et %s selon y, et %s selon z.\n\n", nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));
	
printf("**************************************************************\n");
	printf(" Declarations variables C finie !\n");

    	// allocations de memoires CPU
    	///////////////////////////////
	printf(" Allocation memoire C en cours ... ");

/********************************************
	Il y a plusieurs sortes de tableaux :

taille N, ou M, ou O : valeurs sur chaque segment (x, y, z)

taille N*M*O : valeurs moyennes sur les cellules (u, v, w, p, c, rho, etc)
taille (N + 1)*M*O : valeurs aux interfaces selon x (ubarx, vbarx, wbarx, pbarx, cbarx, etc)
taille N*(M + 1)*O : valeurs aux interfaces selon y (ubary, vbary, wbary, pbary, cbary, etc)
taille N*M*(O + 1) : valeurs aux interfaces selon z (ubarz, vbarz, wbarz, pbarz, cbarz, etc)

taille 5*N*M*O : cinq informations en chaque cellule (U, F, G, H)
taille 5*(N + 1)*M*O : cinq informations en chaque interface selon x (Ubarx, Fc, deltaF, Phix etc)
taille 5*N*(M + 1)*O  : cinq informations en chaque interface selon y (Ubary, Gc, deltaG, Phiy etc)
taille 5*N*M*(O + 1) : cinq informations en chaque interface selon z (Ubarz, Hc, deltaH, Phiz etc)

taille 5*5 : matrice 5x5 temporaire pour le calcul des flux (methode VFFC)
taille 5*5*(N + 1)*M*O : matrices 5x5 en chaque interfaces pour le calcul des flux, selon x (Lx et Rx et SSx)
taille 5*5*N*(M + 1)*O : matrices 5x5 en chaque interfaces pour le calcul des flux, selon y (Ly et Ry et SSy)
taille 5*5*N*M*(O + 1) : matrices 5x5 en chaque interfaces pour le calcul des flux, selon z (Lz et Rz et SSz)

******************************************/

		x	= (double*) malloc(	N	  * sizeof(double));
		y	= (double*) malloc(	M	  * sizeof(double));
		z	= (double*) malloc(	O	  * sizeof(double));

		U	= (double*) malloc(	5*N*M*O	  * sizeof(double));
		Ubarx	= (double*) malloc(	5*(N + 1)*M*O	  * sizeof(double));
		Ubary	= (double*) malloc(	5*N*(M + 1)*O	  * sizeof(double));
		Ubarz	= (double*) malloc(	5*N*M*(O + 1)	  * sizeof(double));

		Phix	= (double*) malloc(	5*(N + 1)*M*O	  * sizeof(double));
		Phiy	= (double*) malloc(	5*N*(M + 1)*O	  * sizeof(double));
		Phiz	= (double*) malloc(	5*N*M*(O + 1)	  * sizeof(double));

// Pour les flux exacts intervenants dans l'équation
// f_{n^{->}}(U) = [ rho . u^{->}×n^{->} ; rho . (u^{->}×n^{->}) . u^{->}  +  p . n^{->} ; (rho . E  +  p) . (u^{->}×n^{->}) ]
// F = f_{ex^{->}}
// G = f_{ey^{->}}
// H = f_{ez^{->}}

		F	= (double*) malloc(	5*N*M*O	  * sizeof(double));
		Fc	= (double*) malloc(	5*(N + 1)*M*O	  * sizeof(double));
		deltaF	= (double*) malloc(	5*(N + 1)*M*O	  * sizeof(double));

		G	= (double*) malloc(	5*N*M*O	  * sizeof(double));
		Gc	= (double*) malloc(	5*N*(M + 1)*O	  * sizeof(double));
		deltaG	= (double*) malloc(	5*N*(M + 1)*O	  * sizeof(double));

		H	= (double*) malloc(	5*N*M*O	  * sizeof(double));
		Hc	= (double*) malloc(	5*N*M*(O + 1)	  * sizeof(double));
		deltaH	= (double*) malloc(	5*N*M*(O + 1)	  * sizeof(double));

		Source	= (double*) malloc(	5*N*M*O	* sizeof(double));

		rho	= (double*) malloc(	N*M*O	  * sizeof(double));
		u	= (double*) malloc(	N*M*O		  * sizeof(double));
		v	= (double*) malloc(	N*M*O		  * sizeof(double));
		w	= (double*) malloc(	N*M*O		  * sizeof(double));
		p	= (double*) malloc(	N*M*O		  * sizeof(double));
		c	= (double*) malloc(	N*M*O		  * sizeof(double));

		Sx	= (double*) malloc(	5*(N + 1)*M*O	  * sizeof(double));
		Sy	= (double*) malloc(	5*N*(M + 1)*O	  * sizeof(double));
		Sz	= (double*) malloc(	5*N*M*(O + 1)	  * sizeof(double));

		Rx	= (double*) malloc(	5*5*(N + 1)*M*O * sizeof(double));
		Ry	= (double*) malloc(	5*5*N*(M + 1)*O * sizeof(double));
		Rz	= (double*) malloc(	5*5*N*M*(O + 1) * sizeof(double));

		Lx	= (double*) malloc(	5*5*(N + 1)*M*O	* sizeof(double));
		Ly	= (double*) malloc(	5*5*N*(M + 1)*O	* sizeof(double));
		Lz	= (double*) malloc(	5*5*N*M*(O + 1)	* sizeof(double));

		ubarx	= (double*) malloc(	(N + 1)*M*O	  * sizeof(double));
		ubary	= (double*) malloc(	N*(M + 1)*O	  * sizeof(double));
		ubarz	= (double*) malloc(	N*M*(O + 1)	  * sizeof(double));

		vbarx	= (double*) malloc(	(N + 1)*M*O	  * sizeof(double));
		vbary	= (double*) malloc(	N*(M + 1)*O	  * sizeof(double));
		vbarz	= (double*) malloc(	N*M*(O + 1)	  * sizeof(double));

		wbarx	= (double*) malloc(	(N + 1)*M*O	  * sizeof(double));
		wbary	= (double*) malloc(	N*(M + 1)*O	  * sizeof(double));
		wbarz	= (double*) malloc(	N*M*(O + 1)	  * sizeof(double));

		pbarx	= (double*) malloc(	(N + 1)*M*O	  * sizeof(double));
		pbary	= (double*) malloc(	N*(M + 1)*O	  * sizeof(double));
		pbarz	= (double*) malloc(	N*M*(O + 1)	  * sizeof(double));

		cbarx	= (double*) malloc(	(N + 1)*M*O	  * sizeof(double));
		cbary	= (double*) malloc(	N*(M + 1)*O	  * sizeof(double));
		cbarz	= (double*) malloc(	N*M*(O + 1)	  * sizeof(double));

		Hbarx	= (double*) malloc(	(N + 1)*M*O	  * sizeof(double));
		Hbary	= (double*) malloc(	N*(M + 1)*O	  * sizeof(double));
		Hbarz	= (double*) malloc(	N*M*(O + 1)	  * sizeof(double));

		SSx	= (double*) malloc(	5*5*(N + 1)*M*O	  * sizeof(double));
		SSy	= (double*) malloc(	5*5*N*(M + 1)*O	  * sizeof(double));
		SSz	= (double*) malloc(	5*5*N*M*(O + 1)	  * sizeof(double));

		resu	= (double*) malloc(	5*5	  * sizeof(double));
		IR	= (double*) malloc(	5*5	  * sizeof(double));
		SG	= (double*) malloc(	5*5	  * sizeof(double));
		PROD	= (double*) malloc(	5	  * sizeof(double));
		df2	= (double*) malloc(	5	  * sizeof(double));

		testid	= (double*) malloc(	5*5	  * sizeof(double)); // TODO : deprecated

// Nom du fichier texte dans lequel on ecrit les donnees numeriques
		nom_graphique	= (char*) malloc(	sizeof(char)*	500);


	printf(" Allocation memoire CPU finie !\n");
	////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////////////////////
   	// Initialisation des tableaux C
	//////////////////////////////////////////////////////////////////////////////////////////

	printf(" Initialisation C en cours ... \n");

		// Points x
		for (j = 0 ; j < N ; j ++)
		{
			x[j]=hx*0.5 + j*hx;
			if(GET_DEBUG__VOIR_VTK > 3) printf("x[%d]=%f\n",j,x[j]);
		}

		// Points y
		for (i = 0 ; i < M ; i ++)
		{
			y[i]=hy*0.5 + i*hy;
			if(GET_DEBUG__VOIR_VTK > 3) printf("y[%d]=%f\n",i,y[i]);
		}

		// Points z
		for (l = 0 ; l < O ; l ++)
		{
			z[l]=hz*0.5 + l*hz;
			if(GET_DEBUG__VOIR_VTK > 3) printf("z[%d]=%f\n",l,z[l]);
		}


printf("**************************************************************\n");
		for (i = 0 ; i < M ; i ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				for (l = 0 ; l < O ; l ++)
				{
		// Initialisation de rho
		//
		// Si rho est trop faible, la simulation ne fonctionne pas (instable et diverge très vite !!)
		//
		// FIXME : why ?
		/////////////////////////
					if (GET_CONDITION_INITIALES == 0){	
						if (y[i]<=0.5)
						{
							rho[i*N*O + j*O + l]=0.125;
						}
						else
						{
							rho[i*N*O + j*O + l]=1.0;
						}
					}
			
					if (GET_CONDITION_INITIALES == 1){	
						if (x[j]<=0.5)
						{
							rho[i*N*O + j*O + l]=0.125;
						}
						else
						{
							rho[i*N*O + j*O + l]=1.0;
						}
					}
			
					if (GET_CONDITION_INITIALES == 2){	
						if (z[l]<=0.5)
						{
							rho[i*N*O + j*O + l]=0.125;
						}
						else
						{
							rho[i*N*O + j*O + l]=1.0;
						}
					}
			
					if (GET_CONDITION_INITIALES == 3){	
/*		 					rho[i*N*O + j*O + l] = 0.125  +  0.875*((norme_carre3(x[j] - 0.25, y[i] - 0.25, z[l] - 0.5))<carre(0.125));	// rhobar faible*/
		 					rho[i*N*O + j*O + l] = 0.125  +  0.875*((norme_carre(x[j] - 0.25, y[i] - 0.25))<carre(0.125));	// rhobar faible
		/*					rho[i*N*O + j*O + l] = 0  +  0.875*((norme_carre(x[j] - 0.75, y[i] - 0.75))<carre(0.25));*/		// rho faible
		/*					rho[i*N*O + j*O + l] = 15  +  9*((norme_carre(x[j] - 0.75, y[i] - 0.75))<carre(0.25));		// rho élevé, rho bar élevé*/
					}
/*					rho[i*N*O + j*O + l] = 1.0 ;*/
						if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,%d,%d]=%f\n",i,j,l,rho[i*N*O + j*O + l]);
		///////////////////////
		// Initialisation de p
					if (GET_CONDITION_INITIALES == 0){	
						if (y[i]<=0.5)
						{
							p[i*N*O + j*O + l]=0.1;
						}
						else
						{
							p[i*N*O + j*O + l]=1.0;
						}
					}
			
					if (GET_CONDITION_INITIALES == 1){	
						if (x[j]<=0.5)
						{
							p[i*N*O + j*O + l]=0.1;
						}
						else
						{
							p[i*N*O + j*O + l]=1.0;
						}
					}
			
					if (GET_CONDITION_INITIALES == 2){	
						if (z[l]<=0.5)
						{
							p[i*N*O + j*O + l]=0.1;
						}
						else
						{
							p[i*N*O + j*O + l]=1.0;
						}
					}
					if (GET_CONDITION_INITIALES == 3){	
/*		 					p[i*N*O + j*O + l] = 0.1  +  0.9*((norme_carre3(x[j] - 0.25, y[i] - 0.25, z[l] - 0.5))<carre(0.125));*/
		 					p[i*N*O + j*O + l] = 0.1  +  0.9*((norme_carre(x[j] - 0.25, y[i] - 0.25))<carre(0.125));
		 			}
/*		 			p[i*N*O + j*O + l] = 1;*/
				if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,%d,%d]=%f\n",i,j,l,p[i*N*O + j*O + l]);
				}
			}
		}

printf("**************************************************************\n");
		for (i = 0 ; i < M ; i ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				for (l = 0 ; l < O ; l ++)
				{
		// Initialisation de u TODO : tester des nouvelles CI
					u[i*N*O + j*O + l]=0.0;	/* FIXME */
				if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,%d,%d]=%f\n",i,j,l,u[i*N*O + j*O + l]);
		// Initialisation de v
					v[i*N*O + j*O + l]=0.0;	/* FIXME */
				if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,%d,%d]=%f\n",i,j,l,v[i*N*O + j*O + l]);
		// Initialisation de w
					w[i*N*O + j*O + l]=0.0;	/* FIXME */
				if(GET_DEBUG__VOIR_VTK > 3) printf("w[%d,%d,%d]=%f\n",i,j,l,w[i*N*O + j*O + l]);
		// Initialisation de c
					c[i*N*O + j*O + l]=sqrt(GAMMA*p[i*N*O + j*O + l]/rho[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,%d,%d]=%f\n",i,j,l,c[i*N*O + j*O + l]);
		// Initialisation de U
		// U = [ rho ; rho u ; rho v ; rho w ; rho E ]
					U[0*N*M*O  +  i*N*O + j*O + l]	= rho[i*N*O + j*O + l];
					U[1*N*M*O  +  i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*u[i*N*O + j*O + l];
					U[2*N*M*O  +  i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*v[i*N*O + j*O + l];
					U[3*N*M*O  +  i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*w[i*N*O + j*O + l];
					U[4*N*M*O  +  i*N*O + j*O + l]	= 0.5*rho[i*N*O + j*O + l]*(norme_carre3( u[i*N*O + j*O + l], v[i*N*O + j*O + l], w[i*N*O + j*O + l])) + p[i*N*O + j*O + l]/(GAMMA-1);
				if(GET_DEBUG__VOIR_VTK > 3) {
					printf("U0[%d\t,%d\t,%d\t]=%f\n", i, j, l, U[i*N*O + j*O + l + 0*N*M*O]);
					printf("U1[%d\t,%d\t,%d\t]=%f\n", i, j, l, U[i*N*O + j*O + l + 1*N*M*O]);
					printf("U2[%d\t,%d\t,%d\t]=%f\n", i, j, l, U[i*N*O + j*O + l + 2*N*M*O]);
					printf("U3[%d\t,%d\t,%d\t]=%f\n", i, j, l, U[i*N*O + j*O + l + 3*N*M*O]);
					printf("U4[%d\t,%d\t,%d\t]=%f\n", i, j, l, U[i*N*O + j*O + l + 4*N*M*O]);
					printf("\n");
					}
				}
			}
		}


	printf(" Initialisation CPU finie \n");
	/////////////////////////////////////////

	// On va sauvegarder les valeurs du pas de temps pour les étudier après
	sprintf(nom_graphique, "Euler3DC_dt_N%d_M%d_O%d_CLD_%d_rho_%s_X%s_Y%s_Z%s.vtk", N, M, O, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));
	FILE *filedt=fopen(nom_graphique, "w");	// On ouvre en ecriture 'w'
	assert(filedt != NULL);
	fprintf(filedt,"# vtk DataFile Version 3.0\n#EULER_3D_C dt\nASCII\nDATASET STRUCTURED_POINTS\n");
	fprintf(filedt,"DIMENSIONS %d %d %d\n",	borne,	1,	1);
	fprintf(filedt,"ORIGIN %f %f %f\n",	0.0,	0.0,	0.0);
	fprintf(filedt,"SPACING %f %f %f\n",	1.0,	1.0,	1.0);
	fprintf(filedt,"POINT_DATA %d\n",		borne);
	fprintf(filedt,"SCALARS dt float\n");
	fprintf(filedt,"LOOKUP_TABLE default\n");
	printf("\033[01;34mOuverture du fichier <%s> pour l'écriture des valeurs de dt.\n\033[01;37m", nom_graphique);

////////////////////////////////////////////////////////////////
	// Ecriture des donnees numeriques dans les fichiers VTK
	//  Pour plus de details sur la syntaxe de tels fichiers, voir les rapports du mois d'avril.

	// On imprime Rho
	sprintf(nom_graphique, "Euler3DC_N%d_M%d_O%d_avecG_CLD_%d_rho_%s_X%s_Y%s_Z%s.vtk", N, M, O, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));
/*	sprintf(nom_graphique, "Ecrire_valeurs_euler_3D_C_avecG_CLD_%d_rho_%s.vtk", it, nom_conditions_initiales(GET_CONDITION_INITIALES));*/
	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [RHO]...\n",0, nom_graphique);
	WriteVtk3D(nom_graphique, rho, N, M, O, hx, hy, hz, "Rho", GET_DEBUG__VOIR_VTK);

	// On imprime P
/*	sprintf(nom_graphique, "Euler3DC_N%d_M%d_O%d_avecG_CLD_%d_p_%s_X%s_Y%s_Z%s.vtk", N, M, O, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));*/
/*	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [P]...\n", 0, nom_graphique);*/
/*	WriteVtk3D(nom_graphique, p, N, M, O, hx, hy, hz, "P", GET_DEBUG__VOIR_VTK);*/

debugm(GET_DEBUG__VOIR_VTK, "Initialisation faite : je me lance");

//////////////////////////////////////////////////////////////////////////////////////////
// DEBUT du corps de la boucle en temps
//////////////////////////////////////////////////////////////////////////////////////////

	// on limite le temps, et le nombre de pas de temps (pour eviter de laisser la simulation s'emballer)
while ((t < T) && (it < borne) )
{

debugm(GET_DEBUG__VOIR_VTK, "Nouvelle boucle");
fflush(stdout);

//////////////////////////////////
	//Calcul du pas de temps
	dt = pas_de_temps3(u, c, v, w, N, M, O, hx, hy, hz, it);

	t	 += dt;		// on actualise la valeur du temps reel (inutilisee)
	it	 ++;		// et le compteur de boucle de temps (UTILE)
	printf("\033[01;31m Nouvelle etape : t = %f dt = %f it = %d.\n \033[01;37m", t, dt, it);
	fprintf(filedt, "%f\n", dt);

//////////////////////////////////////////////////
//
//	SPLITTING SELON X
//
//////////////////////////////////////////////////

////////////////////////////////////////////////////////
	// Actualisation des variables aux interfaces
 
	// Calcul de Ubarx
	for (k = 0 ; k < 5 ; k ++)
	{
		for (i = 0 ; i < M ; i ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				for (j = 1 ; j < N ; j ++)
				{
					Ubarx[i*(N + 1)*O + j*O + l + k*(N + 1)*M*O]=0.5*(U[i*N*O + (j-1)*O + l + k*N*M*O] + U[i*N*O + j*O + l + k*N*M*O]);	// valeur moyenne
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarx%d[%d\t,%d\t,%d\t]=%f\n",k,i,j,l,Ubarx[i*(N + 1)*O + j*O + l + k*(N + 1)*M*O]);
				}
				Ubarx[i*(N + 1)*O + 0*O + l + k*(N + 1)*M*O]=0.5*(U[i*N*O + l + k*N*M*O] + U[i*N*O + (N-1)*O + l + k*N*M*O]);		// interface extreme gauche
				Ubarx[i*(N + 1)*O + N*O + l + k*(N + 1)*M*O]=0.5*(U[i*N*O + l + k*N*M*O] + U[i*N*O + (N-1)*O + l + k*N*M*O]);		// interface extreme droite
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarx%d[%d\t,%d\t,%d\t]=%f\n",k,i,0,l,Ubarx[i*(N + 1)*O + 0*O + l + k*(N + 1)*M*O]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarx%d[%d\t,%d\t,%d\t]=%f\n",k,i,N,l,Ubarx[i*(N + 1)*O + N*O + l + k*(N + 1)*M*O]);
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ubarx");

	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N + 1 ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
	// Calcul de ubarx
				ubarx[i*(N + 1)*O + j*O + l]= Ubarx[1*(N + 1)*M*O + i*(N + 1)*O + j*O + l]/Ubarx[i*(N + 1)*O + j*O + l];
/**/				// u = U[1] / U[0] = rho u / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("ubarx[%d\t,%d\t,%d\t]=%f\n",i,j,l,ubarx[i*(N + 1)*O + j*O + l]);
	// Calcul de vbarx
				vbarx[i*(N + 1)*O + j*O + l]= Ubarx[2*(N + 1)*M*O + i*(N + 1)*O + j*O + l]/Ubarx[i*(N + 1)*O + j*O + l];
/**/				// v = U[2] / U[0] = rho v / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("vbarx[%d\t,%d\t,%d\t]=%f\n",i,j,l,vbarx[i*(N + 1)*O + j*O + l]);
	// Calcul de wbarx
				wbarx[i*(N + 1)*O + j*O + l]= Ubarx[3*(N + 1)*M*O + i*(N + 1)*O + j*O + l]/Ubarx[i*(N + 1)*O + j*O + l];
/**/				// w = U[3] / U[0] = rho w / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("wbarx[%d\t,%d\t,%d\t]=%f\n",i,j,l,wbarx[i*(N + 1)*O + j*O + l]);
	// Calcul de pbarx
				pbarx[i*(N + 1)*O + j*O + l]= (GAMMA-1)*Ubarx[4*(N + 1)*M*O + i*(N + 1)*O + j*O + l]-0.5*(GAMMA-1)*Ubarx[0*(N + 1)*M*O + i*(N + 1)*O + j*O + l]*(norme_carre3( ubarx[i*(N + 1)*O + j*O + l], vbarx[i*(N + 1)*O + j*O + l], wbarx[i*N*O + j*O + l]));
					check_negatif3(pbarx[i*(N + 1)*O + j*O + l], "pbarx", i, j, l);
/**/				// p = (gamma - 1) * rho * e
				if(GET_DEBUG__VOIR_VTK > 3) printf("pbarx[%d\t,%d\t,%d\t]=%f\n",i,j,l,pbarx[i*(N + 1)*O + j*O + l]);
	// Calcul de cbarx
				cbarx[i*(N + 1)*O + j*O + l]= sqrt(GAMMA*(pbarx[i*(N + 1)*O + j*O + l])/Ubarx[0*(N + 1)*M*O + i*(N + 1)*O + j*O + l]);
/**/				// c = racine(gamma * p / rho)
					check_negatif3(cbarx[i*(N + 1)*O + j*O + l], "cbarx", i, j, l);
				if(GET_DEBUG__VOIR_VTK > 3) printf("cbarx[%d\t,%d\t,%d\t]=%f\n",i,j,l,cbarx[i*(N + 1)*O + j*O + l]);
	// Calcul de Hbarx (variable intermediaire pour le calcul des valeurs propres)
				Hbarx[i*(N + 1)*O + j*O + l]= Ubarx[4*(N + 1)*M*O + i*(N + 1)*O + j*O + l]/Ubarx[0*(N + 1)*M*O + i*(N + 1)*O + j*O + l] + pbarx[i*(N + 1)*O + j*O + l]/Ubarx[0*(N + 1)*M*O + i*(N + 1)*O + j*O + l];
/**/				// H = E  +  p / rho = rho E / rho  +  p / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("Hbarx[%d\t,%d\t,%d\t]=%f\n",i,j,l,Hbarx[i*(N + 1)*O + j*O + l]);
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de ubarx vbarx wbarx cbarx pbarx Hbarx");


////////////////////////////////////
	// Calcul des valeurs propres

	// On utilise les valeurs formelles connues
	//  d'apres le document trouves a l'adresse suivante : <http://www.cmla.ens-cachan.fr/fileadmin/Membres/ghidaglia/fluide1.pdf>
	//	et <http://www.cmla.ens-cachan.fr/fileadmin/Membres/ghidaglia/rapport.zip> page 14
	//  de Jean-Michel Ghidaglia et collaborateurs (CMLA).

	double petit_c;	// variables temporaires pour faciliter l'écriture des valeurs !
	double c_carre;

/* Nos matrices 5x5 sont indicées comme cela :
[
 0   1   2   3   4
 5   6   7   8   9
 10  11  12  13  14
 15  16  17  18  19
 20  21  22  23  24
]
********/
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N + 1 ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				////////////////////////////////////
				// Matrice R : valeurs propres de A
				Rx[0 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;
				Rx[1 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;
				Rx[2 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Rx[3 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Rx[4 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;

				Rx[5 +	25*(i*(N + 1)*O + j*O + l) ] = ubarx[i*(N + 1)*O + j*O + l]-cbarx[i*(N + 1)*O + j*O + l];
				Rx[6 +	25*(i*(N + 1)*O + j*O + l) ] = ubarx[i*(N + 1)*O + j*O + l];
				Rx[7 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Rx[8 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Rx[9 +	25*(i*(N + 1)*O + j*O + l) ] = ubarx[i*(N + 1)*O + j*O + l] + cbarx[i*(N + 1)*O + j*O + l];

				Rx[10 + 25*(i*(N + 1)*O + j*O + l) ] = vbarx[i*(N + 1)*O + j*O + l];
				Rx[11 +	25*(i*(N + 1)*O + j*O + l) ] = vbarx[i*(N + 1)*O + j*O + l];
				Rx[12 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;
				Rx[13 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Rx[14 +	25*(i*(N + 1)*O + j*O + l) ] = vbarx[i*(N + 1)*O + j*O + l];

				Rx[15 +	25*(i*(N + 1)*O + j*O + l) ] = wbarx[i*(N + 1)*O + j*O + l];
				Rx[16 +	25*(i*(N + 1)*O + j*O + l) ] = wbarx[i*(N + 1)*O + j*O + l];
				Rx[17 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Rx[18 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;
				Rx[19 +	25*(i*(N + 1)*O + j*O + l) ] = wbarx[i*(N + 1)*O + j*O + l];

				Rx[20 +	25*(i*(N + 1)*O + j*O + l) ] = Hbarx[i*(N + 1)*O + j*O + l]-ubarx[i*(N + 1)*O + j*O + l]*cbarx[i*(N + 1)*O + j*O + l];
				Rx[21 +	25*(i*(N + 1)*O + j*O + l) ] = 0.5*(norme_carre3(ubarx[i*(N + 1)*O + j*O + l], vbarx[i*(N + 1)*O + j*O + l], wbarx[i*(N + 1)*O + j*O + l]));
				Rx[22 +	25*(i*(N + 1)*O + j*O + l) ] = vbarx[i*(N + 1)*O + j*O + l];
				Rx[23 +	25*(i*(N + 1)*O + j*O + l) ] = wbarx[i*(N + 1)*O + j*O + l];
				Rx[24 +	25*(i*(N + 1)*O + j*O + l) ] = Hbarx[i*(N + 1)*O + j*O + l] + ubarx[i*(N + 1)*O + j*O + l]*cbarx[i*(N + 1)*O + j*O + l];

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("Rx%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,Rx[k +	25*(i*(N + 1)*O + j*O + l) ]);
		if ((k % 5) == 4) printf("\n");
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de Rx");

				////////////////////////////////////
				// Matrice L : valeurs propres de A^t
				petit_c = 0.5/(cbarx[i*(N + 1)*O + j*O + l]*cbarx[i*(N + 1)*O + j*O + l]);
				c_carre = cbarx[i*(N + 1)*O + j*O + l]*cbarx[i*(N + 1)*O + j*O + l];

				Lx[	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(c_carre + (GAMMA-1)*(norme_carre3(ubarx[i*(N + 1)*O + j*O + l], vbarx[i*(N + 1)*O + j*O + l], wbarx[i*(N + 1)*O + j*O + l])-Hbarx[i*(N + 1)*O + j*O + l]) + ubarx[i*(N + 1)*O + j*O + l]*cbarx[i*(N + 1)*O + j*O + l]);
				Lx[1 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(-(GAMMA-1)*ubarx[i*(N + 1)*O + j*O + l]-cbarx[i*(N + 1)*O + j*O + l]);
				Lx[2 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(-(GAMMA-1)*vbarx[i*(N + 1)*O + j*O + l]);
				Lx[3 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(-(GAMMA-1)*wbarx[i*(N + 1)*O + j*O + l]);
				Lx[4 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(GAMMA-1);

				Lx[5 +	25*(i*(N + 1)*O + j*O + l) ] = ((GAMMA-1)/c_carre)*(Hbarx[i*(N + 1)*O + j*O + l]-(norme_carre3(ubarx[i*(N + 1)*O + j*O + l], vbarx[i*(N + 1)*O + j*O + l], wbarx[i*(N + 1)*O + j*O + l])));
				Lx[6 +	25*(i*(N + 1)*O + j*O + l) ] = ((GAMMA-1)/c_carre)*ubarx[i*(N + 1)*O + j*O + l];
				Lx[7 +	25*(i*(N + 1)*O + j*O + l) ] = ((GAMMA-1)/c_carre)*vbarx[i*(N + 1)*O + j*O + l];
				Lx[8 +	25*(i*(N + 1)*O + j*O + l) ] = ((GAMMA-1)/c_carre)*wbarx[i*(N + 1)*O + j*O + l];
				Lx[9 +	25*(i*(N + 1)*O + j*O + l) ] = -((GAMMA-1)/c_carre);

				Lx[10 +	25*(i*(N + 1)*O + j*O + l) ] = -vbarx[i*(N + 1)*O + j*O + l];
				Lx[11 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Lx[12 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;
				Lx[13 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Lx[14 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;

				Lx[15 +	25*(i*(N + 1)*O + j*O + l) ] = -wbarx[i*(N + 1)*O + j*O + l];
				Lx[16 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Lx[17 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				Lx[18 +	25*(i*(N + 1)*O + j*O + l) ] = 1.0;
				Lx[19 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;

				Lx[20 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(c_carre + (GAMMA-1)*(norme_carre3(ubarx[i*(N + 1)*O + j*O + l], vbarx[i*(N + 1)*O + j*O + l], wbarx[i*(N + 1)*O + j*O + l])-Hbarx[i*(N + 1)*O + j*O + l])-ubarx[i*(N + 1)*O + j*O + l]*cbarx[i*(N + 1)*O + j*O + l]);
				Lx[21 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(-(GAMMA-1)*ubarx[i*(N + 1)*O + j*O + l] + cbarx[i*(N + 1)*O + j*O + l]);
				Lx[22 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(-(GAMMA-1)*vbarx[i*(N + 1)*O + j*O + l]);
				Lx[23 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(-(GAMMA-1)*wbarx[i*(N + 1)*O + j*O + l]);
				Lx[24 +	25*(i*(N + 1)*O + j*O + l) ] = petit_c*(GAMMA-1);;

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("Lx%d[%d,\t%d,\t%d]=%f\n",k,i,j,l, Lx[k +	25*(i*(N + 1)*O + j*O + l) ]);
		if ((k % 5) == 4) printf("\n");
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de Lx");
	    	// Calculs de R et L : faits
	///////////////////////////////////////////

/*	produitmatriciel(&Rx[(i*(N + 1)*O + j*O+l)*25], &Lx[(i*(N + 1)*O + j*O+l)*25], 5, testid);*/
/*	test_identite_5_5(GET_DEBUG__VOIR_VTK, testid, it, i, j, l);*/
	
	//////////////////////////////////////////////
		// Calcul de la matrice diagonale SS
		//  valeurs propres de A
	
				SSx[		25*(i*(N + 1)*O + j*O + l) ] = sgn(ubarx[i*(N + 1)*O + j*O + l]-cbarx[i*(N + 1)*O + j*O + l]);
				SSx[1  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[2  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[3  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[4  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;

				SSx[5  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[6  +	25*(i*(N + 1)*O + j*O + l) ] = sgn(ubarx[i*(N + 1)*O + j*O + l]);
				SSx[7  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[8  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[9  +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;

				SSx[10 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[11 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[12 +	25*(i*(N + 1)*O + j*O + l) ] = sgn(ubarx[i*(N + 1)*O + j*O + l]);
				SSx[13 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[14 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;

				SSx[15 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[16 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[17 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[18 +	25*(i*(N + 1)*O + j*O + l) ] = sgn(ubarx[i*(N + 1)*O + j*O + l]);
				SSx[19 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;

				SSx[20 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[21 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[22 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[23 +	25*(i*(N + 1)*O + j*O + l) ] = 0.0;
				SSx[24 +	25*(i*(N + 1)*O + j*O + l) ] = sgn(ubarx[i*(N + 1)*O + j*O + l] + cbarx[i*(N + 1)*O + j*O + l]);

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("SSx%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,SSx[k +	25*(i*(N + 1)*O + j*O + l) ]);
		if ((k % 5) == 4) printf("\n");
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de SSx");
			}
		}
	}

//////////////////////////////////////////////
	// Calcul du vecteur des flux F
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				F[0*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*u[i*N*O + j*O + l];
				F[1*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*u[i*N*O + j*O + l]*u[i*N*O + j*O + l] + p[i*N*O + j*O + l];
				F[2*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*u[i*N*O + j*O + l]*v[i*N*O + j*O + l];
				F[3*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*u[i*N*O + j*O + l]*w[i*N*O + j*O + l];
				F[4*N*M*O + i*N*O + j*O + l]	= (U[4*N*M*O + i*N*O + j*O + l] + p[i*N*O + j*O + l])*u[i*N*O + j*O + l];
				if(GET_DEBUG__VOIR_VTK > 3){
					for (k = 0 ; k < 5 ; k ++)
					{
						printf("F%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,F[k*N*M*O + i*N*O + j ]);
					}
				}
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de F");

//////////////////////////////////////////////
	// Calcul du vecteur des flux F centres et Calcul du vecteur des differences de flux F
	for (k = 0 ; k < 5 ; k ++)
	{
		for (i = 0 ; i < M ; i ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				for (j = 1 ; j < N ; j ++)
				{
					Fc[k*(N + 1)*M*O + i*(N + 1)*O + j*O + l]	= 0.5*(F[k*N*M*O + i*N*O + (j-1)*O + l] + F[k*N*M*O + i*N*O + j*O + l]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Fc%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,Fc[k*(N + 1)*M*O + i*(N + 1)*O + j*O + l ]);
/**/					// valeurs moyennes aux interfaces gauche et droite
					deltaF[k*(N + 1)*M*O + i*(N + 1)*O + j*O + l]	= F[k*N*M*O + i*N*O + j*O + l]-F[k*N*M*O + i*N*O + (j-1)*O + l];
/**/					// valeur a droite MOINS valeur a gauche
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaF%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,deltaF[k*(N + 1)*M*O + i*(N + 1)*O + j ]);
				}
/* NEW : on a le choix de nombreuses conditions limites */
			switch (condition_limites_x)
			{
			case 0:		// limites périodiques
			{
				Fc[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[k*N*M*O + i*N + 0*O + l] + F[k*N*M*O + i*N*O + (N-1)*O + l]);	// interface extreme gauche
				Fc[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[k*N*M*O + i*N + 0*O + l] + F[k*N*M*O + i*N*O + (N-1)*O + l]);	// interface extreme droite (FN !)
				deltaF[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[k*N*M*O + i*N + 0*O + l]-F[k*N*M*O + i*N*O + (N-1)*O + l];
				deltaF[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= F[k*N*M*O + i*N + 0*O + l]-F[k*N*M*O + i*N*O + (N-1)*O + l];
				break;
			}
			case 1:		// limites absorbantes
			{
				Fc[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[k*N*M*O + i*N + 0*O + l] + F[k*N*M*O + i*N*O + 0*O + l]);
				Fc[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[k*N*M*O + i*N + (N-1)*O + l] + F[k*N*M*O + i*N*O + (N-1)*O + l]);
				
				deltaF[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[k*N*M*O + i*N + 0*O + l]-F[k*N*M*O + i*N*O + 0*O + l];	// oui ca vaut 0
				deltaF[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= F[k*N*M*O + i*N + (N-1)*O + l]-F[k*N*M*O + i*N*O + (N-1)*O + l];	// oui ca vaut 0
				break;
			}
			case 2:		// limites mur stratégie pauvre
			{
				pression_speciale = p[i*N*O + 0*O + l];
					
					Fc[0*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[0*N*M*O + i*N + 0*O + l] + 0.0);
					Fc[1*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[1*N*M*O + i*N + 0*O + l] + pression_speciale);
					Fc[2*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[2*N*M*O + i*N + 0*O + l] + 0.0);
					Fc[3*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[3*N*M*O + i*N + 0*O + l] + 0.0);
					Fc[4*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[4*N*M*O + i*N + 0*O + l] + 0.0);

					deltaF[0*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[0*N*M*O + i*N + 0*O + l] - 0.0;
					deltaF[1*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[1*N*M*O + i*N + 0*O + l] - pression_speciale;
					deltaF[2*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[2*N*M*O + i*N + 0*O + l] - 0.0;
					deltaF[3*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[3*N*M*O + i*N + 0*O + l] - 0.0;
					deltaF[4*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[4*N*M*O + i*N + 0*O + l] - 0.0;

				pression_speciale = p[i*N*O + (N-1)*O + l];

					Fc[0*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[0*N*M*O + i*N + (N-1)*O + l] + 0.0);
					Fc[1*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[1*N*M*O + i*N + (N-1)*O + l] + pression_speciale);
					Fc[2*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[2*N*M*O + i*N + (N-1)*O + l] + 0.0);
					Fc[3*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[3*N*M*O + i*N + (N-1)*O + l] + 0.0);
					Fc[4*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[4*N*M*O + i*N + (N-1)*O + l] + 0.0);

					deltaF[0*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[0*N*M*O + i*N + (N-1)*O + l] + 0.0;
					deltaF[1*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[1*N*M*O + i*N + (N-1)*O + l] + pression_speciale;
					deltaF[2*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[2*N*M*O + i*N + (N-1)*O + l] + 0.0;
					deltaF[3*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[3*N*M*O + i*N + (N-1)*O + l] + 0.0;
					deltaF[4*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[4*N*M*O + i*N + (N-1)*O + l] + 0.0;

				break;
			}
			case 3:		// limites mur stratégie pas pauvre (papier JM.Ghidaglia F.Pascal, page 58 (10.54))
			{
				pression_speciale = p[i*N*O + 0*O + l]*(1 + (u[i*N*O + 0*O + l]*GAMMA)/(c[i*N*O + 0*O + l] - (GAMMA-1)*u[i*N*O + 0*O + l]));
					
					Fc[0*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[0*N*M*O + i*N + 0*O + l] + 0.0);
					Fc[1*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[1*N*M*O + i*N + 0*O + l] + pression_speciale);
					Fc[2*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[2*N*M*O + i*N + 0*O + l] + 0.0);
					Fc[3*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[3*N*M*O + i*N + 0*O + l] + 0.0);
					Fc[4*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[4*N*M*O + i*N + 0*O + l] + 0.0);

					deltaF[0*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[0*N*M*O + i*N + 0*O + l] - 0.0;
					deltaF[1*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[1*N*M*O + i*N + 0*O + l] - pression_speciale;
					deltaF[2*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[2*N*M*O + i*N + 0*O + l] - 0.0;
					deltaF[3*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[3*N*M*O + i*N + 0*O + l] - 0.0;
					deltaF[4*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[4*N*M*O + i*N + 0*O + l] - 0.0;
				
				pression_speciale = p[i*N*O + (N-1)*O + l]*(1 + (u[i*N*O + (N-1)*O + l]*GAMMA)/(c[i*N*O + (N-1)*O + l] - (GAMMA-1)*u[i*N*O + (N-1)*O + l]));

					Fc[0*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[0*N*M*O + i*N + (N-1)*O + l] + 0.0);
					Fc[1*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[1*N*M*O + i*N + (N-1)*O + l] + pression_speciale);
					Fc[2*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[2*N*M*O + i*N + (N-1)*O + l] + 0.0);
					Fc[3*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[3*N*M*O + i*N + (N-1)*O + l] + 0.0);
					Fc[4*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[4*N*M*O + i*N + (N-1)*O + l] + 0.0);

					deltaF[0*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[0*N*M*O + i*N + (N-1)*O + l] + 0.0;
					deltaF[1*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[1*N*M*O + i*N + (N-1)*O + l] + pression_speciale;
					deltaF[2*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[2*N*M*O + i*N + (N-1)*O + l] + 0.0;
					deltaF[3*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[3*N*M*O + i*N + (N-1)*O + l] + 0.0;
					deltaF[4*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= - F[4*N*M*O + i*N + (N-1)*O + l] + 0.0;
				break;
			}
			case 4:		// limites mur stratégie subtile [FIXME : a faire]
			{
				fprintf(stderr, "Erreur : ces conditions subtiles n'ont pas encore été implémentées !");
				fflush(stderr); exit(1);
				break;
			}
			} 
/*					Fc[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= 0.5*(F[k*N*M*O + i*N + 0*O + l] + F[k*N*M*O + i*N*O + (N-1)*O + l]);	// interface extreme gauche*/
/*					Fc[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= 0.5*(F[k*N*M*O + i*N + 0*O + l] + F[k*N*M*O + i*N*O + (N-1)*O + l]);	// interface extreme droite (FN !)*/
					if(GET_DEBUG__VOIR_VTK > 3) printf("Fc%d[%d,\t%d,\t%d]=%f *\n",k,i,0,l,Fc[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l ]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Fc%d[%d,\t%d,\t%d]=%f *\n",k,i,N,l,Fc[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l ]);
/*					deltaF[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l]	= F[k*N*M*O + i*N + 0*O + l]-F[k*N*M*O + i*N*O + (N-1)*O + l];*/
/*					deltaF[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l]	= F[k*N*M*O + i*N + 0*O + l]-F[k*N*M*O + i*N*O + (N-1)*O + l];*/
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaF%d[%d,\t%d,\t%d]=%f *\n",k,i,0,l,deltaF[k*(N + 1)*M*O + i*(N + 1)*O + 0*O + l ]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaF%d[%d,\t%d,\t%d]=%f *\n",k,i,N,l,deltaF[k*(N + 1)*M*O + i*(N + 1)*O + N*O + l ]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de Fc et deltaF");

//////////////////////////////////////////////
	// Calcul matriciel
/**/	// On doit calculer Phi_i = (F_i + 1  +  F_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (F_i + 1 - F_i)/2
	for (i = 0 ; i < M ; i ++)
	{
		for (l = 0 ; l < O ; l ++)
		{
			for (j = 0 ; j < N + 1 ; j ++)
			{
				df2[0]	= deltaF[0*(N + 1)*M*O + i*(N + 1)*O + j*O + l];
				df2[1]	= deltaF[1*(N + 1)*M*O + i*(N + 1)*O + j*O + l];
				df2[2]	= deltaF[2*(N + 1)*M*O + i*(N + 1)*O + j*O + l];
				df2[3]	= deltaF[3*(N + 1)*M*O + i*(N + 1)*O + j*O + l];
				df2[4]	= deltaF[4*(N + 1)*M*O + i*(N + 1)*O + j*O + l];

				produitmatriciel(&Rx[(i*(N + 1)*O + j*O+l)*25], &SSx[(i*(N + 1)*O + j*O+l)*25], 5, resu);
/**/				// resu = R * diag(sign(lamba_k))
				produitmatriciel(resu, &Lx[(i*(N + 1)*O + j*O+l)*25], 5, SG);
/**/				// SG = R * diag(sign(lamba_k)) * R^{-1}
				produitvecteur(SG, df2, 5, PROD);
/**/				// PROD =  R * diag(sign(lamba_k)) * R^{-1} * (F_{i + 1} - F_{i})/2

				for (k = 0 ; k < 5 ; k ++)
				{
					Phix[k*(N + 1)*M*O + i*(N + 1)*O + j*O + l]	= Fc[k*(N + 1)*M*O + i*(N + 1)*O + j*O + l]-0.5*PROD[k];
				}
			}
		}
	}
	// Fin Calcul matriciel
//////////////////////////////////////////////

////////////////////////////////////////////////////////////////
	// ACTUALISATION du vecteur des variables conservatives
	for (k = 0 ; k < 5 ; k ++)
	{
		for (i = 0 ; i < M ; i ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				for (l = 0 ; l < O ; l ++)
				{
					U[k*N*M*O + i*N*O + j*O + l] -= (dt / hx)*(Phix[k*(N + 1)*M*O + i*(N + 1)*O + (j+1)*O + l]-Phix[k*(N + 1)*M*O + i*(N + 1)*O + j*O + l]);
/**/					// U_j^{n + 1} = U_j^n  +  (dt / h) . Phi_{j + 1}^n - Phi_j^n
					if(GET_DEBUG__VOIR_VTK > 3) printf("U%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,U[k*N*M*O + i*N*O + j*O + l]);
				}
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}	
	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de U n*");
/*printf(" : splitting selon X\n");*/


//////////////////////////////////////////////////////////////////////////////
	// ACTUALISATION des variables conservatives en fonction du nouveau U
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				rho[i*N*O + j*O + l ]	= U[i*N*O + j*O + l];	// rho = U_0
				u[i*N*O + j*O + l ]	= U[1*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// u = U_1 / U_0
				v[i*N*O + j*O + l ]	= U[2*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// v = U_2 / U_0
				w[i*N*O + j*O + l ]	= U[3*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// w = U_3 / U_0
				p[i*N*O + j*O + l ]	= (GAMMA-1)*(U[4*N*M*O + i*N*O + j*O + l]-0.5*U[0*N*M*O + i*N*O + j*O + l]*(norme_carre3(u[i*N*O + j*O + l], v[i*N*O + j*O + l], w[i*N*O + j*O + l])));
					// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
					// car U_4 = rho.E
					// E =  e  +  0.5*||u^{->}||^2
				c[i*N*O + j*O + l ]	= sqrt(GAMMA*p[i*N*O + j*O + l]/rho[i*N*O + j*O + l]);

				if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,\t%d,\t%d]=%f\n",i,j,l,rho[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,\t%d,\t%d]=%f\n",i,j,l,u[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,\t%d,\t%d]=%f\n",i,j,l,v[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("w[%d,\t%d,\t%d]=%f\n",i,j,l,w[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,\t%d,\t%d]=%f\n",i,j,l,p[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,\t%d,\t%d]=%f\n",i,j,l,c[i*N*O + j*O + l]);
				check_negatif3((rho[i*N*O + j*O + l]), "rho n*", i, j, l);
				check_negatif3((p[i*N*O + j*O + l]), "p n*", i, j, l);
				check_negatif3((c[i*N*O + j*O + l]), "c n*", i, j, l);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de rho u v w p c : n*");

//////////////////////////////////////////////////
//
//	SPLITTING SELON Y
//
//////////////////////////////////////////////////

////////////////////////////////////////////////////////
	// Actualisation des variables aux interfaces
 
	// Calcul de Ubary
	for (k = 0 ; k < 5 ; k ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				for (i = 1 ; i < M ; i ++)
				{
					Ubary[i*N*O + j*O + l + k*N*(M + 1)*O]=0.5*(U[(i-1)*N*O + j*O + l + k*N*M*O] + U[i*N*O + j*O + l + k*N*M*O]);	// valeur moyenne
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubary%d[%d\t,%d\t,%d\t]=%f\n",k,i,j,l,Ubary[i*N*O + j*O + l + k*N*M*O]);
				}
				Ubary[0*N*O + j*O + l + k*N*(M + 1)*O]=0.5*(U[0*N*O+ j*O + l + k*N*M*O] + U[(M-1)*N*O+ j*O + l + k*N*M*O]);		// interface extreme gauche
				Ubary[M*N*O + j*O + l + k*N*(M + 1)*O]=0.5*(U[0*N*O+ j*O + l + k*N*M*O] + U[(M-1)*N*O+ j*O + l + k*N*M*O]);		// interface extreme droite
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubary%d[%d\t,%d\t,%d\t]=%f\n",k,0,j,l,Ubary[0*N*O + j*O + l + k*N*(M + 1)*O]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubary%d[%d\t,%d\t,%d\t]=%f\n",k,M,j,l,Ubary[M*N*O + j*O + l + k*N*(M + 1)*O]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ubary");

	for (i = 0 ; i < M + 1 ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
	// Calcul de ubary
				ubary[i*N*O + j*O + l]= Ubary[1*N*(M+1)*O + i*N*O + j*O + l]/Ubary[i*N*O + j*O + l];
/**/				// u = U[1] / U[0] = rho u / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("ubary[%d\t,%d\t,%d\t]=%f\n",i,j,l,ubary[i*N*O + j*O + l]);
	// Calcul de vbary
				vbary[i*N*O + j*O + l]= Ubary[2*N*(M+1)*O + i*N*O + j*O + l]/Ubary[i*N*O + j*O + l];
/**/				// v = U[2] / U[0] = rho v / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("vbary[%d\t,%d\t,%d\t]=%f\n",i,j,l,vbary[i*N*O + j*O + l]);
	// Calcul de wbary
				wbary[i*N*O + j*O + l]= Ubary[3*N*(M+1)*O + i*N*O + j*O + l]/Ubary[i*N*O + j*O + l];
/**/				// w = U[3] / U[0] = rho w / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("wbary[%d\t,%d\t,%d\t]=%f\n",i,j,l,wbary[i*N*O + j*O + l]);
	// Calcul de pbary
				pbary[i*N*O + j*O + l]= (GAMMA-1)*Ubary[4*N*(M + 1)*O + i*N*O + j*O + l]-0.5*(GAMMA-1)*Ubary[0*N*(M + 1)*O + i*N*O + j*O + l]*(norme_carre3( ubary[i*N*O + j*O + l], vbary[i*N*O + j*O + l], wbary[i*N*O + j*O + l]));
				check_negatif3(pbary[i*N*O + j*O + l], "pbary", i, j, l);
/**/				// p = (gamma - 1) * rho * e
				if(GET_DEBUG__VOIR_VTK > 3) printf("pbary[%d\t,%d\t,%d\t]=%f\n",i,j,l,pbary[i*N*O + j*O + l]);
	// Calcul de cbary
				cbary[i*N*O + j*O + l]= sqrt(GAMMA*(pbary[i*N*O + j*O + l])/Ubary[0*N*(M + 1)*O + i*N*O + j*O + l]);
/**/				// c = racine(gamma * p / rho)
				check_negatif3(cbary[i*N*O + j*O + l], "cbary", i, j, l);
				if(GET_DEBUG__VOIR_VTK > 3) printf("cbary[%d\t,%d\t,%d\t]=%f\n",i,j,l,cbary[i*N*O + j*O + l]);
	// Calcul de Hbary (variable intermediaire pour le calcul des valeurs propres)
				Hbary[i*N*O + j*O + l]= Ubary[4*N*(M + 1)*O + i*N*O + j*O + l]/Ubary[0*N*(M + 1)*O + i*N*O + j*O + l] + pbary[i*N*O + j*O + l]/Ubary[0*N*(M + 1)*O + i*N*O + j*O + l];
/**/				// H = E  +  p / rho = rho E / rho  +  p / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("Hbary[%d\t,%d\t,%d\t]=%f\n",i,j,l,Hbary[i*N*O + j*O + l]);
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de ubary vbary wbary cbary pbary Hbary");


////////////////////////////////////
	// Calcul des valeurs propres

	for (i = 0 ; i < M + 1 ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				////////////////////////////////////
				// Matrice R : valeurs propres de A
				Ry[0 +	25*(i*N*O + j*O + l) ] = 1.0;
				Ry[1 +	25*(i*N*O + j*O + l) ] = 1.0;
				Ry[2 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ry[3 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ry[4 +	25*(i*N*O + j*O + l) ] = 1.0;

				Ry[5 +	25*(i*N*O + j*O + l) ] = ubary[i*N*O + j*O + l];
				Ry[6 +	25*(i*N*O + j*O + l) ] = ubary[i*N*O + j*O + l];
				Ry[7 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ry[8 +	25*(i*N*O + j*O + l) ] = 1.0;
				Ry[9 +	25*(i*N*O + j*O + l) ] = ubary[i*N*O + j*O + l];

				Ry[10 + 25*(i*N*O + j*O + l) ] = vbary[i*N*O + j*O + l] - cbary[i*N*O + j*O + l];
				Ry[11 +	25*(i*N*O + j*O + l) ] = vbary[i*N*O + j*O + l];
				Ry[12 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ry[13 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ry[14 +	25*(i*N*O + j*O + l) ] = vbary[i*N*O + j*O + l] + cbary[i*N*O + j*O + l];

				Ry[15 +	25*(i*N*O + j*O + l) ] = wbary[i*N*O + j*O + l];
				Ry[16 +	25*(i*N*O + j*O + l) ] = wbary[i*N*O + j*O + l];
				Ry[17 +	25*(i*N*O + j*O + l) ] = 1.0;
				Ry[18 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ry[19 +	25*(i*N*O + j*O + l) ] = wbary[i*N*O + j*O + l];

				Ry[20 +	25*(i*N*O + j*O + l) ] = Hbary[i*N*O + j*O + l] - vbary[i*N*O + j*O + l]*cbary[i*N*O + j*O + l];
/**/				Ry[21 +	25*(i*N*O + j*O + l) ] = 0.5*(norme_carre3(ubary[i*N*O + j*O + l], vbary[i*N*O + j*O + l], wbary[i*N*O + j*O + l]));
				Ry[22 +	25*(i*N*O + j*O + l) ] = wbary[i*N*O + j*O + l];
				Ry[23 +	25*(i*N*O + j*O + l) ] = ubary[i*N*O + j*O + l];
				Ry[24 +	25*(i*N*O + j*O + l) ] = Hbary[i*N*O + j*O + l] + vbary[i*N*O + j*O + l]*cbary[i*N*O + j*O + l];

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("Ry%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,Ry[k +	25*(i*N*O + j*O + l) ]);
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de Ry");

				////////////////////////////////////
				// Matrice L : valeurs propres de A^t
				petit_c = 0.5/(cbary[i*N*O + j*O + l]*cbary[i*N*O + j*O + l]);
				c_carre = cbary[i*N*O + j*O + l]*cbary[i*N*O + j*O + l];

				Ly[	25*(i*N*O + j*O + l) ] = petit_c*(c_carre + (GAMMA-1)*(norme_carre3(ubary[i*N*O + j*O + l], vbary[i*N*O + j*O + l], wbary[i*N*O + j*O + l])-Hbary[i*N*O + j*O + l]) + vbary[i*N*O + j*O + l]*cbary[i*N*O + j*O + l]);
				Ly[1 +	25*(i*N*O + j*O + l) ] = petit_c*(-(GAMMA-1)*ubary[i*N*O + j*O + l]);
				Ly[2 +	25*(i*N*O + j*O + l) ] = petit_c*(-(GAMMA-1)*vbary[i*N*O + j*O + l] - cbary[i*N*O + j*O + l]);
				Ly[3 +	25*(i*N*O + j*O + l) ] = petit_c*(-(GAMMA-1)*wbary[i*N*O + j*O + l]);
				Ly[4 +	25*(i*N*O + j*O + l) ] = petit_c*(GAMMA-1);

				Ly[5 +	25*(i*N*O + j*O + l) ] = ((GAMMA-1)/c_carre)*(Hbary[i*N*O + j*O + l]-(norme_carre3(ubary[i*N*O + j*O + l], vbary[i*N*O + j*O + l], wbary[i*N*O + j*O + l])));
				Ly[6 +	25*(i*N*O + j*O + l) ] = ((GAMMA-1)/c_carre)*ubary[i*N*O + j*O + l];
				Ly[7 +	25*(i*N*O + j*O + l) ] = ((GAMMA-1)/c_carre)*vbary[i*N*O + j*O + l];
				Ly[8 +	25*(i*N*O + j*O + l) ] = ((GAMMA-1)/c_carre)*wbary[i*N*O + j*O + l];
				Ly[9 +	25*(i*N*O + j*O + l) ] = -((GAMMA-1)/c_carre);

				Ly[10 +	25*(i*N*O + j*O + l) ] = -wbary[i*N*O + j*O + l];
				Ly[11 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ly[12 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ly[13 +	25*(i*N*O + j*O + l) ] = 1.0;
				Ly[14 +	25*(i*N*O + j*O + l) ] = 0.0;

				Ly[15 +	25*(i*N*O + j*O + l) ] = -ubary[i*N*O + j*O + l];
				Ly[16 +	25*(i*N*O + j*O + l) ] = 1.0;
				Ly[17 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ly[18 +	25*(i*N*O + j*O + l) ] = 0.0;
				Ly[19 +	25*(i*N*O + j*O + l) ] = 0.0;

				Ly[20 +	25*(i*N*O + j*O + l) ] = petit_c*(c_carre + (GAMMA-1)*(norme_carre3(ubary[i*N*O + j*O + l], vbary[i*N*O + j*O + l], wbary[i*N*O + j*O + l])-Hbary[i*N*O + j*O + l]) - vbary[i*N*O + j*O + l]*cbary[i*N*O + j*O + l]);
				Ly[21 +	25*(i*N*O + j*O + l) ] = petit_c*(-(GAMMA-1)*ubary[i*N*O + j*O + l]);
				Ly[22 +	25*(i*N*O + j*O + l) ] = petit_c*(-(GAMMA-1)*vbary[i*N*O + j*O + l] + cbary[i*N*O + j*O + l]);
				Ly[23 +	25*(i*N*O + j*O + l) ] = petit_c*(-(GAMMA-1)*wbary[i*N*O + j*O + l]);
				Ly[24 +	25*(i*N*O + j*O + l) ] = petit_c*(GAMMA-1);;

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("Ly%d[%d,\t%d,\t%d]=%f\n",k,i,j,l, Ly[k +	25*(i*N*O + j*O + l) ]);
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de Ly");
	    	// Calculs de R et L : faits
	///////////////////////////////////////////
	
/*produitmatriciel(&Ry[(i*N*O + j*O + l)*25], &Ly[(i*N*O + j*O + l)*25], 5, testid);*/
/*test_identite_5_5(GET_DEBUG__VOIR_VTK, testid, it, i, j, l);*/

	//////////////////////////////////////////////
		// Calcul de la matrice diagonale SS
		//  valeurs propres de A
	
				SSy[		25*(i*N*O + j*O + l) ] = sgn(vbary[i*N*O + j*O + l]-cbary[i*N*O + j*O + l]);
				SSy[1  +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[2  +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[3  +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[4  +	25*(i*N*O + j*O + l) ] = 0.0;

				SSy[5  +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[6  +	25*(i*N*O + j*O + l) ] = sgn(vbary[i*N*O + j*O + l]);
				SSy[7  +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[8  +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[9  +	25*(i*N*O + j*O + l) ] = 0.0;

				SSy[10 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[11 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[12 +	25*(i*N*O + j*O + l) ] = sgn(vbary[i*N*O + j*O + l]);
				SSy[13 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[14 +	25*(i*N*O + j*O + l) ] = 0.0;

				SSy[15 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[16 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[17 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[18 +	25*(i*N*O + j*O + l) ] = sgn(vbary[i*N*O + j*O + l]);
				SSy[19 +	25*(i*N*O + j*O + l) ] = 0.0;

				SSy[20 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[21 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[22 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[23 +	25*(i*N*O + j*O + l) ] = 0.0;
				SSy[24 +	25*(i*N*O + j*O + l) ] = sgn(vbary[i*N*O + j*O + l] + cbary[i*N*O + j*O + l]);

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("SSy%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,SSy[k +	25*(i*N*O + j*O + l) ]);
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de SSy");
			}
		}
	}

//////////////////////////////////////////////
	// Calcul du vecteur des flux G
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				G[0*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*v[i*N*O + j*O + l];
				G[1*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*v[i*N*O + j*O + l]*u[i*N*O + j*O + l];
				G[2*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*v[i*N*O + j*O + l]*v[i*N*O + j*O + l] + p[i*N*O + j*O + l];
				G[3*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*v[i*N*O + j*O + l]*w[i*N*O + j*O + l];
				G[4*N*M*O + i*N*O + j*O + l]	= (U[4*N*M*O + i*N*O + j*O + l] + p[i*N*O + j*O + l])*v[i*N*O + j*O + l];
				if(GET_DEBUG__VOIR_VTK > 3){
					for (k = 0 ; k < 5 ; k ++)
					{
						printf("G%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,G[k*N*M*O + i*N*O + j ]);
					}
				}
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de G");

//////////////////////////////////////////////
	// Calcul du vecteur des flux G centres et Calcul du vecteur des differences de flux G
	for (k = 0 ; k < 5 ; k ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				for (i = 1 ; i < M ; i ++)
				{
					Gc[k*N*(M + 1)*O + i*N*O + j*O + l]	= 0.5*(G[k*N*M*O + (i-1)*N*O + j*O + l] + G[k*N*M*O + i*N*O + j*O + l]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Gc%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,Gc[k*N*(M + 1)*O + i*N*O + j*O + l ]);
/**/					// valeurs moyennes aux interfaces gauche et droite
					deltaG[k*N*(M + 1)*O + i*N*O + j*O + l]	= G[k*N*M*O + i*N*O + j*O + l]-G[k*N*M*O + (i-1)*N*O + j*O + l];
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaG%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,deltaG[k*N*(M + 1)*O + i*N*O + j*O + l ]);
/**/					// valeur a droite MOINS valeur a gauche
				}
/* NEW : on a le choix de nombreuses conditions limites */
			switch (condition_limites_y)
			{
			case 0:		// limites périodiques
			{
				Gc[k*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[k*N*M*O + 0*N*O + j*O + l] + G[k*N*M*O + (M-1)*N*O + j*O + l]);	// interface extreme bas
				Gc[k*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[k*N*M*O + 0*N*O + j*O + l] + G[k*N*M*O + (M-1)*N*O + j*O + l]);	// interface extreme haut (FN !)
				deltaG[k*N*(M + 1)*O + 0*N*O + j*O + l]	= G[k*N*M*O + 0*N*O + j*O + l] - G[k*N*M*O + (M-1)*N*O + j*O + l];
				deltaG[k*N*(M + 1)*O + M*N*O + j*O + l]	= G[k*N*M*O + 0*N*O + j*O + l] - G[k*N*M*O + (M-1)*N*O + j*O + l];
				break;
			}
			case 1:		// limites absorbantes
			{
				Gc[k*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[k*N*M*O + 0*N*O + j*O + l] + G[k*N*M*O + 0*N*O + j*O + l]);	// interface extreme bas
				Gc[k*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[k*N*M*O + (M-1)*N*O + j*O + l] + G[k*N*M*O + (M-1)*N*O + j*O + l]);	// interface extreme haut (FN !)
				deltaG[k*N*(M + 1)*O + 0*N*O + j*O + l]	= G[k*N*M*O + 0*N*O + j*O + l] - G[k*N*M*O + 0*N*O + j*O + l];	// oui ca vaut 0
				deltaG[k*N*(M + 1)*O + M*N*O + j*O + l]	= G[k*N*M*O + (M-1)*N*O + j*O + l] - G[k*N*M*O + (M-1)*N*O + j*O + l];	// oui ca vaut 0
				break;
			}
			case 2:		// limites mur stratégie pauvre
			{
				pression_speciale = p[0*N*O + j*O + l];
					
					Gc[0*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[0*N*M*O + 0*N*O + j*O + l] + 0.0);
					Gc[1*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[1*N*M*O + 0*N*O + j*O + l] + 0.0);
					Gc[2*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[2*N*M*O + 0*N*O + j*O + l] + pression_speciale);
					Gc[3*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[3*N*M*O + 0*N*O + j*O + l] + 0.0);
					Gc[4*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[4*N*M*O + 0*N*O + j*O + l] + 0.0);

					deltaG[0*N*(M + 1)*O + 0*N*O + j*O + l]	= G[0*N*M*O + 0*N*O + j*O + l] - 0.0;
					deltaG[1*N*(M + 1)*O + 0*N*O + j*O + l]	= G[1*N*M*O + 0*N*O + j*O + l] - 0.0;
					deltaG[2*N*(M + 1)*O + 0*N*O + j*O + l]	= G[2*N*M*O + 0*N*O + j*O + l] - pression_speciale;
					deltaG[3*N*(M + 1)*O + 0*N*O + j*O + l]	= G[3*N*M*O + 0*N*O + j*O + l] - 0.0;
					deltaG[4*N*(M + 1)*O + 0*N*O + j*O + l]	= G[4*N*M*O + 0*N*O + j*O + l] - 0.0;

				pression_speciale = p[(M-1)*N*O + j*O + l];

					Gc[0*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[0*N*M*O + (M-1)*N*O + j*O + l] + 0.0);
					Gc[1*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[1*N*M*O + (M-1)*N*O + j*O + l] + 0.0);
					Gc[2*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[2*N*M*O + (M-1)*N*O + j*O + l] + pression_speciale);
					Gc[3*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[3*N*M*O + (M-1)*N*O + j*O + l] + 0.0);
					Gc[4*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[4*N*M*O + (M-1)*N*O + j*O + l] + 0.0);

					deltaG[0*N*(M + 1)*O + M*N*O + j*O + l]	= - G[0*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
					deltaG[1*N*(M + 1)*O + M*N*O + j*O + l]	= - G[1*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
					deltaG[2*N*(M + 1)*O + M*N*O + j*O + l]	= - G[2*N*M*O + (M-1)*N*O + j*O + l] + pression_speciale;
					deltaG[3*N*(M + 1)*O + M*N*O + j*O + l]	= - G[3*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
					deltaG[4*N*(M + 1)*O + M*N*O + j*O + l]	= - G[4*N*M*O + (M-1)*N*O + j*O + l] + 0.0;

				break;
			}
			case 3:		// limites mur stratégie pas pauvre (papier JM.Ghidaglia F.Pascal, page 58 (10.54))
			{
				pression_speciale = p[(M-1)*N*O + j*O + l]*(1 + (v[(M-1)*N*O + j*O + l]*GAMMA)/(c[(M-1)*N*O + j*O + l] - (GAMMA-1)*v[(M-1)*N*O + j*O + l]));
					
					Gc[0*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[0*N*M*O + 0*N*O + j*O + l] + 0.0);
					Gc[1*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[1*N*M*O + 0*N*O + j*O + l] + 0.0);
					Gc[2*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[2*N*M*O + 0*N*O + j*O + l] + pression_speciale);
					Gc[3*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[3*N*M*O + 0*N*O + j*O + l] + 0.0);
					Gc[4*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[4*N*M*O + 0*N*O + j*O + l] + 0.0);

					deltaG[0*N*(M + 1)*O + 0*N*O + j*O + l]	= G[0*N*M*O + 0*N*O + j*O + l] - 0.0;
					deltaG[1*N*(M + 1)*O + 0*N*O + j*O + l]	= G[1*N*M*O + 0*N*O + j*O + l] - 0.0;
					deltaG[2*N*(M + 1)*O + 0*N*O + j*O + l]	= G[2*N*M*O + 0*N*O + j*O + l] - pression_speciale;
					deltaG[3*N*(M + 1)*O + 0*N*O + j*O + l]	= G[3*N*M*O + 0*N*O + j*O + l] - 0.0;
					deltaG[4*N*(M + 1)*O + 0*N*O + j*O + l]	= G[4*N*M*O + 0*N*O + j*O + l] - 0.0;
				
				pression_speciale = p[(M-1)*N*O + j*O + l]*(1 + (v[(M-1)*N*O + j*O + l]*GAMMA)/(c[(M-1)*N*O + j*O + l] - (GAMMA-1)*v[(M-1)*N*O + j*O + l]));

					Gc[0*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[0*N*M*O + (M-1)*N*O + j*O + l] + 0.0);
					Gc[1*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[1*N*M*O + (M-1)*N*O + j*O + l] + 0.0);
					Gc[2*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[2*N*M*O + (M-1)*N*O + j*O + l] + pression_speciale);
					Gc[3*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[3*N*M*O + (M-1)*N*O + j*O + l] + 0.0);
					Gc[4*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[4*N*M*O + (M-1)*N*O + j*O + l] + 0.0);

					deltaG[0*N*(M + 1)*O + M*N*O + j*O + l]	= - G[0*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
					deltaG[1*N*(M + 1)*O + M*N*O + j*O + l]	= - G[1*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
					deltaG[2*N*(M + 1)*O + M*N*O + j*O + l]	= - G[2*N*M*O + (M-1)*N*O + j*O + l] + pression_speciale;
					deltaG[3*N*(M + 1)*O + M*N*O + j*O + l]	= - G[3*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
					deltaG[4*N*(M + 1)*O + M*N*O + j*O + l]	= - G[4*N*M*O + (M-1)*N*O + j*O + l] + 0.0;
				break;
			}
			case 4:		// limites mur stratégie subtile [FIXME : a faire]
			{
				fprintf(stderr, "Erreur : ces conditions subtiles n'ont pas encore été implémentées !");
				fflush(stderr); exit(1);
				break;
			}
			}
/*					Gc[k*N*(M + 1)*O + 0*N*O + j*O + l]	= 0.5*(G[k*N*M*O + 0*N*O + j*O + l] + G[k*N*M*O + (M-1)*N*O + j*O + l]);	// interface extreme gauche*/
/*					Gc[k*N*(M + 1)*O + M*N*O + j*O + l]	= 0.5*(G[k*N*M*O + 0*N*O + j*O + l] + G[k*N*M*O + (M-1)*N*O + j*O + l]);	// interface extreme droite (FN !)*/
					if(GET_DEBUG__VOIR_VTK > 3) printf("Gc%d[%d,\t%d,\t%d]=%f *\n",k,0,j,l,Gc[k*N*(M + 1)*O + i*N*O + 0*O + l ]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Gc%d[%d,\t%d,\t%d]=%f *\n",k,M,j,l,Gc[k*N*(M + 1)*O + i*N*O + N*O + l ]);
/*					deltaG[k*N*(M + 1)*O + 0*N*O + j*O + l]	= G[k*N*M*O + 0*N*O + j*O + l] - G[k*N*M*O + (M-1)*N*O + j*O + l];*/
/*					deltaG[k*N*(M + 1)*O + M*N*O + j*O + l]	= G[k*N*M*O + 0*N*O + j*O + l] - G[k*N*M*O + (M-1)*N*O + j*O + l];*/
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaG%d[%d,\t%d,\t%d]=%f *\n",k,0,j,l,deltaG[k*N*(M + 1)*O + i*N*O + 0*O + l ]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaG%d[%d,\t%d,\t%d]=%f *\n",k,M,j,l,deltaG[k*N*(M + 1)*O + i*N*O + N*O + l ]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de Gc");

//////////////////////////////////////////////
	// Calcul matriciel
/**/	// On doit calculer Phiy_i = (G_i + 1  +  G_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (G_i + 1 - G_i)/2
	for (i = 0 ; i < M + 1 ; i ++)
	{
		for (l = 0 ; l < O ; l ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				df2[0]	= deltaG[0*N*(M + 1)*O + i*N*O + j*O + l];
				df2[1]	= deltaG[1*N*(M + 1)*O + i*N*O + j*O + l];
				df2[2]	= deltaG[2*N*(M + 1)*O + i*N*O + j*O + l];
				df2[3]	= deltaG[3*N*(M + 1)*O + i*N*O + j*O + l];
				df2[4]	= deltaG[4*N*(M + 1)*O + i*N*O + j*O + l];

				produitmatriciel(&Ry[(i*N*O + j*O+l)*25], &SSy[(i*N*O + j*O+l)*25], 5, resu);
/**/				// resu = R * diag(sign(lamba_k))
				produitmatriciel(resu, &Ly[(i*N*O + j*O+l)*25], 5, SG);
/**/				// SG = R * diag(sign(lamba_k)) * R^{-1}
				produitvecteur(SG, df2, 5, PROD);
/**/				// PROD =  R * diag(sign(lamba_k)) * R^{-1} * (F_{i + 1} - F_{i})/2

				for (k = 0 ; k < 5 ; k ++)
				{
					Phiy[k*N*(M + 1)*O + i*N*O + j*O + l]	= Gc[k*N*(M + 1)*O + i*N*O + j*O + l]-0.5*PROD[k];
				}
			}
		}
	}
	// Fin Calcul matriciel
//////////////////////////////////////////////

////////////////////////////////////////////////////////////////
	// ACTUALISATION du vecteur des variables conservatives
	for (k = 0 ; k < 5 ; k ++)
	{
		for (i = 0 ; i < M ; i ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				for (l = 0 ; l < O ; l ++)
				{
					U[k*N*M*O + i*N*O + j*O + l] -= (dt / hy)*(Phiy[k*N*(M + 1)*O + (i+1)*N*O + j*O + l]-Phiy[k*N*(M + 1)*O + i*N*O + j*O + l]);
/**/					// U_j^{n + 1} = U_j^n  +  (dt / h) . Phi_{j + 1}^n - Phi_j^n
					if(GET_DEBUG__VOIR_VTK > 3) printf("U%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,U[k*N*M*O + i*N*O + j*O + l]);
				}
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}	
	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de U n2*");
/*printf(" : splitting selon Y\n");*/


//////////////////////////////////////////////////////////////////////////////
	// ACTUALISATION des variables conservatives en fonction du nouveau U
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				rho[i*N*O + j*O + l ]	= U[i*N*O + j*O + l];	// rho = U_0
				u[i*N*O + j*O + l ]	= U[1*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// u = U_1 / U_0
				v[i*N*O + j*O + l ]	= U[2*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// v = U_2 / U_0
				w[i*N*O + j*O + l ]	= U[3*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// w = U_3 / U_0
				p[i*N*O + j*O + l ]	= (GAMMA-1)*(U[4*N*M*O + i*N*O + j*O + l]-0.5*U[0*N*M*O + i*N*O + j*O + l]*(norme_carre3(u[i*N*O + j*O + l ], v[i*N*O + j*O + l], w[i*N*O + j*O + l])));
					// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
					// car U_4 = rho.E
					// E =  e  +  0.5*||u^{->}||^2
				c[i*N*O + j*O + l ]	= sqrt(GAMMA*p[i*N*O + j*O + l]/rho[i*N*O + j*O + l]);
				
				if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,\t%d,\t%d]=%f\n",i,j,l,rho[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,\t%d,\t%d]=%f\n",i,j,l,u[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,\t%d,\t%d]=%f\n",i,j,l,v[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("w[%d,\t%d,\t%d]=%f\n",i,j,l,w[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,\t%d,\t%d]=%f\n",i,j,l,p[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,\t%d,\t%d]=%f\n",i,j,l,c[i*N*O + j*O + l]);
				check_negatif3((rho[i*N*O + j*O + l]), "rho n2*", i, j, l);
				check_negatif3((p[i*N*O + j*O + l]), "p n2*", i, j, l);
				check_negatif3((c[i*N*O + j*O + l]), "c n2*", i, j, l);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de rho u v w p c : n2*");


//////////////////////////////////////////////////
//
//	SPLITTING SELON Z
//
//////////////////////////////////////////////////

////////////////////////////////////////////////////////
	// Actualisation des variables aux interfaces
 
	// Calcul de Ubarz
	for (k = 0 ; k < 5 ; k ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (i = 0 ; i < M ; i ++)
			{
				for (l = 1 ; l < O ; l ++)
				{
					Ubarz[i*N*(O+1) + j*(O+1) + l + k*N*M*(O+1)]=0.5*(U[i*N*O  + j*O  + (l-1) + k*N*M*O ] + U[i*N*O  + j*O  + l + k*N*M*O ]);	// valeur mozenne
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarz%d[%d\t,%d\t,%d\t]=%f\n",k,i,j,l,Ubarz[i*N*(O+1) + j*(O+1) + l + k*N*M*(O+1)]);
				}
				Ubarz[i*N*(O+1) + j*(O+1) + 0 + k*N*M*(O+1)]=0.5*(U[i*N*O+ j*O  + 0 + k*N*M*O ] + U[i*N*O+ j*O  + (O-1) + k*N*M*O ]);		// interface extreme gauche
				Ubarz[i*N*(O+1) + j*(O+1) + O + k*N*M*(O+1)]=0.5*(U[i*N*O+ j*O  + 0 + k*N*M*O ] + U[i*N*O+ j*O  + (O-1) + k*N*M*O ]);		// interface extreme droite
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarz%d[%d\t,%d\t,%d\t]=%f\n",k,i,j,0,Ubarz[i*N*(O+1) + j*(O+1) + 0 + k*N*M*(O+1)]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Ubarz%d[%d\t,%d\t,%d\t]=%f\n",k,i,j,O,Ubarz[i*N*(O+1) + j*(O+1) + O + k*N*M*(O+1)]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de Ubarz");

	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O + 1 ; l ++)
			{
	// Calcul de ubarz
				ubarz[i*N*(O+1) + j*(O+1) + l]= Ubarz[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]/Ubarz[i*N*(O+1) + j*(O+1) + l];
/**/				// u = U[1] / U[0] = rho u / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("ubarz[%d\t,%d\t,%d\t]=%f\n",i,j,l,ubarz[i*N*(O+1) + j*(O+1) + l]);
	// Calcul de vbarz
				vbarz[i*N*(O+1) + j*(O+1) + l]= Ubarz[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]/Ubarz[i*N*(O+1) + j*(O+1) + l];
/**/				// v = U[2] / U[0] = rho v / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("vbarz[%d\t,%d\t,%d\t]=%f\n",i,j,l,vbarz[i*N*(O+1) + j*(O+1) + l]);
	// Calcul de wbarz
				wbarz[i*N*(O+1) + j*(O+1) + l]= Ubarz[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]/Ubarz[i*N*(O+1) + j*(O+1) + l];
/**/				// w = U[3] / U[0] = rho w / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("wbarz[%d\t,%d\t,%d\t]=%f\n",i,j,l,wbarz[i*N*(O+1) + j*(O+1) + l]);
	// Calcul de pbarz
				pbarz[i*N*(O+1) + j*(O+1) + l]= (GAMMA-1)*Ubarz[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]-0.5*(GAMMA-1)*Ubarz[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]*(norme_carre3( ubarz[i*N*(O+1) + j*(O+1) + l], vbarz[i*N*(O+1) + j*(O+1) + l], wbarz[i*N*(O+1) + j*(O+1) + l]));
				check_negatif3(pbarz[i*N*(O+1) + j*(O+1) + l], "pbarz", i, j, l);
/**/				// p = (gamma - 1) * rho * e
				if(GET_DEBUG__VOIR_VTK > 3) printf("pbarz[%d\t,%d\t,%d\t]=%f\n",i,j,l,pbarz[i*N*(O+1) + j*(O+1) + l]);
	// Calcul de cbarz
				cbarz[i*N*(O+1) + j*(O+1) + l]= sqrt(GAMMA*(pbarz[i*N*(O+1) + j*(O+1) + l])/Ubarz[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]);
/**/				// c = racine(gamma * p / rho)
				check_negatif3(cbarz[i*N*(O+1) + j*(O+1) + l], "cbarz", i, j, l);
				if(GET_DEBUG__VOIR_VTK > 3) printf("cbarz[%d\t,%d\t,%d\t]=%f\n",i,j,l,cbarz[i*N*(O+1) + j*(O+1) + l]);
	// Calcul de Hbarz (variable intermediaire pour le calcul des valeurs propres)
				Hbarz[i*N*(O+1) + j*(O+1) + l]= Ubarz[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]/Ubarz[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l] + pbarz[i*N*(O+1) + j*(O+1) + l]/Ubarz[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l];
		/**/			// H = E  +  p / rho = rho E / rho  +  p / rho
				if(GET_DEBUG__VOIR_VTK > 3) printf("Hbarz[%d\t,%d\t,%d\t]=%f\n",i,j,l,Hbarz[i*N*(O+1) + j*(O+1) + l]);
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de ubarz vbarz wbarz pbarz cbarz Hbarz");


////////////////////////////////////
	// Calcul des valeurs propres

	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O + 1 ; l ++)
			{
				////////////////////////////////////
				// Matrice R : valeurs propres de A
				Rz[0 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;
				Rz[1 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;
				Rz[2 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Rz[3 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Rz[4 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;

				Rz[5 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ubarz[i*N*(O+1) + j*(O+1) + l];
				Rz[6 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ubarz[i*N*(O+1) + j*(O+1) + l];
				Rz[7 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;
				Rz[8 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Rz[9 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ubarz[i*N*(O+1) + j*(O+1) + l];

				Rz[10 + 25*(i*N*(O+1) + j*(O+1) + l) ] = vbarz[i*N*(O+1) + j*(O+1) + l];
				Rz[11 +	25*(i*N*(O+1) + j*(O+1) + l) ] = vbarz[i*N*(O+1) + j*(O+1) + l];
				Rz[12 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Rz[13 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;
				Rz[14 +	25*(i*N*(O+1) + j*(O+1) + l) ] = vbarz[i*N*(O+1) + j*(O+1) + l];

				Rz[15 +	25*(i*N*(O+1) + j*(O+1) + l) ] = wbarz[i*N*(O+1) + j*(O+1) + l] - cbarz[i*N*(O+1) + j*(O+1) + l];
				Rz[16 +	25*(i*N*(O+1) + j*(O+1) + l) ] = wbarz[i*N*(O+1) + j*(O+1) + l];
				Rz[17 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Rz[18 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Rz[19 +	25*(i*N*(O+1) + j*(O+1) + l) ] = wbarz[i*N*(O+1) + j*(O+1) + l] + cbarz[i*N*(O+1) + j*(O+1) + l];

				Rz[20 +	25*(i*N*(O+1) + j*(O+1) + l) ] = Hbarz[i*N*(O+1) + j*(O+1) + l] - wbarz[i*N*(O+1) + j*(O+1) + l]*cbarz[i*N*(O+1) + j*(O+1) + l];
				Rz[21 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.5*(norme_carre3(ubarz[i*N*(O+1) + j*(O+1) + l], vbarz[i*N*(O+1) + j*(O+1) + l], wbarz[i*N*(O+1) + j*(O+1) + l]));
				Rz[22 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ubarz[i*N*(O+1) + j*(O+1) + l];
				Rz[23 +	25*(i*N*(O+1) + j*(O+1) + l) ] = vbarz[i*N*(O+1) + j*(O+1) + l];
				Rz[24 +	25*(i*N*(O+1) + j*(O+1) + l) ] = Hbarz[i*N*(O+1) + j*(O+1) + l] + wbarz[i*N*(O+1) + j*(O+1) + l]*cbarz[i*N*(O+1) + j*(O+1) + l];

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("Rz%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,Rz[k +	25*(i*N*(O+1) + j*(O+1) + l) ]);
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de Rz");

				////////////////////////////////////
				// Matrice L : valeurs propres de A^t
				petit_c = 0.5/(cbarz[i*N*(O+1) + j*(O+1) + l]*cbarz[i*N*(O+1) + j*(O+1) + l]);
				c_carre = cbarz[i*N*(O+1) + j*(O+1) + l]*cbarz[i*N*(O+1) + j*(O+1) + l];

				Lz[	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(c_carre + (GAMMA-1)*(norme_carre3(ubarz[i*N*(O+1) + j*(O+1) + l], vbarz[i*N*(O+1) + j*(O+1) + l], wbarz[i*N*(O+1) + j*(O+1) + l])-Hbarz[i*N*(O+1) + j*(O+1) + l]) + wbarz[i*N*(O+1) + j*(O+1) + l]*cbarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[1 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(-(GAMMA-1)*ubarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[2 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(-(GAMMA-1)*vbarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[3 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(-(GAMMA-1)*wbarz[i*N*(O+1) + j*(O+1) + l] - cbarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[4 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(GAMMA-1);

				Lz[5 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ((GAMMA-1)/c_carre)*(Hbarz[i*N*(O+1) + j*(O+1) + l]-(norme_carre3(ubarz[i*N*(O+1) + j*(O+1) + l], vbarz[i*N*(O+1) + j*(O+1) + l], wbarz[i*N*(O+1) + j*(O+1) + l])));
				Lz[6 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ((GAMMA-1)/c_carre)*ubarz[i*N*(O+1) + j*(O+1) + l];
				Lz[7 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ((GAMMA-1)/c_carre)*vbarz[i*N*(O+1) + j*(O+1) + l];
				Lz[8 +	25*(i*N*(O+1) + j*(O+1) + l) ] = ((GAMMA-1)/c_carre)*wbarz[i*N*(O+1) + j*(O+1) + l];
				Lz[9 +	25*(i*N*(O+1) + j*(O+1) + l) ] = -((GAMMA-1)/c_carre);

				Lz[10 +	25*(i*N*(O+1) + j*(O+1) + l) ] = -ubarz[i*N*(O+1) + j*(O+1) + l];
				Lz[11 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;
				Lz[12 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Lz[13 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Lz[14 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;

				Lz[15 +	25*(i*N*(O+1) + j*(O+1) + l) ] = -vbarz[i*N*(O+1) + j*(O+1) + l];
				Lz[16 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Lz[17 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 1.0;
				Lz[18 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				Lz[19 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;

				Lz[20 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(c_carre + (GAMMA-1)*(norme_carre3(ubarz[i*N*(O+1) + j*(O+1) + l], vbarz[i*N*(O+1) + j*(O+1) + l], wbarz[i*N*(O+1) + j*(O+1) + l])-Hbarz[i*N*(O+1) + j*(O+1) + l]) - wbarz[i*N*(O+1) + j*(O+1) + l]*cbarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[21 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(-(GAMMA-1)*ubarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[22 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(-(GAMMA-1)*vbarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[23 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(-(GAMMA-1)*wbarz[i*N*(O+1) + j*(O+1) + l] + cbarz[i*N*(O+1) + j*(O+1) + l]);
				Lz[24 +	25*(i*N*(O+1) + j*(O+1) + l) ] = petit_c*(GAMMA-1);;

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("Lz%d[%d,\t%d,\t%d]=%f\n",k,i,j,l, Lz[k +	25*(i*N*(O+1) + j*(O+1) + l) ]);
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de Lz");
	    	// Calculs de R et L : faits
	///////////////////////////////////////////
	
/*produitmatriciel(&Rz[(i*N*(O+1) + j*(O+1) + l)*25], &Lz[(i*N*(O+1) + j*(O+1) + l)*25], 5, testid);*/
/*test_identite_5_5(GET_DEBUG__VOIR_VTK, testid, it, i, j, l);*/

	//////////////////////////////////////////////
		// Calcul de la matrice diagonale SS
		//  valeurs propres de A
	
				SSz[		25*(i*N*(O+1) + j*(O+1) + l) ] = sgn(wbarz[i*N*(O+1) + j*(O+1) + l]-cbarz[i*N*(O+1) + j*(O+1) + l]);
				SSz[1  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[2  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[3  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[4  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;

				SSz[5  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[6  +	25*(i*N*(O+1) + j*(O+1) + l) ] = sgn(wbarz[i*N*(O+1) + j*(O+1) + l]);
				SSz[7  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[8  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[9  +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;

				SSz[10 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[11 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[12 +	25*(i*N*(O+1) + j*(O+1) + l) ] = sgn(wbarz[i*N*(O+1) + j*(O+1) + l]);
				SSz[13 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[14 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;

				SSz[15 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[16 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[17 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[18 +	25*(i*N*(O+1) + j*(O+1) + l) ] = sgn(wbarz[i*N*(O+1) + j*(O+1) + l]);
				SSz[19 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;

				SSz[20 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[21 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[22 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[23 +	25*(i*N*(O+1) + j*(O+1) + l) ] = 0.0;
				SSz[24 +	25*(i*N*(O+1) + j*(O+1) + l) ] = sgn(wbarz[i*N*(O+1) + j*(O+1) + l] + cbarz[i*N*(O+1) + j*(O+1) + l]);

if(GET_DEBUG__VOIR_VTK > 20) {
	for (k = 0 ; k < 25 ; k ++)
	{
		printf("SSz%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,SSz[k +	25*(i*N*(O+1) + j*(O+1) + l) ]);
	}
}
debugm(GET_DEBUG__VOIR_VTK-20, "Calcul de SSz");
			}
		}
	}

//////////////////////////////////////////////
	// Calcul du vecteur des flux H
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				H[0*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*w[i*N*O + j*O + l];
				H[1*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*w[i*N*O + j*O + l]*u[i*N*O + j*O + l];
				H[2*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*w[i*N*O + j*O + l]*v[i*N*O + j*O + l];
				H[3*N*M*O + i*N*O + j*O + l]	= rho[i*N*O + j*O + l]*w[i*N*O + j*O + l]*w[i*N*O + j*O + l] + p[i*N*O + j*O + l];
				H[4*N*M*O + i*N*O + j*O + l]	= (U[4*N*M*O + i*N*O + j*O + l] + p[i*N*O + j*O + l])*w[i*N*O + j*O + l];
				if(GET_DEBUG__VOIR_VTK > 3){
					for (k = 0 ; k < 5 ; k ++)
					{
						printf("H%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,H[k*N*M*O + i*N*O + j ]);
					}
				}
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
debugm(GET_DEBUG__VOIR_VTK, "Calcul de H");

//////////////////////////////////////////////
	// Calcul du vecteur des flux H centres et Calcul du vecteur des differences de flux H
	for (k = 0 ; k < 5 ; k ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (i = 0 ; i < M ; i ++)
			{
				for (l = 1 ; l < O ; l ++)
				{
					Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]	= 0.5*(H[k*N*M*O + i*N*O + j*O + (l-1)] + H[k*N*M*O + i*N*O + j*O + l]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Hc%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,Hc[k*N*M*(O+1) + i*N*(O+1) + j*O + l ]);
/**/					// valeurs mozennes aux interfaces gauche et droite
					deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]	= H[k*N*M*O + i*N*O + j*O + l] - H[k*N*M*O + i*N*O + j*O + (l-1)];
/**/					// valeur a droite MOINS valeur a gauche
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaH%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,deltaH[k*N*M*(O+1) + i*N*(O+1) + j*O + l ]);
				}
/* NEW : on a le choix de nombreuses conditions limites */
			switch (condition_limites_z)
			{
			case 0:		// limites périodiques
			{
				Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[k*N*M*O + i*N*O + j*O + 0] + H[k*N*M*O + i*N*O + j*O + (O - 1)]);	// interface extreme arriere
				Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[k*N*M*O + i*N*O + j*O + 0] + H[k*N*M*O + i*N*O + j*O + (O - 1)]);	// interface extreme devant
				deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[k*N*M*O + i*N*O + j*O + 0] - H[k*N*M*O + i*N*O + j*O + (O-1)];
				deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= H[k*N*M*O + i*N*O + j*O + 0] - H[k*N*M*O + i*N*O + j*O + (O-1)];
				break;
			}
			case 1:		// limites absorbantes
			{
				Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[k*N*M*O + i*N*O + j*O + 0] + H[k*N*M*O + i*N*O + j*O + 0]);	// interface extreme arriere
				Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[k*N*M*O + i*N*O + j*O + (O - 1)] + H[k*N*M*O + i*N*O + j*O + (O - 1)]);	// interface extreme devant
				deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[k*N*M*O + i*N*O + j*O + 0] - H[k*N*M*O + i*N*O + j*O + 0];	// oui ca vaut 0
				deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= H[k*N*M*O + i*N*O + j*O + (O-1)] - H[k*N*M*O + i*N*O + j*O + (O-1)];	// oui ca vaut 0
				break;
			}
			case 2:		// limites mur stratégie pauvre
			{
				pression_speciale = p[i*N*O + j*O + 0];
					
					Hc[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[0*N*M*O + i*N*O + j*O + 0] + 0.0);
					Hc[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[1*N*M*O + i*N*O + j*O + 0] + 0.0);
					Hc[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[2*N*M*O + i*N*O + j*O + 0] + 0.0);
					Hc[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[3*N*M*O + i*N*O + j*O + 0] + pression_speciale);
					Hc[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[4*N*M*O + i*N*O + j*O + 0] + 0.0);

					deltaH[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[0*N*M*O + i*N*O + j*O + 0] - 0.0;
					deltaH[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[1*N*M*O + i*N*O + j*O + 0] - 0.0;
					deltaH[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[2*N*M*O + i*N*O + j*O + 0] - 0.0;
					deltaH[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[3*N*M*O + i*N*O + j*O + 0] - pression_speciale;
					deltaH[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[4*N*M*O + i*N*O + j*O + 0] - 0.0;

				pression_speciale = p[i*N*O + j*O + (O-1)];

					Hc[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[0*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);
					Hc[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[1*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);
					Hc[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[2*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);
					Hc[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[3*N*M*O + i*N*O + j*O + (O - 1)] + pression_speciale);
					Hc[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[4*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);

					deltaH[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[0*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
					deltaH[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[1*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
					deltaH[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[2*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
					deltaH[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[3*N*M*O + i*N*O + j*O + (O - 1)] + pression_speciale;
					deltaH[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[4*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;

				break;
			}
			case 3:		// limites mur stratégie pas pauvre (papier JM.Ghidaglia F.Pascal, page 58 (10.54))
			{
				pression_speciale = p[i*N*O + j*O + 0]*(1 + (w[i*N*O + j*O + 0]*GAMMA)/(c[i*N*O + j*O + 0] - (GAMMA-1)*w[i*N*O + j*O + 0]));
					
					Hc[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[0*N*M*O + i*N*O + j*O + 0] + 0.0);
					Hc[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[1*N*M*O + i*N*O + j*O + 0] + 0.0);
					Hc[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[2*N*M*O + i*N*O + j*O + 0] + 0.0);
					Hc[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[3*N*M*O + i*N*O + j*O + 0] + pression_speciale);
					Hc[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[4*N*M*O + i*N*O + j*O + 0] + 0.0);

					deltaH[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[0*N*M*O + i*N*O + j*O + 0] - 0.0;
					deltaH[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[1*N*M*O + i*N*O + j*O + 0] - 0.0;
					deltaH[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[2*N*M*O + i*N*O + j*O + 0] - 0.0;
					deltaH[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[3*N*M*O + i*N*O + j*O + 0] - pression_speciale;
					deltaH[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[4*N*M*O + i*N*O + j*O + 0] - 0.0;
				
				pression_speciale = p[i*N*O + j*O + (O-1)]*(1 + (w[i*N*O + j*O + (O-1)]*GAMMA)/(c[i*N*O + j*O + (O-1)] - (GAMMA-1)*w[i*N*O + j*O + (O-1)]));

					Hc[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[0*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);
					Hc[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[1*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);
					Hc[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[2*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);
					Hc[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[3*N*M*O + i*N*O + j*O + (O - 1)] + pression_speciale);
					Hc[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[4*N*M*O + i*N*O + j*O + (O - 1)] + 0.0);

					deltaH[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[0*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
					deltaH[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[1*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
					deltaH[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[2*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
					deltaH[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[3*N*M*O + i*N*O + j*O + (O - 1)] + pression_speciale;
					deltaH[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= - H[4*N*M*O + i*N*O + j*O + (O - 1)] + 0.0;
				break;
			}
			case 4:		// limites mur stratégie subtile [FIXME : a faire]
			{
				fprintf(stderr, "Erreur : ces conditions subtiles n'ont pas encore été implémentées !");
				fflush(stderr); exit(1);
				break;
			}
			}
/*					Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= 0.5*(H[k*N*M*O + i*N*O + j*O + 0] + H[k*N*M*O + i*N*O + j*O + (O - 1)]);	// interface extreme arriere*/
/*					Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= 0.5*(H[k*N*M*O + i*N*O + j*O + 0] + H[k*N*M*O + i*N*O + j*O + (O - 1)]);	// interface extreme devant*/
					if(GET_DEBUG__VOIR_VTK > 3) printf("Hc%d[%d,\t%d,\t%d]=%f *\n",k,i,j,0,Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("Hc%d[%d,\t%d,\t%d]=%f *\n",k,i,j,O,Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]);
/*					deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]	= H[k*N*M*O + i*N*O + j*O + 0] - H[k*N*M*O + i*N*O + j*O + (O-1)];*/
/*					deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]	= H[k*N*M*O + i*N*O + j*O + 0] - H[k*N*M*O + i*N*O + j*O + (O-1)];*/
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaH%d[%d,\t%d,\t%d]=%f *\n",k,i,j,0,deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + 0]);
					if(GET_DEBUG__VOIR_VTK > 3) printf("deltaH%d[%d,\t%d,\t%d]=%f *\n",k,i,j,O,deltaH[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + O]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de Hc");

//////////////////////////////////////////////
	// Calcul matriciel
/**/	// On doit calculer Phiz_i = (H_i + 1  +  H_i)/2  -  R * diag(sign(lamba_k)) * R^{-1} * (H_i + 1 - H_i)/2
	for (i = 0 ; i < M ; i ++)
	{
		for (l = 0 ; l < O + 1 ; l ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				df2[0]	= deltaH[0*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l];
				df2[1]	= deltaH[1*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l];
				df2[2]	= deltaH[2*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l];
				df2[3]	= deltaH[3*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l];
				df2[4]	= deltaH[4*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l];

				produitmatriciel(&Rz[(i*N*(O+1) + j*(O+1)+l)*25], &SSz[(i*N*(O+1) + j*(O+1)+l)*25], 5, resu);
/**/				// resu = R * diag(sign(lamba_k))
				produitmatriciel(resu, &Lz[(i*N*(O+1) + j*(O+1)+l)*25], 5, SG);
/**/				// SG = R * diag(sign(lamba_k)) * R^{-1}
				produitvecteur(SG, df2, 5, PROD);
/**/				// PROD =  R * diag(sign(lamba_k)) * R^{-1} * (F_{i + 1} - F_{i})/2

				for (k = 0 ; k < 5 ; k ++)
				{
					Phiz[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]	= Hc[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]-0.5*PROD[k];
				}
			}
		}
	}
	// Fin Calcul matriciel
//////////////////////////////////////////////

	// calcul du terme Source
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				Source[0*N*M*O + i*N*O + j*O + l] = 0.0;
				Source[1*N*M*O + i*N*O + j*O + l] = 0.0;
				Source[2*N*M*O + i*N*O + j*O + l] = 0.0;
				Source[3*N*M*O + i*N*O + j*O + l] = coefficient_g * U[0*N*M*O + i*N*O + j*O + l];	// g.rho et rho=U[0]
				Source[4*N*M*O + i*N*O + j*O + l] = coefficient_g * U[3*N*M*O + i*N*O + j*O + l];	// g.w et w=U[3]
			}
		}
	}

////////////////////////////////////////////////////////////////
	// ACTUALISATION du vecteur des variables conservatives
	for (k = 0 ; k < 5 ; k ++)
	{
		for (i = 0 ; i < M ; i ++)
		{
			for (j = 0 ; j < N ; j ++)
			{
				for (l = 0 ; l < O ; l ++)
				{
					U[k*N*M*O + i*N*O + j*O + l] -= (dt / hz)*(Phiz[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + (l+1)]-Phiz[k*N*M*(O+1) + i*N*(O+1) + j*(O+1) + l]);
/**/					// U_j^{n + 1} = U_j^n  +  (dt / h) . Phi_{j + 1}^n - Phi_j^n

		/////////////////////////////////////////////////////////////////////////////////////////////////
/* FIXME : on rajoute le terme source ICI : sans que cela soit justifié mathématiquement, on pense qu'on peut le mettre ou on veut */
if (SOURCE_PRESENTE != 0){		U[k*N*M*O + i*N*O + j*O + l] -= dt * (Source[k*N*M*O + i*N*O + j*O + l]); }
		/////////////////////////////////////////////////////////////////////////////////////////////////
					if(GET_DEBUG__VOIR_VTK > 3) printf("U%d[%d,\t%d,\t%d]=%f\n",k,i,j,l,U[k*N*M*O + i*N*O + j*O + l]);
				}
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}	
	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de U n2*");
/*printf(" : splitting selon Z\n");*/


//////////////////////////////////////////////////////////////////////////////
	// ACTUALISATION des variables conservatives en fonction du nouveau U
	for (i = 0 ; i < M ; i ++)
	{
		for (j = 0 ; j < N ; j ++)
		{
			for (l = 0 ; l < O ; l ++)
			{
				rho[i*N*O + j*O + l ]	= U[i*N*O + j*O + l];	// rho = U_0
				u[i*N*O + j*O + l ]	= U[1*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// u = U_1 / U_0
				v[i*N*O + j*O + l ]	= U[2*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// v = U_2 / U_0
				w[i*N*O + j*O + l ]	= U[3*N*M*O + i*N*O + j*O + l]/U[0*N*M*O + i*N*O + j*O + l];	// w = U_3 / U_0
				p[i*N*O + j*O + l ]	= (GAMMA-1)*(U[4*N*M*O + i*N*O + j*O + l]-0.5*U[0*N*M*O + i*N*O + j*O + l]*(norme_carre3(u[i*N*O + j*O + l], v[i*N*O + j*O + l], w[i*N*O + j*O + l])));
					// p = (gamma - 1) * ( U_4 - rho * ||u^{->}||^2 )
					// car U_4 = rho.E
					// E =  e  +  0.5*||u^{->}||^2
				c[i*N*O + j*O + l ]	= sqrt(GAMMA*p[i*N*O + j*O + l]/rho[i*N*O + j*O + l]);
				
				if(GET_DEBUG__VOIR_VTK > 3) printf("rho[%d,\t%d,\t%d]=%f\n",i,j,l,rho[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("u[%d,\t%d,\t%d]=%f\n",i,j,l,u[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("v[%d,\t%d,\t%d]=%f\n",i,j,l,v[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("w[%d,\t%d,\t%d]=%f\n",i,j,l,w[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("p[%d,\t%d,\t%d]=%f\n",i,j,l,p[i*N*O + j*O + l]);
				if(GET_DEBUG__VOIR_VTK > 3) printf("c[%d,\t%d,\t%d]=%f\n",i,j,l,c[i*N*O + j*O + l]);
				check_negatif3((rho[i*N*O + j*O + l]), "rho n + 1", i, j, l);
				check_negatif3((p[i*N*O + j*O + l]), "p n + 1", i, j, l);
				check_negatif3((c[i*N*O + j*O + l]), "c n + 1", i, j, l);
				if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
			}
			if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
		}
		if(GET_DEBUG__VOIR_VTK > 3) printf("\n");
	}
	
debugm(GET_DEBUG__VOIR_VTK, "Calcul de rho u v w p c : n + 1");


/////////////////////////////////////////////////////////////////
	// Ecriture des donnees numeriques dans les fichiers VTK
	//  Pour plus de details sur la syntaxe de tels fichiers, voir les rapport du mois d'avril.

	// On imprime Rho
	sprintf(nom_graphique, "Euler3DC_N%d_M%d_O%d_avecG_CLD_%d_rho_%s_X%s_Y%s_Z%s.vtk", N, M, O, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));
	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [RHO]...\n", it, nom_graphique);
	WriteVtk3D(nom_graphique, rho, N, M, O, hx, hy, hz, "Rho", GET_DEBUG__VOIR_VTK);

	// On imprime P
/*	sprintf(nom_graphique, "Euler3DC_N%d_M%d_O%d_avecG_CLD_%d_p_%s_X%s_Y%s_Z%s.vtk", N, M, O, it, nom_conditions_initiales(GET_CONDITION_INITIALES), nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));*/
/*	printf("Etape %d :\ten cours ...\tecriture dans le fichier externe <%s> en cours [P]...\n", it, nom_graphique);*/
/*	WriteVtk3D(nom_graphique, p, N, M, O, hx, hy, hz, "P", GET_DEBUG__VOIR_VTK);*/

}
	
	printf("\033[01;34m");
	//Ecrire les parametres de la simulation :
	//Ecrire les parametres de la simulation :
	printf ("\nNombre de points de la simulation selon x : N = \033[01;32m%d\033[01;37m\n", N);
	printf ("\t\tPas de la simulation selon x : hx = \033[01;32m%f\033[01;37m\n", hx);
	printf ("Nombre de points de la simulation selon y : M = \033[01;32m%d\033[01;37m\n", M);
	printf ("\t\tPas de la simulation selon y : hy = \033[01;32m%f\033[01;37m\n", hy);
	printf ("Nombre de points de la simulation selon z : O = \033[01;32m%d\033[01;37m\n", O);
	printf ("\t\tPas de la simulation selon z : hz = \033[01;32m%f\033[01;37m\n", hz);

	printf ("Nombre d'etapes maximum de la simulation : borne = \033[01;32m%d\033[01;37m\n", borne);
	printf ("Orientation des conditions initiales de pression (P) et de masse (rho) : %s.\n\n", nom_conditions_initiales(GET_CONDITION_INITIALES));
	printf ("Indice de débug debugint = \033[01;32m%d\033[01;37m.\n", GET_DEBUG__VOIR_VTK);

	printf ("Coefficient de Courant-Friedrich-Levy : CFL = \033[01;32m%.2f\033[01;37m;\n", CFL);
	printf ("Coefficient gamma du gaz modèle : GAMMA = \033[01;32m%.2f\033[01;37m.\n\n", GAMMA);
	printf("Type de condition aux limites : \033[01;32m%s\033[01;37m selon x, et \033[01;32m%s\033[01;37m selon y, et \033[01;32m%s\033[01;37m selon z.\n\n", nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));

/*	printf ("Nombre de points de la simulation selon x : N = %d;\n", N);*/
/*	printf ("\t\tPas de la simulation hx selon x = %f;\n", hx);*/
/*	printf ("Nombre de points de la simulation selon y : M = %d;\n", M);*/
/*	printf ("\t\tPas de la simulation hy selon y = %f;\n", hy);*/
/*	printf ("Nombre de points de la simulation selon z : O = %d;\n", O);*/
/*	printf ("\t\tPas de la simulation hz selon z = %f;\n", hz);*/

/*	printf ("Nombre d'etapes maximum de la simulation = %d;\n", borne);*/
/*	printf ("Orientation des conditions initiales de pression (P) et de masse (rho) : %s;\n\n", nom_conditions_initiales(GET_CONDITION_INITIALES));*/
/*	printf ("Coefficient de Courant-Friedrich-Levy : CFL = %f;\n", CFL);*/
/*	printf ("Coefficient gamma du gaz modèle : GAMMA = %f.\n\n", GAMMA);*/
/*	printf("Type de condition aux limites : %s selon x, et %s selon y, et %s selon z.\n\n", nom_conditions_limites(condition_limites_x), nom_conditions_limites(condition_limites_y), nom_conditions_limites(condition_limites_z));*/

//////////////////////////////////////////////////////////////////////////////////////////
// FIN de la boucle en temps
//////////////////////////////////////////////////////////////////////////////////////////
	printf("\nDONE : euler_3D_C : simulation terminee.\nHecth & Isnard & Besson (c) et (r) 2012\n");
	printf("  Les resultats ont ete ecrits dans deux listes de fichiers VTK :\n\
			Ecrire_valeurs_euler_3D_C_avecG_CLD_*_p_[x|y|circ]_[X*Y*Z*].vtk  \t : pour la pression\n\
			Ecrire_valeurs_euler_3D_C_avecG_CLD_*_rho_[x|y|circ]_[X*Y*Z*].vtk\t : pour la masse volumique\n\
		Ils peuvent etre visualises via le logiciel Paraview \n\
			recuperable a l'adresse suivante : [http://www.paraview.org/paraview/resources/software.php] \n\
		 ou	[sudo apt-get install paraview]	sous Linux \n\
		\n");

	printf("\033[01;37m");
	/////////////////////////
	// truc de frimeur
	// On libere la memoire allouee sur le CPU
	//
	free(x); free(y); free(z);
	free(U); free(Ubarx); free(Ubary); free(Ubarz); free(Phix); free(Phiy); free(Phiz);
	free(F); free(Fc); free(deltaF); 
	free(G); free(Gc); free(deltaG); 
	free(H); free(Hc); free(deltaH); 

	free(u); free(v); free(w); free(p); free(c); free(rho);
	free(Rx); free(Lx); free(SSx);
	free(Ry); free(Ly); free(SSy);
	free(Rz); free(Lz); free(SSz);
	free(ubarx); free(vbarx); free(wbarx); free(pbarx); free(cbarx); free(Hbarx);
	free(ubary); free(vbary); free(wbary); free(pbary); free(cbary); free(Hbary);
	free(ubarz); free(vbarz); free(wbarz); free(pbarz); free(cbarz); free(Hbarz);
	free(resu); free(IR); free(SG); free(PROD); free(df2);

	free(nom_graphique);
	fclose(filedt);
    	//////////////////////////////////
    	
	return 0;
}
	/////////////////////////////////////////////////////
	//
	//	Fin de la fonction Main
	//
	//////////////////////////////////////////////////////



	////////////////////////////////////////////////////////////////
	//	Fonctions auxiliaires CPU

////////////////////////////////////////
// Fonction signe
double sgn(double x)
{
	if (x<0){	return -1.0;}
	else if (x>0){	return 1.0;}
	else{		return 0.0;}
}

//////////////////////////////////////////////////////////
// Pour ecrire notre matrice rho ou P dans un fichier VTK
// TODO : nom_variable : est limitee a trois caracteres /!\ /!\ TODO
void WriteVtk3D(char *nom, double *f, int N, int M, int O, float hx, float hy, float hz, char* nom_variable, int debugint)
{
	int i,j,l;
	char* nom_destination;
	
	nom_destination = (char*) malloc(800 * sizeof(char));
	
	sprintf(nom_destination, "%s", nom);
if (debugint != 2){
	FILE *file=fopen(nom_destination, "w");	// On ouvre en ecriture 'w'
	assert(file != NULL);
	fprintf(file,"# vtk DataFile Version 3.0\n# Valeurs Euler_3D_POINTS\nASCII\nDATASET STRUCTURED_POINTS\n");
	fprintf(file,"DIMENSIONS %d %d %d\n",	N,	M,	O);
	fprintf(file,"ORIGIN %f %f %f\n",	0.5*hx,	0.5*hy,	0.5*hz);
	fprintf(file,"SPACING %f %f %f\n",	hx,	hy,	hz);
	fprintf(file,"POINT_DATA %d\n",		N*M*O);
	fprintf(file,"SCALARS %s float\n", 	nom_variable);
	fprintf(file,"LOOKUP_TABLE default\n");
	
	if (debugint){
		printf("* ECRIRE RESULTATS : \n* Variable %s.\tFichier %s.\tN=%d\tM=%d\tO=%d\thx=%f\thy=%f\thz=%f\n", nom_variable, nom_destination, N, M, O, hx, hy, hz);
		printf("*********\n");
	}
	for (j=0; j<N; j ++){
		for (i=0; i<M; i ++){
			for (l=0; l<O; l ++){
				if (debugint) { printf("%s\t[%d\t,%d\t,%d\t] = %f\n",nom_variable, i, j, l, f[i*N*O + j*O + l]); }
				fprintf(file, "%f\n",f[i*N*O + j*O + l]);
			}
		}
	}
	if (debugint){
		debugm(debugint, "Valeurs ecrites : je continue");
	}
	fflush(file);
	fclose(file);	// TODO : Tres important !!
			// Le nombre de stream ouvert est limite, il faut donc les fermer des qu'on a plus besoin d'ecrire.
			// /!\ Cause de nombreuses 'seg fault' inexplicables !
	free(nom_destination);
}
}


///////////////////////////////////////
// Actualise le pas de temps.
// Selon la formule dt = CFL * min(hx , hy, hz) * MAX ( |W|  +  c)
//	W = (u, v, w)
//	N * M * O est la taille de u,c,v,w,
double pas_de_temps3(double* u, double* c, double* v, double* w, int N, int M, int O, double hx, double hy, double hz, int it){
	double maxcalc, pasdetemps;
	int indice;
	
	maxcalc = 0.0;
	pasdetemps = 0.00001;
	
	for (indice=0; indice<N*M*O; indice ++) {
		if (maxcalc < ((norme3(u[indice], v[indice], w[indice]))  +  c[indice])) {
			maxcalc = ((norme3(u[indice], v[indice], w[indice]))  +  c[indice]);
		}
	}
	if (maxcalc != 0.0){pasdetemps = (CFL * (min(hx,min(hy,hz)))) / maxcalc;}	// on se protège.
	else{
		fprintf(stderr, "Pas de temps foireux ! it=%d\n", it);
		fflush(stderr);
		exit(2);
	}
	return (pasdetemps);
}

////////////////////////////////////////////////////////////////
//	Maxime Isnard & Sophie Hecht & Lilian Besson
//		 ENS de Cachan & LSV & CMLA - Juin 2012
////////////////////////////////////////////////////////////////
