% vffc_m6.m
%
N = 400;
h = 1.0 / N;
x = h/2 : h : 1-h/2;
%
% Declaration des tableaux
%
U      = zeros(4,N); % vecteur des variables conservative
Ubar   = zeros(4,N+1); % etat moyen d'interface
Phi    = zeros(4,N+1); % flux numerique
F      = zeros(4,N); % vecteur de flux
Fc     = zeros(4,N+1); % flux centre
deltaF = zeros(4,N+1); % difference de flux
rho    = zeros(1,N); % masse volumique
u      = zeros(1,N); % vitesse x
v      = zeros(1,N); % vitesse "y" (devrait etre =0)
p      = zeros(1,N); % pression
c      = zeros(1,N); % vitesse du son
S      = zeros(4,N+1); % matrice signe (diagonale)
R      = zeros(4,4,N+1); % matrice de passage (vps a droite)
%
% Initialisation
%
rho = 1.0 * (x<= 0.5) + 0.125 * (x>0.5);
u   = u*0;
v   = v*0;
p   = 1.0 * (x<= 0.5) + 0.1 * (x>0.5);
c   = sqrt(1.4*p./rho);
U(1,:) = rho;
U(2,:) = rho.*u;
U(3,:) = rho.*v;
U(4,:) = 0.5*rho.*(u.^2+v.^2) + p / 0.4;
%
% Preparation de R ...
R(1,1,:) = 1;
R(1,2,:) = 1;
R(1,4,:) = 1;
%
% Boucle en temps
t = 0; it=0;
while(t<2)
   Ubar = 0.5*(U(:,[1:N,1]) + U(:,[N,1:N]));
   ubar = Ubar(2,:)./Ubar(1,:);
   vbar = Ubar(3,:)./Ubar(1,:);
   pbar = 0.4*(Ubar(4,:) - 0.5*Ubar(1,:).*(ubar.^2+vbar.^2));
   cbar = sqrt(1.4*pbar./Ubar(1,:));
   Hbar = Ubar(4,:)./Ubar(1,:) + pbar./Ubar(1,:);
   %
   % Calcul du pas de temps
   %
   dt = 0.9 * h / max(abs(ubar)+cbar);
   t  = t + dt; it=it+1;
   %
   R(2,1,:) = ubar - cbar;
   R(2,2,:) = ubar;
   R(2,4,:) = ubar + cbar;
   R(3,1,:) = vbar ;
   R(3,2,:) = vbar;
   R(3,3,:) = -1;
   R(3,4,:) = vbar ;
   R(4,1,:) = Hbar - cbar.*ubar;
   R(4,2,:) = 0.5*(ubar.^2+vbar.^2);
   R(4,3,:) = -vbar;
   R(4,4,:) = Hbar + cbar.*ubar;
   %
   S = sign([ubar - cbar; ubar; ubar; ubar + cbar]);
   %
   F(1,:) = U(2,:);
   F(2,:) = rho.*(u.^2) + p;
   F(3,:) = U(2,:).*v;
   F(4,:) = (U(4,:)+p).*u;
   %
   Fc     = 0.5*(F(:,[N,1:N])+F(:,[1:N,1]));
   deltaF = F(:,[1:N,1]) - F(:,[N,1:N]);   
   %
   for j=1:N+1,
     w        = R(:,:,j) \ deltaF(:,j);   
     Phi(:,j) = Fc(:,j) - 0.5 * R(:,:,j) * diag(S(:,j)) * w;
   end;
   %
   % Update schema
   %
   U = U - (dt/h) * (Phi(:,2:N+1) - Phi(:,1:N));
   %
   % Calcul des variables primitives
   %
   rho = U(1,:);
   u   = U(2,:) ./ U(1,:);
   v   = U(3,:) ./ U(1,:);
   p   = 0.4*(U(4,:) - 0.5*U(1,:).*(u.^2+v.^2));
   c   = sqrt(1.4*p./U(1,:));
   %
   % Plot
   %
   if ~mod(it,10)
     clf(); 
     subplot(1,4,1), plot(x, rho, '.-');
     subplot(1,4,2), plot(x, u, '.-');
     subplot(1,4,3), plot(x, v, '.-');
     subplot(1,4,4), plot(x, p, '.-');
     drawnow();
   end;
end;